/*****************************************************************

  Copyright (c) 1999 - 2012 by Patrick Amar - pa@lri.fr

    http://www.lri.fr/~pa

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#if ZLOG
#include <zlib.h>
#endif

#ifndef bcopy
#include <string.h>
#define index(s,c) strchr((s),(c))
#define rindex(s,c) strrchr((s),(c))
#define bcopy(s,d,n) memcpy((d),(s),(n))
#define bcmp(a,b,n) memcmp((a),(b),(n))
#define bzero(a,n) memset((a),0,(n))
#endif

#include "Service.h"
#include "Enseignement.h"
#include "Enseignant.h"
#include "Commands.h"
#include "display.h"

#define	MSERVLEN	512			// taille max d'un service.
#define	MFILNB		512			// nombre max de filires.
#define SMALLBUFLEN	512			// taille d'un "petit" buffer.

extern char*	_outp_;			// pointeur du buffer de sortie.
extern char*	Host;			// import aussi de server.cc

static void affiche_total_filieres (char** av, char* user_passwd, FILE* sock);

static int	ignore_ret = 0;		// faire taire g++  propos des valeurs de retour ignores.

static char	superuser [] = ADMIN_PASSWD;
static char	mandator [] = MAND_PASSWD;
static char	filadmin [] = FIL_PASSWD;
static bool	permanents_seulements = true;	// seuls les titulaires peuvent s'inscrire.

static char* admin_title = (char*)"Administration";

#define SUPERUSER (!strcmp (user_passwd, superuser))
#define MANDATOR (!strcmp (user_passwd, mandator))
#define FILADMIN (!strcmp (user_passwd, filadmin))

bool	is_superuser = false;

static char	version_sig [] = "Voeux_1.3\n";

static char	fmt_service [] = "<script>top.location='service_%s.cmd'</script>\n";
static char	cancel_get_passwd [] = "You must supply an identification";

// Enttes pour toutes les pages.

static char	fmt_head [] =
	"<html><head>\n"
	"<link rel=\"stylesheet\" type=\"text/css\" href=\"style-voeux.css\" title=\"voeux\">\n"
	"<link rel=\"stylesheet\" type=\"text/css\" href=\"style-print.css\" title=\"print\">\n"
	"<title>(%d-%d) %s</title><link REL='SHORTCUT ICON' HREF=\"voeux.ico\">\n"
	"</head>\n"
	"<body background=fond.jpg bgcolor=#000000 text=#ffffff link=\"%s\" vlink=\"%s\">\n"
	;

/*
 *	Donne la date, l'heure et la provenance de la requte en clair.
 */

static char* timestamp (void)
{
	time_t		now = time (0);
	struct tm*	t;
	static char	tc [256];

	t = gmtime (&now);
	sprintf (tc, "%02d/%02d - %02d:%02d (%30.30s)",
			t -> tm_mday, t -> tm_mon+1,
			(t -> tm_mon < 3 || t -> tm_mon > 9)
				? t -> tm_hour+1
				: t -> tm_hour+2,
			t -> tm_min,
			Host);
	return tc;
}

/*
 *	Affichent l'enrobage html.
 */

// header & footer.

static char fmt_other_years_end [] = "</a>";
static char fmt_current_beg [] = "<font color=pink><b>";
static char fmt_current_end [] = "</b></font>";

static char fmt_header_footer [] =
	"<table width=\"100%%\"><tr>\n"
	"<td align=left><a href=index.html><img alt=accueil src=accueil.png border=0></a></td>\n"
	"<td align=left><a href=index_enseignements.cmd><img alt=enseignements src=enseignements.png border=0></a></td>\n"
	"<td align=left><a href=index_enseignants.cmd><img alt=enseignants src=enseignants.png border=0></a></td>\n"
	"<td align=left><a href=aide.html><img alt=aide src=aide.png border=0></a></td>\n"
	"<td align=right valign=center>\n"
		"<table>\n"
		"<tr><td>%s%d-%d%s&nbsp;&nbsp;</td></tr>\n"
		"<tr><td>%s%d-%d%s&nbsp;&nbsp;</td></tr>\n"
		"<tr><td>%s%d-%d%s&nbsp;&nbsp;</td></tr>\n"
		"</table>\n"
	"</td>\n"
	"</tr></table>\n";

inline static void show_years ()
{
	char	milieu [128];
	char	haut [128];
	char	bas [128];
	char	fmt_other_years_beg [128];

	sprintf (fmt_other_years_beg, "<a href=\"%s\">", Taux::_url_format);
	switch (Taux::_position) {
		default:
		case 0:		// anne courante affiche en bas, non cliquable.
			sprintf (milieu, fmt_other_years_beg, Taux::_year1-1, Taux::_year1);
			sprintf (haut, fmt_other_years_beg, Taux::_year1-2, Taux::_year1-1);
			_outp_ += sprintf (_outp_, fmt_header_footer,
				haut, Taux::_year1-2, Taux::_year1-1, fmt_other_years_end,
				milieu, Taux::_year1-1, Taux::_year1, fmt_other_years_end,
				fmt_current_beg, Taux::_year1, Taux::_year2, fmt_current_end);
			return;

		case 1:		// anne courante affiche au milieu, non cliquable.
			sprintf (bas, fmt_other_years_beg, Taux::_year2, Taux::_year2+1);
			sprintf (haut, fmt_other_years_beg, Taux::_year1-1, Taux::_year1);
			_outp_ += sprintf (_outp_, fmt_header_footer,
				haut, Taux::_year1-1, Taux::_year1, fmt_other_years_end,
				fmt_current_beg, Taux::_year1, Taux::_year2, fmt_current_end,
				bas, Taux::_year2, Taux::_year2+1, fmt_other_years_end);
			return;

		case 2:		// anne courante affiche en haut, non cliquable.
			sprintf (milieu, fmt_other_years_beg, Taux::_year2, Taux::_year2+1);
			sprintf (bas, fmt_other_years_beg, Taux::_year2+1, Taux::_year2+2);
			_outp_ += sprintf (_outp_, fmt_header_footer,
				fmt_current_beg, Taux::_year1, Taux::_year2, fmt_current_end,
				milieu, Taux::_year2, Taux::_year2+1, fmt_other_years_end,
				bas, Taux::_year2+1, Taux::_year2+2, fmt_other_years_end);
			return;
	}
}

static void begin_html (const char* title)
{
	_outp_ += sprintf (_outp_, fmt_head, Taux::_year1, Taux::_year2, title,
					   page_colors [Lien], page_colors [LienVisite]);
	show_years ();
}

static void end_html (void)
{
	time_t		now = time (0);
	struct tm*	lt = localtime (&now);	// date de fabrication de la page.

	_outp_ += sprintf (_outp_,
				"<br><table width=100%%>"
				"<tr><td align=right><b>Date: %02d-%02d-%02d @ %02d:%02d</b>"
				"</td></tr></table>\n",
				lt -> tm_mday, lt -> tm_mon+1,1900 + lt -> tm_year,
				lt -> tm_hour, lt -> tm_min);

	show_years ();
	_outp_ += sprintf (_outp_, "</body></html>\n");
}

/*
 *	Retourne l'identification de l'utilisateur.
 */

static bool get_ident (FILE* sock, char* user_passwd, Enseignant* prof)
{
	if (user_passwd [0] == 0) {
		// pas encore identifi.
		get_user_passwd (prof -> _nom, sock);
		return false;
	}
	if (SUPERUSER) {
		return true;					// ok, le superuser a tous les droits.
	}

	if (! prof -> pw_compare (user_passwd)) {
		get_user_passwd (prof -> _nom, sock);
		return false;
	}
	zlog ("%s (%s)\n", prof -> _nom, Host);
	return true;
}

#if WITH_CHG_PASSWD

/*
 *	Change de mot de passe.
 */

static void change_passwd (char* nom_ens, char* user_passwd, FILE* sock)
{
	static char	fmt_chg_passwd [] =
		"<form method=post action=chg_passwd>"
		"<input type=hidden name=e value=\"%s\">"
		"<p>Une fois votre nouveau mot de passe enregistr, il vous sera redemand pour accder \n"
		"votre page<p>"
		"<table>\n"
		"<tr>\n"
		"	<td align=right><b>Nouveau mot de passe:</b></td>\n"
		"	<td align=left><input type=text name=pwd1 size=30></td>\n"
		"</tr>\n"
		"<tr>\n"
		"	<td align=right><b>Une autre fois:</b></td>\n"
		"	<td align=left><input type=text name=pwd2 size=30></td>\n"
		"</tr>\n"
		"</table><p>\n"
		"<input type=\"submit\" value=\"Valider\">\n"
		"</form>\n";

	Enseignant*	prof = Enseignant::name2prof (nom_ens);
	char		tc [256];

	if (prof == 0) {
		_outp_ += sprintf (_outp_, "<h3>Enseignant inconnu: %s\n", nom_ens);
		return;
	}
	if (!SUPERUSER && !get_ident (sock, user_passwd, prof)) {
		return;
	}
	begin_html ("Nouveau mot de passe");
	sprintf (tc, "Nouveau mot de passe pour: <font color=cyan>%s</font>", prof -> _nom);
	TNG_title (tc);
	_outp_ += sprintf (_outp_, fmt_chg_passwd, prof -> nospaces ());
	end_html ();
}

/*
 *	Retour du formulaire de modification de mot de passe.
 */

static void chg_passwd (char** av, char* user_passwd, FILE* sock)
{
	Enseignant*	prof = Enseignant::name2prof (av [0] + 2);
	char*		pwd1;
	char*		pwd2;

	if (prof == 0) {
		_outp_ += sprintf (_outp_, "<h3>Enseignant inconnu: %s\n", av [0]+2);
		return;
	}

	pwd1 = av [1]+5;
	pwd2 = av [2]+5;

	if (strcmp (pwd1, pwd2) != 0) {
		_outp_ += sprintf (_outp_, "<script>alert('mot de passe mal retape');\ntop.history.back();</script>\n");
		return;
	}
	if (strcmp (pwd1, STD_PASSWD) == 0) {
		_outp_ += sprintf (_outp_, "<script>alert('Mot de passe standard interdit');\ntop.history.back();</script>\n");
		return;
	}
	// OK mettre le nouveau mot de passe.
	strcpy (prof -> _passwd, pwd1);
	_outp_ += sprintf (_outp_, "<script>alert('Nouveau mot de passe enregistre');\ntop.history.back();</script>\n");
	zlog ("chg_passwd: %s\n\n", prof -> _nom);
	zlog ("old = '%s' new1 = '%s' new2 = '%s'\n", prof -> _passwd, pwd1, pwd2);
}
#endif

/*
 *	Montre la page de l'administrateur.
 */

static void admin (char* user_passwd, FILE* sock)
{
	int			v = 1;
	const char*	label = "Edition";
	static const char	fmt_admin_edit [] =
		"<form method=post action=edit_mode>"
		"<input type=hidden name=e value=\"%d\">"
		"Pour passer en mode <b><input type=button onClick='submit()' value=\"%s\"></b> cliquer sur le bouton."
		"</form>\n";

	static const char	fmt_acces_tous [] =
		"<form method=post action=inscription_restreinte>"
		"<input type=hidden name=t value=0>"
		"Autoriser <b><input type=button onClick='submit()' value=tous></b> les enseignants  s'inscrire (mme s'ils sont extrieurs)."
		"</form>\n";
	static const char	fmt_acces_restreint [] =
		"<form method=post action=inscription_restreinte>"
		"<input type=hidden name=t value=1>"
		"N'autoriser que les <b><input type=button onClick='submit()' value=Permanents></b>  s'inscrire."
		"</form>\n";

	static const char	fmt_tous [] =
		"<form method=post action=tous_enseignants>"
		"<input type=hidden name=t value=1>"
		"Afficher <b><input type=button onClick='submit()' value=tous></b> les enseignants (mme si leur service est vide)."
		"</form>\n";
	static const char	fmt_non_vide [] =
		"<form method=post action=tous_enseignants>"
		"<input type=hidden name=t value=0>"
		"N'afficher <b><input type=button onClick='submit()' value=que></b> les enseignants dont le service est non vide."
		"</form>\n";
	static const char	fmt_dump_server [] =
		"<hr><form method=post action=dump>"
		"Sauvegarder les services <b><input type=button onclick='submit()' value=Dump></b>"
		"</form>\n";
	static const char	fmt_mandator [] =
		"<form method=post action=mandator>"
		"Mandater des heures complmentaires <b><input type=button onclick='submit()' value=Mandater></b>"
		"</form>\n";
	static const char	fmt_total_filiere [] =
		"<form method=post action=total_filiere>"
		"Afficher le <b><input type=button onclick='submit()' value=Rcapitulatif></b> des heures par filire."
		"</form>\n";
	static const char	fmt_admin_stop [] =
		"<hr><form method=post action=stop>"
		"Arrter le serveur <b><input type=button onClick='submit()' value=Stop></b>"
		"</form>\n";
	static const char	fmt_admin_restart [] =
		"<hr><form method=post action=restart>"
		"Relancer le serveur <b><input type=button onClick='submit()' value=Restart></b>"
		"</form>\n";
	/*
	 *	Identifier le client.
	 */
	if (! SUPERUSER) {
		get_user_passwd (admin_title, sock);
		// le client refera la requte avec l'authentification,
		return;
	}
	if (Enseignant::_edit_mode) {
		v = 0;
		label = "Consultation";
	}


	begin_html (admin_title);
	TNG_title ((char*)"Administration du serveur");
	_outp_ += sprintf (_outp_,
				"Rcuprer les <b><a href=\"potentiel_efficace.csv\">potentiels enseignants</a></b> (\"clic droit\" puis \"Save link as...\") "
				"et mettre dans un fichier .csv  ouvrir avec Excel (en laissant la tabulation "
				"comme sparateur de colonnes)<hr>\n");
	_outp_ += sprintf (_outp_, fmt_admin_edit, v, label);
	_outp_ += sprintf (_outp_, "%s", permanents_seulements ? fmt_acces_tous : fmt_acces_restreint);
	_outp_ += sprintf (_outp_, "%s", Enseignant::_tous ? fmt_non_vide : fmt_tous);
	_outp_ += sprintf (_outp_, "%s", fmt_dump_server);
	_outp_ += sprintf (_outp_, "%s", fmt_mandator);
	_outp_ += sprintf (_outp_, "%s", fmt_total_filiere);
	_outp_ += sprintf (_outp_, "%s", fmt_admin_stop);
	_outp_ += sprintf (_outp_, "%s", fmt_admin_restart);
	end_html ();
}

/*
 *	Sauve les services (sauvegarde automatique)
 */

#define FREQ	8	// sauvegarde toutes les 8 modifications.

void auto_save (bool force, const char* filename)
{
	extern const char*	__services;
	static int	count = 0;
	FILE*		fp;

	if ((++count % FREQ) != 0 && !force)
		return;		// pas encore le moment.

	if (filename == 0)
		filename = __services;		// fichier de sauvegarde des services
	// crer le fichier de sauvegarde.
	fp = fopen (filename, "w");
	if (fp == 0) {
		perror (filename);
		return;
	}
	// mettre le numro de version.
	fprintf (fp, "%s", version_sig);
	Enseignant* prof = Enseignant::_head;
	do {
		prof -> save_service (fp);
	} while ((prof = prof -> _suivant) != Enseignant::_head);
	fclose (fp);
}

/*
 *	Termine le serveur.
 */

static void exit_server (char** av, char* user_passwd, FILE* sock)
{
#if ZLOG
	extern gzFile	log_fp;
#endif
	static char	fmt_exited [] =
				"HTTP/1.1 200 Data Follows\n"
				"Server: Patrick 1.0\n"
				"Content-Length: %d\n"
				"Content-Type: text/html\n"
				"\n%s";
	static char msg [] =
		"<body background=fond.jpg><center><h1><font color=red>Server stopped</font></h1></center></body>\n";
	char		tmp [1024];
	int			length;

	if (! SUPERUSER) {
		get_user_passwd (admin_title, sock);
		// le client refera la requte avec l'authentification,
		return;
	}
	length = sprintf (tmp, fmt_exited, sizeof (msg) - 1, msg);
	length = write (fileno (sock), tmp, length);
#if ZLOG
	// vider le buffer.
	gzflush (log_fp, Z_FINISH);
#endif
	::exit (0);
}

/*
 *	Relance le serveur.
 */

static void restart_server (char** av, char* user_passwd, FILE* sock)
{
	static char	fmt_reload [] =
				"HTTP/1.1 200 Data Follows\n"
				"Server: Patrick 1.0\n"
				"Content-Length: %d\n"
				"Content-Type: text/html\n"
				"\n%s";
	static char msg [] =	"<script>\n"
							"setTimeout(function(){top.location='index.html';}, 3000);\n"
							"alert('Server reloaded');</script>\n";

	char		tmp [1024];
	int			length;
	if (! SUPERUSER) {
		get_user_passwd (admin_title, sock);
		// le client refera la requte avec l'authentification,
		return;
	}
	// sauvegarde automatique.
	auto_save ();
	zlog ("Saving data, server restart ...\n");
	// rpondre par anticipation!
	length = sprintf (tmp, fmt_reload, sizeof (msg) - 1, msg);
	(void) write (fileno (sock), tmp, length);
	// re-executer le programme.
	restart ();
	zlog ("restart failed, exiting...\n");
	::exit (1);
}

/*
 *	Affiche tous les enseignants, ou seulement ceux qui ont un service non vide.
 */

static void set_tous (char** av, char* user_passwd, FILE* sock)
{
	if (! SUPERUSER) {
		get_user_passwd (admin_title, sock);
		// le client refera la requte avec l'authentification,
		return;
	}
	Enseignant::_tous = (av [0][2] == '1');
	admin (superuser, sock);	// sock n'est pas rfrenc car le client est identifi.
}

/*
 *	Change le mode d'dition.
 */

static void set_edit_mode (char** av, char* user_passwd, FILE* sock)
{
	if (! SUPERUSER) {
		get_user_passwd (admin_title, sock);
		// le client refera la requte avec l'authentification,
		return;
	}
	Enseignant::_edit_mode = (av [0][2] == '1');
	if (Enseignant::_edit_mode)
		Enseignant::_tous = true;	// on montre tout le monde en mode dition.

	admin (superuser, sock);	// sock n'est pas rfrenc car le client est identifi.
}

/*
 *	Affiche tous les enseignants, ou seulement ceux qui ont un service non vide.
 */

static void inscription_restreinte (char** av, char* user_passwd, FILE* sock)
{
	if (! SUPERUSER) {
		get_user_passwd (admin_title, sock);
		// le client refera la requte avec l'authentification,
		return;
	}
	permanents_seulements = (av [0][2] == '1');
	admin (superuser, sock);	// sock n'est pas rfrenc car le client est identifi.
}

/*
 *	Sauve les services sur disque (sauvegarde explicite).
 */

static void dump_server (char** av, char* user_passwd, FILE* sock)
{
	extern const char*	__services;	// fichier de sauvegarde des services (import de server.cc)

	FILE*	fp;
	// tester les droits.
	if (! SUPERUSER) {
		get_user_passwd (admin_title, sock);
		// le client refera la requte avec l'authentification,
		return;
	}
	// crer le fichier de sauvegarde.
	fp = fopen (__services, "w");
	if (fp == 0) {
		zlog ("fopen (__services) ...\n");
		perror (__services);
		no_data (sock);
		return;
	}
	// mettre le numro de version.
	fprintf (fp, "%s", version_sig);
	Enseignant* prof = Enseignant::_head;
	do {
		prof -> save_service (fp);
	} while ((prof = prof -> _suivant) != Enseignant::_head);
	fclose (fp);
	admin (user_passwd, sock);
	_outp_ += sprintf (_outp_, "<script>alert('Dump done (%s)')</script>\n", __services);
}

/*
 *	Permet de devenir 'Mandator'
 */

static void index_enseignements (char*, FILE*);

static void mandatorize (char** av, char* user_passwd, FILE* sock)
{
	// demander l'identification du client.
	if (! MANDATOR) {
		get_user_passwd (admin_title, sock);
		// le client refera la requte avec l'authentification,
		return;
	}
	index_enseignements (user_passwd, sock);
}

/*
 *	Permet de devenir 'Director'
 */

static void directorize (char** av, char* user_passwd, FILE* sock)
{
	// demander l'identification du client.
	if (! FILADMIN) {
		get_user_passwd ((char*)"Administration du dpartement", sock);
		// le client refera la requte avec l'authentification,
		return;
	}
	affiche_total_filieres (av, user_passwd, sock);
}

/*
 *	Recharge le fichier des services.
 */

#if ANCIEN_FORMAT

void load_service (const char* services)
{
	FILE*		fp;
	char		line [256];
	char*		p;
	Enseignant*	prof;
	int			format = 0;	// 0 super ancien, 1 ancien, 2 nouveau.

	fp = fopen (services, "r");
	if (fp == 0)
		return;

	fgets (line, sizeof (line), fp);
	if (0 == strncmp (line, version_sig, 8)) {
		format = version_sig [8] + '0';	// 1 ou 2 (ou plus).
	} else
		rewind (fp);	// vieux format sans mot magique.

	while (0 != fgets (line, sizeof (line), fp)) {
		if (line [0] != '\'') {
			zlog ("%s: syntaxe 1.\n", services);
			fclose (fp);
			::exit (1);
			return;
		}
		for (p = line+1; *p != '\''; ++p)
			if (*p == '\n') {
				zlog ("Pas de nom ?? : %s\n", line);
				fclose (fp);
				::exit (1);
				return;
			}
		*p++ = 0;
		prof = Enseignant::name2prof (line+1);
		if (prof == 0) {
			zlog ("Enseignant inconnu: %s\n", line+1);
			fclose (fp);
			::exit (1);
			return;
		}
		bool	ok;
		switch (format) {
			case 0:
				ok =  prof -> load_service (p+1);	// sauter l'espace suivant.
				break;
			case 1:
			case 2:
				ok =  prof -> load_service_new (p+1, format);	// sauter l'espace suivant.
				break;
			default:
				zlog ("%s: bad format.\n", services);
				::exit (1);
				return;
		}
		if (!ok) {
			zlog ("%s: syntaxe 2: %s\n", services, prof -> _nom);
			::exit (1);
			return;
		}
	}
	fclose (fp);
}

#else

void load_service (const char* services)
{
	FILE*		fp;
	char		nom [256];
	const char*	err;
	Enseignant*	prof;
	int			format = 0;	// 1 ancien, 2 nouveau, 3 plus nouveau.

	if ((fp = fopen (services, "r")) == 0) {
		zlog ("cannot open '%s'\n", services);
		return;
	}
	// mot magique et version.
	if (1 != fscanf (fp, "Voeux_1.%d\n", &format) || format > NB_VERSIONS) {
		zlog ("\n%s: bad format number: %d\n\n", services, format);
		return;
	}
	--format;	// on commence  0.

	while (1 == fscanf (fp, "'%[^']s", nom))
	{
		(void) fgetc (fp);	// sauter la quote terminant le nom.
		prof = Enseignant::name2prof (nom);
		if (prof == 0) {
			zlog ("Enseignant inconnu: %s\n", nom);
			fclose (fp);
			::exit (1);
			return;
		}
		if (0 != (err = prof -> load_service (fp, format))) {
			zlog ("Error: %s\n", err);
			::exit (1);
			return;
		}
	}
	if (EOF != fgetc (fp))
		zlog ("%s: remaining content ignored...\n", services);
	fclose (fp);
}

#endif

/*
 *	Affiche le service d'un enseignant.
 */

static void service_enseignant (char* nom_ens, char* user_passwd, FILE* sock)
{
	char		title [SMALLBUFLEN];
	Enseignant*	prof = Enseignant::name2prof (nom_ens);

	if (prof == 0) {
		_outp_ += sprintf (_outp_, "<h3>Enseignant inconnu: %s\n", nom_ens);
		return;
	}
	if (!SUPERUSER && !MANDATOR && !FILADMIN && !get_ident (sock, user_passwd, prof)) {
		return;
	}

	sprintf (title, "Service de %s", nom_ens);
	begin_html (title);
	if (MANDATOR)
		prof -> affiche_page_mandatements ();
	else {
		bool	ed = Enseignant::_edit_mode && prof -> _categorie != VAC;
		// restreindre ventuellement l'accs aux extrieurs et autres vacataires de tous poils.
		if (permanents_seulements)
			ed = ed && (prof -> _employeur -> _type == UNIVERSITE || prof -> _categorie == MON);
		// le superuser a tous les droits.
		ed = ed || SUPERUSER;
		prof -> affiche_page (ed ? EDIT_SERVICE	: NON_EDITABLE);
	}
	end_html ();
}

/*
 *	affiche des infos sur les ajouts/retraits de mandatements.
 */

static void log_mandatements (Enseignant* prof, time_t* deb, time_t* fin,
							  int fi, int div, float service_effectue [N_ENS_TYPES])
{
	static const char*	sctp [N_ENS_TYPES] = {
		"Cours", "TD", "TP"
	};
	static const char*	div_name [] = {
		"de l'orientation", "de la formation", "de la recherche", "STAPS", "de la formation permanente"
	};
	char	prenom [SMALLBUFLEN];
	char	nom [SMALLBUFLEN];

	// extraire le prnom de l'enseignant.
	prof -> nom_prenom (nom, prenom);
	zlog ("MAND: %s ", timestamp ());
	zlog ("%s %s: ", prenom, nom);
	zlog ("deb: %s ", AscTime (deb));
	zlog ("fin: %s ", AscTime (fin));
	zlog ("%s %s ", "Formation", fi ? "initiale" : "continue");
	zlog ("Division %s ", div_name [div]);
	for (int i = COURS; i < N_ENS_TYPES; ++i) {
		if (service_effectue [i] != 0)
			zlog ("%s: %g ", sctp [i], service_effectue [i]);
	}
	zlog ("\n");
}

/*
 *	Affiche le formulaire de modification de mandatement.
 */

static void modif_mandatement (char** av, char* user_passwd, FILE* sock)
{
	Enseignant*	prof = Enseignant::name2prof (av [0] + 2);
	char		title [SMALLBUFLEN];
	int			n = 0;

	if (prof == 0 || ! MANDATOR || av [1][0] != 'M')
		return;		// non superuser, ou pas la bonne personne ou pas le bon code.

	for (int i = 1; av [1][i] != '='; ++i)
		n = n*10 + av [1][i] - '0';

	sprintf (title, "Mandatement pour %s", prof -> _nom);
	begin_html (title);
	prof -> form_mandatement (n);	// n rfrence un mandatement existant!
	end_html ();
}

/*
 *	Affiche le formulaire de modification de mandatement pr-rempli avec un enseignement.
 */

static void select_mandatement (char** av, char* user_passwd, FILE* sock)
{
	Enseignant*		prof = Enseignant::name2prof (av [0] + 2);
	char			title [SMALLBUFLEN];
	Enseignement*	e = 0;
	Service*		s = 0;
	int				i;
	float			vol = 0.;
	float			volume [N_ENS_TYPES];

	if (prof == 0 || ! MANDATOR)
		return;		// non superuser, ou pas la bonne personne.

	for (int i = COURS; i < N_ENS_TYPES; ++i)
		volume [i] = 0.;
#ifdef DEBUG
	zlog ("select_mandatement\n");
	for (char** a = av; *a; ++a)
		zlog ("'%s', ", *a);
	zlog ("\n");
#endif

	for (++av ; *av; ++av)
		switch (av [0][0]) {
			case 'E':
				e = Filiere::name2ens (av [0]+2);
				if (e == 0) {
					zlog ("Cours inconnu: %s\n", av [0]+2);
					return;
				}
				continue;
			case 'T':	// cours  mandater.
				i = atoi (av [0]+2);
				if (av [1][0] != 'V') {
					zlog ("Pb de syntaxe.\n");
					return;
				}
				// vrifier qu'il y a bien un cours.
				if (e == 0) {
					zlog ("Pb de syntaxe.\n");
					return;
				}
				// rcuprer le service qui va avec.
				if (s != 0 && s != prof -> get_service (e)) {
					_outp_ += sprintf (_outp_, "<script>alert('Ne mandater qu\\'un service  la fois')</script>\n");
					_outp_ += sprintf (_outp_, fmt_service, prof -> nospaces ());
					return;
				}
				s = prof -> get_service (e);
				vol = atof (av [1]+3);	// V1=12.8

				switch (i) {
					// rpartir les cours/td entre cours et td.
					case COURS:
						if (e -> _cours_td) {
							volume [TD] += vol / 2.;
							volume [COURS] += vol / 2.;
							continue;
						}
						// fall through.
					case TD:
					case TP:
					case MP:
					case DTD:
					case STD:
					case SSS:
						volume [i] += vol;
						continue;
				}
				continue;
		}
	if (volume [COURS] + volume [TD] + volume [TP] + volume [MP] +
		volume [DTD] + volume [STD] + volume [SSS] == 0.)
	{
		_outp_ += sprintf (_outp_, "<script>alert('Slectionner un service  mandater')</script>\n");
		_outp_ += sprintf (_outp_, fmt_service, prof -> nospaces ());
		return;
	}
	sprintf (title, "Mandatement pour %s", prof -> _nom);
	begin_html (title);
	// cela fabriquera un nouveau mandatement.
	prof -> form_mandatement (prof -> _mandatement.length (), volume, s);
	end_html ();
}

/*
 *	Retire un mandatement.
 */

static void remove_mandatement (char** av, char* user_passwd, FILE* sock)
{
	Enseignant*	prof = Enseignant::name2prof (av [0] + 2);
	int			n = 0;

	if (prof == 0 || ! MANDATOR || av [1][0] != 'N')
		return;		// non superuser, ou pas la bonne personne ou pas le bon code.

	for (int i = 2; av [1][i] != 0; ++i)
		n = n*10 + av [1][i] - '0';

	prof -> _mandatement.remove (n, 1);
	// et rafficher la page de l'enseignant.
	_outp_ += sprintf (_outp_, fmt_service, prof -> nospaces ());
	// sauvegarde automatique.
	auto_save ();
}

/*
 *	Retour sans changement de la page de mandatement.
 */

static void retour_mandatement (char** av, char* user_passwd, FILE* sock)
{
	Enseignant*	prof = Enseignant::name2prof (av [0] + 2);

	if (prof == 0 || ! MANDATOR)
		return;		// non superuser, ou pas la bonne personne.

	// rafficher la page de l'enseignant.
	_outp_ += sprintf (_outp_, fmt_service, prof -> nospaces ());
}

/*
 *	Ajoute un mandatement (retour du formulaire html).
 */

static void add_mandatement (char** av, char* user_passwd, FILE* sock) {
	Enseignant*	prof;
	int			num = 0;
	int			n;
	time_t		date = 0;
	time_t		deb = 0;
	time_t		fin = 0;
	float		volume [N_ENS_TYPES];
	bool		fifc = true;
	Division	division = FORMATION;

#ifdef DEBUG
	zlog ("add_mandatement\n");
	for (char** a = av; *a; ++a)
		zlog ("'%s', ", *a);
	zlog ("\n");
#endif

	// Rcuprer l'enseignant.
	prof = Enseignant::name2prof (av [0] + 2);
	if (prof == 0) {
		zlog ("Prof inconnu: %s\n", av [0] + 2);
		return;
	}
	// Si on n'est pas superuser, ne rien modifier.
	if (! MANDATOR) {
		_outp_ += sprintf (_outp_, fmt_service, prof -> nospaces ());
		return;
	}
	// interprter le formulaire.
	for (av += 1; *av; ++av) {
		switch (av [0][0]) {
			default:
				zlog ("Bizarre: %s\n", av [0]);
				continue;
			case 'N':
				num = atoi (av [0]+2);
				if (num >= prof -> _mandatement.length ())
					zlog ("Numro de mandatement trop grand??\n");

				continue;
			case 'D':
				date = String2Time (av [0]+2);
				continue;
			case 'd':
				deb = String2Time (av [0]+2);
				continue;
			case 'f':
				fin = String2Time (av [0]+2);
				continue;
			case 'V':
				n = av [0][1] - '0';
				if (n < COURS || n >= COURS_TD)
					continue;	// ne pas se faire avoir btement!
				volume [n] = atof (av [0] + 3);	// nombre d'heures complmentaires
				continue;
			// la division qui paie.
			case 'v':
				n = av [0][2] - '0';
				if (n > (int) FP || n < (int) ORIENTATION)
					continue;	// ne pas se faire avoir btement!
				division = (Division) n;
				continue;
			case 'F':
				fifc = (av [0][2] == 'I');	// formation initiale/continue.
				continue;
		}
	}
	// rcuprer le mandatement (il existe forcment).
	Mandatement* m = (Mandatement*) prof -> _mandatement [num];
	// affecter les nouvelles valeurs.
	m -> _date = date;
	m -> _fi = fifc;
	m -> _division = division;
	// attention: cours/td <==> 1/2 cours + 1/2 td.
	float	service_effectue [N_ENS_TYPES];
	for (int i = COURS; i < N_ENS_TYPES; ++i)
		service_effectue [i] = m -> _s -> _heures [HEURE_COMP_A][i];
	if (m -> _s -> _ens -> _cours_td) {
		service_effectue [COURS] /= 2;
		service_effectue [TD] += service_effectue [COURS];
	}
	// affecter les volumes et tester la cohrence.
	for (int i = COURS; i < N_ENS_TYPES; ++i) {
		m -> _vol [i] = volume [i];
		if (volume [i] > service_effectue [i]) {
			_outp_ += sprintf (_outp_,
				"\n<script>alert('le nombre d\\'heures de %s mandates (%g) "
				"est suprieur au nombre d\\'heures complmentaires (%g)')</script>\n",
				_genre [i],
				volume [i], service_effectue [i]);
		}
	}
	m -> _taux = Taux::get (deb);
	m -> _deb_periode = deb;
	m -> _fin_periode = fin;
	// et rafficher la page de l'enseignant.
	_outp_ += sprintf (_outp_, fmt_service, prof -> nospaces ());
	// sauvegarde automatique.
	auto_save ();
	log_mandatements (prof, &m -> _deb_periode, &m -> _fin_periode, m -> _fi,
					  m -> _division, service_effectue);
}

/*
 *	Retourne un fichier postscript du tableau rcapitulatif des mandatements.
 */

#define PRIV		2
#define FONCT		0
#define IUT			1
#define INITALE		0
#define CONTINUE	1

/*
 *	Affiche le tableau rcapitulatif des mandatements.
 */

static void recap_mandatements (char** av, char* user_passwd, FILE* sock)
{
	static char	fmt_selection [] =
		"<form method=post action=\"recap_mandatements\">\n"
		"<b>Enseignants "
		"<select name=P onChange='submit()'>\n"
		"<option %s value=2>Priv</option>"
		"<option %s value=0>Fonctionnaire</option>"
		"<option %s value=1>IUT</option>"
		"</select>"
		"Formation "
		"<select name=F onChange='submit()'>\n"
		"<option %s value=0>Initiale</option>"
		"<option %s value=1>Continue</option>"
		"</select> "
		"Date "
		"<select name=D onChange='submit()'>\n"
		"<option %s value=1>Janvier</option>"
		"<option %s value=2>Fvrier</option>"
		"<option %s value=3>Mars</option>"
		"<option %s value=4>Avril</option>"
		"<option %s value=5>Mai</option>"
		"<option %s value=6>Juin</option>"
		"<option %s value=7>Juillet</option>"
		"<option %s value=8>Aout</option>"
		"<option %s value=9>Septembre</option>"
		"<option %s value=10>Octobre</option>"
		"<option %s value=11>Novembre</option>"
		"<option %s value=12>Dcembre</option>"
		"</select> "
		"</form>\n<p>\n";

	static char	fmt_title [] =
		"<tr bgcolor=#000000>"
			"<td align=center><b>Nom</b></td>"
			"<td align=center><b>Statut</b></td>"
			"<td align=center><b>Hr stat.</b></td>"
			"<td align=center><b>Priode</b></td>"
			"<td align=center><b>Hr comp.</b></td>"
			"<td align=center><b>Taux</b></td>"
			"<td align=center><b>Montant</b></td>"
		"</tr>\n";

	static char	fmt_total [] =
		"<tr bgcolor=#aa77aa>"
			"<td>&nbsp;</td>"
			"<td>&nbsp;</td>"
			"<td>&nbsp;</td>"
			"<td>&nbsp;</td>"
			"<td>&nbsp;</td>"
			"<td align=center><b>Total</b></td>"
			"<td align=right><font face=verdana><b>%.2f&nbsp;</b></font></td>"
		"</tr>\n";

	static char	selected [] = "selected";
	static char	empty [] = "";
	int			type_ens = av [0][2] - '0';	// type d'enseignant.
	int			fifc = av [1][2] - '0';		// type de formation (initiale/continue).
	int			mois = atoi (av [2]+2);		// mois de l'anne.
	Mandatement	mandatements [100];			// a devrait suffire!!
	int			n_mand;

	begin_html ("Mandatements");

	_outp_ += sprintf (_outp_, 
		"<table cellpadding=5 cellspacing=0 border=0 width=\"100%%\">"
		"<tr>"
			"<td " BGCOLOR1 "><font face=helvetica color=#ffffff size=+3>Mandatements</font></td>"
		"</tr></table><p>\n");

	_outp_ += sprintf (_outp_, fmt_selection,
					   type_ens == PRIV ? selected : empty,
					   type_ens == FONCT ? selected : empty,
					   type_ens == IUT ? selected : empty,
					   fifc == INITALE ? selected : empty,
					   fifc == CONTINUE ? selected : empty,
					   mois == 1 ? selected : empty,
					   mois == 2 ? selected : empty,
					   mois == 3 ? selected : empty,
					   mois == 4 ? selected : empty,
					   mois == 5 ? selected : empty,
					   mois == 6 ? selected : empty,
					   mois == 7 ? selected : empty,
					   mois == 8 ? selected : empty,
					   mois == 9 ? selected : empty,
					   mois == 10 ? selected : empty,
					   mois == 11 ? selected : empty,
					   mois == 12 ? selected : empty);

	Enseignant* prof = Enseignant::_head;
	float		total = 0.;
	_outp_ += sprintf (_outp_, "<table cellspacing=1 border=2 bgcolor=#ffbbff cellpadding=2 width=\"100%%\">\n");
	_outp_ += sprintf (_outp_, "%s", fmt_title);
	do {
		// liminer les enseignants qui ne remplissent pas le critre d'employeur demand.
		switch (prof -> _employeur -> _type) {
			case PRIVE:
			case BOURSIER:
				if (type_ens == PRIV)
					break;	// ok.
				continue;

			case UNIVERSITE:
				if (prof -> _dept -> iut ()) {
					if (type_ens == IUT)
						break;	// ok.
					continue;
				}
			// fall thru.
			case FONCTIONNAIRE:
				if (type_ens == FONCT)
					break;	// ok.
				continue;
		}
		n_mand = 0;
		for (int i = 0; i < prof -> _mandatement.length (); ++i) {
			Mandatement*	m = (Mandatement*) (prof -> _mandatement [i]);

			if (gmtime (&m -> _date) -> tm_mon+1 != mois)
				continue;
			if (m -> _fi != (fifc == INITALE))
				continue;

			// en voila un qui colle : chercher dans les autres s'il n'y en a pas un
			// de la mme date pour la mme priode...
			for (int n = 0; n < n_mand; ++n) {
				if (mandatements [n]._date == m -> _date
// ne pas tester les priodes pour pouvoir cumuler les mandatements de la mme date.
					&& mandatements [n]._deb_periode == m -> _deb_periode
					&& mandatements [n]._fin_periode == m -> _fin_periode
				)
				{
					// oui : l'accumuler dans son homologue.
					for (int k = COURS; k < N_ENS_TYPES; ++k)
						mandatements [n]._vol [k] += m -> _vol [k];
					goto cont_ext;
				}
			}
			// c'est un mandatement pour une autre date/priode.
			mandatements [n_mand++] = *m;	// copie en bloc.
			cont_ext: ;
		}
		// afficher le total des heures mandates (une ligne par date/priode).
		for (int i = 0; i < n_mand; ++i)
			total += mandatements [i].affiche (prof);
	// faut-il continuer?
	} while ((prof = prof -> _suivant) != Enseignant::_head);
	_outp_ += sprintf (_outp_, fmt_total, total);
	_outp_ += sprintf (_outp_, "</table><p>\n");
	end_html ();
#undef IUT
#undef FONCT
#undef PRIV
#undef INITALE
#undef CONTINUE
}

/*
 *	Affiche les enseignants qui ont des heures complmentaires  mandater.
 */

static void reste_a_mandater (char** av, char* user_passwd, FILE* sock)
{
	Enseignant* prof = Enseignant::_head;
	static char	fmt_title [] =
				"<tr bgcolor=#000060>"
					"<td align=center width=30%%><b>Nom</b></td>"
					"<td align=center width=15%%><b>Statut</b></td>"
					"<td align=center><b>Filires</b></td>"
				"</tr>\n";
	static char	fmt_vol [] =
				"<tr bgcolor=#606060>"
					"<td align=left width=30%%><a href=\"service_%s.cmd\">%s</a></td>"	// nom.
					"<td align=left width=15%%>%s</td>"							// statut.
					"<td align=left>";
	static char	fmt_fin [] = "</td></tr>\n";								// fin des filires.

	begin_html ("Mandatements  effectuer");
	_outp_ += sprintf (_outp_, 
		"<table cellpadding=5 cellspacing=0 border=0 width=\"100%%\">"
		"<tr>"
			"<td " BGCOLOR1 "><font face=helvetica color=#ffffff size=+3>"
			"Mandatements  effectuer</font></td>"
		"</tr></table><p>\n");
	_outp_ += sprintf (_outp_, "<table cellspacing=1 border=2 bgcolor=#ffbbff cellpadding=2 width=\"100%%\">\n");
	_outp_ += sprintf (_outp_, "%s", fmt_title);
	// parcourir la liste des profs.
	do {
		// recopier son service (pour cribler ce qui est mandat).
		int			ntmp;
		Service		tmp [SMALLBUFLEN];	// largement suffisant; on n'est pas des btes !
		Service*	ptmp [MSERVLEN];	// pour se rappeler l'adresse (pointeur dans le mandatement).

		for (ntmp = 0; ntmp < prof -> _service.length (); ++ntmp) {
			ptmp [ntmp] = (Service*) (prof -> _service [ntmp]);
			tmp [ntmp] = * ptmp [ntmp];
		}
		// pour chacun, parcourir ses mandatements.
		for (int i = 0; i < prof -> _mandatement.length (); ++i) {
			Mandatement*	m = (Mandatement*) (prof -> _mandatement [i]);

			for (int n = 0; n < ntmp; ++n) {
				// chercher le service mandat.
				if (m -> _s != ptmp [n])
					continue;
				// dduire des heures complmentaires celles qui sont mandates.
				for (int th = COURS; th < N_ENS_TYPES; ++th)
					tmp [n]._heures [HEURE_COMP_A][th] -= m -> _vol [th];
				// corriger les Cours/TD.
				if (m -> _s -> _ens -> _cours_td && m -> _vol [COURS] != 0) {
					tmp [n]._heures [HEURE_COMP_A][COURS] -= m -> _vol [COURS];
					tmp [n]._heures [HEURE_COMP_A][TD] += m -> _vol [COURS];
				}
			}
		}
		// marquer les filires pour ne les afficher qu'une fois.
		Filiere*	filieres [MFILNB];
		int			nfil;
		bool		deja_vu = false;

		for (nfil = 0; Filiere::filieres [nfil] != 0; ++nfil)
			filieres [nfil] = 0;
		// reparcourir son service pour voir s'il reste des heures non mandates.
		for (int n = 0; n < ntmp; ++n)
			for (int th = COURS; th < N_ENS_TYPES; ++th)
				if (tmp [n]._heures [HEURE_COMP_A][th] != 0.) {
					Filiere* f = ptmp [n] -> _ens -> _groupe -> _filiere;

					if (! deja_vu) {
						// c'est la premire fois qu'on le voit.
						deja_vu = true;
						_outp_ += sprintf (_outp_, fmt_vol,
											prof -> _nom, prof -> _nom,
											prof -> cat2str ());
						filieres [0] = f;	// donc c'est la premire filire.
						_outp_ += sprintf (_outp_, "%s", f -> _nom);
						continue;
					}
					// tester si on a dj vu cette filire.
					for (int i = 0; i < nfil; ++i) {
						if (filieres [i] == f)
							break;
						if (filieres [i] == 0) {
							// non, la mettre dans le tableau et afficher son nom.
							filieres [i] = f;
							_outp_ += sprintf (_outp_, ", %s", f -> _nom);
							break;
						}
					}
				}
		if (deja_vu)
			_outp_ += sprintf (_outp_, "%s", fmt_fin);
	// faut-il continuer?
	} while ((prof = prof -> _suivant) != Enseignant::_head);
	_outp_ += sprintf (_outp_, "</table><p>\n");
	end_html ();
}

/*
 *	Affiche le total des heures par filieres, en quatre colonnes:
 *		statutaires, complmentaires, extrieures, charges.
 */

static void affiche_total_filieres (char** av, char* user_passwd, FILE* sock)
{
	if (! (SUPERUSER || MANDATOR)) {
		no_data (sock);
		return;
	}
	float		ts = 0;
	float		tc = 0;
	float		te = 0;
	Filiere*	f;

	begin_html ("Total des heures par filieres");
	_outp_ += sprintf (_outp_, 
		"<p><table cellpadding=2 cellspacing=0 border=0 width=\"100%%\">"
		"<tr><td bgcolor=#0000d0 align=left>"
			"<font face=helvetica color=#ffffff size=+4>Total des heures par filire</font>"
			"</td></tr></table><p>\n");

	_outp_ += sprintf (_outp_, 
		"<table cellpadding=2 border=1 width=\"100%%\">\n"
		"<tr bgcolor=#ffffff>"
			"<td align=center>Filire</td>"
			"<td align=center>H. Statutaires</td>"
			"<td align=center>H. Complmentaires</td>"
			"<td align=center>H. Extrieures</td>"
			"<td align=center>Total</td>"
		"</tr>\n");

	// parcourir la liste des filires.
	for (int i = 0; (f = Filiere::filieres [i]) != 0; ++i) {
		float	stat = 0;
		float	comp = 0;
		float	ext = 0;

		f -> total (stat, comp, ext, false);
		_outp_ += sprintf (_outp_, 
			"<tr bgcolor=#ffff00>"
				"<td align=left>%s</td>"
				"<td align=right><font face=verdana>%d</font></td>"
				"<td align=right><font face=verdana>%d</font></td>"
				"<td align=right><font face=verdana>%d</font></td>"
				"<td align=right><font face=verdana>%d</font></td>"
			"</tr>\n", f -> _nom,
				(int) (stat+0.5), (int) (comp+0.5), (int) (ext+0.5),
				(int) (stat+0.5) + (int) (comp+0.5) + (int) (ext+0.5));
		ts += stat; stat = 0;
		tc += comp; comp = 0;
		te += ext; ext = 0;
	}
	_outp_ += sprintf (_outp_, 
		"<tr bgcolor=#ffbbff>"
			"<td align=left>Total</td>"
			"<td align=right><font face=verdana><b>%d</b></font></td>"
			"<td align=right><font face=verdana><b>%d</b></font></td>"
			"<td align=right><font face=verdana><b>%d</b></font></td>"
			"<td align=right><font face=verdana><b>%d</b></font></td>"
		"</tr></table>\n",
			(int) (ts+0.5), (int) (tc+0.5), (int) (te+0.5),
			(int) (ts+0.5) + (int) (tc+0.5) + (int) (te+0.5));
	end_html ();
}

/*
 *	affiche des infos sur les ajouts/retraits d'heures de cours.
 */

static void log_infos (Enseignant* prof, Enseignement* ens, int ctp, int hc, float volume, bool prime)
{
	static const char*	th [N_HRS_TYPES/2] = {
		"h stat.", "h comp.", "h comp.-ext", "h stat-ext.", "h grat"
	};
	static const char*	sctp [3] = {
		"Cours", "TD", "TP"
	};
	char	prenom [SMALLBUFLEN];
	char	nom [SMALLBUFLEN];

	// extraire le prnom de l'enseignant.
	prof -> nom_prenom (nom, prenom);
	zlog ("MOD: %s ", timestamp ());
	zlog ("%s %s: ", prenom, nom);
	zlog ("'%s (%s)' ", ens -> _identification, ens -> _intitule);
	if (prime) {
		if (volume == -0.)
			zlog ("Prime retire\n");
		else
			zlog ("Prime ajoute\n");
	} else
		zlog ("%s: %g%s%s\n", sctp [ctp], volume,
				th [hc/2],
				((hc%2)==0) ? "" : " (2nd chx)");
}

/*
 *	Ajoute un service (retour du formulaire html).
 */

static void add_service (char** av, char* user_passwd, FILE* sock) {
	char*	ident;
	char*	nom_ens;
	int		i, th;
	float	heures [N_HRS_TYPES][N_ENS_TYPES];
	float	volume [N_ENS_TYPES];
	bool	gouh [N_ENS_TYPES];
	int		hc [N_ENS_TYPES];
	bool	second_choix [N_ENS_TYPES];
	bool	prioritaire [N_ENS_TYPES];
	int		prime = -1;

#ifdef DEBUG
	for (char** a = av; *a; ++a)
		zlog ("'%s', ", *a);
	zlog ("\n");
#endif

	ident = av [0] + 2;		// E=I110s1a
	nom_ens = av [1] + 2;	// U=Patrick Amar
	if (av [2][0] == 'R') {
		// charge en heures de service ou sous forme de prime.
		prime = (av [2][2] == '0') ? 0 : 1;
	}

	// Rcuprer l'enseignant.
	Enseignant*	prof = Enseignant::name2prof (nom_ens);
	if (prof == 0) {
		zlog ("Prof inconnu: %s\n", nom_ens);
		return;
	}
	// Si on n'est pas en mode dition, ne rien modifier.
	if (! (Enseignant::_edit_mode || SUPERUSER)) {
		_outp_ += sprintf (_outp_, fmt_service, prof -> nospaces ());
		return;
	}
	// Rcuprer le cours.
	Enseignement*	ens = Filiere::name2ens (ident);
	if (ens == 0) {
		zlog ("Cours inconnu: %s\n", ident);
		return;
	}
	if (ens -> _charge_np) {
		if (prime == -1) {
			_outp_ += sprintf (_outp_, "<script>alert('Choisir \"prime\" ou \"heures\"');\ntop.history.back();\n</script>\n");
			return;
		}
	}
	// initialiser les volumes et choix divers
	for (i = 0; i < N_ENS_TYPES; ++i) {
		volume [i] = 0.;
		second_choix [i] = false;
		prioritaire [i] = false;
	}
	// viter que R=? soit qualifi de 'Bizarre'.
	if (av [2][0] == 'R')
		av += 3;
	else
		av += 2;
	for (; *av; ++av) {
		i = av [0][0] - '0';	// 0 -> cours, 1 -> td, 2 -> tp, 3 -> mp
		switch (av [0][1]) {
			default:
				zlog ("Bizarre: %s\n", av [0]);
				continue;
			case '0': case '1':
			case '2': case '3':
			case '4': case '5':
			case '6': case '7':
			case '8': case '9':
				th = av [0][1] - '0';	// stat a, comp a, stat b, comp b
				heures [th][i] = atof (av [0] + 3);	// volume qu' dj l'enseignant.
				continue;
			case 'V':
				volume [i] = atof (av [0] + 3);	// volume de cours
				continue;
			case 'P':
				prioritaire [i] = true;		// prioritaire sur cet enseignement.
				continue;
			case 'U':
				gouh [i] = (av [0][3] == 'g');	// en nombre de groupes
				continue;
			case 'H':
				switch (av [0][3]) {
					case 'c':	hc [i] = HEURE_COMP_A; continue;	// heures complmentaires
					case 'e':	hc [i] = HEURE_COMP_EXT; continue;	// heures complmentaires extrieures
					case 'x':	hc [i] = HEURE_STAT_EXT; continue;	// heures statutaires extrieures
					case 'g':	hc [i] = HEURE_GRAT; continue;	// heures gratos
					default:
					case 's':	hc [i] = HEURE_STAT_A; continue;	// heures statutaires
				}
				continue;
			case 'C':
				second_choix [i] = true;		// plan B
				continue;
		}
	}
	for (i = 0; i < N_ENS_TYPES; ++i) {
		// ajuster les volumes si des groupes sont slectionns.
		if (gouh [i])
			volume [i] *= ens -> _volume [i];
		// mettre le bon type d'heures.
		if (second_choix [i])
			hc [i] += 1;
	}
	// journaliser les modifications.
	for (i = 0; i < 3; ++i)
		if (volume [i] != 0.)
			log_infos (prof, ens, i,
						hc [i],
						volume [i], ens -> _charge_np && prime == 1);
	// chercher si l'enseignant n'a pas dj un service dans ce cours.
	Service*	s = prof -> get_service (ens);
	bool		nouv_serv = (s == 0);	// nouveau cours.
	bool		form_vide = true;		// traquer le formulaire vide (tout  0 ???).

	if (nouv_serv)
		s = new Service (ens);	// premier service dans cet enseignement.
	// indiquer s'il s'agit d'une charge paye avec une prime.
	s -> _prime = (ens -> _charge_np && prime == 1);
	// affecter le service.
	for (i = 0; i < N_ENS_TYPES; ++i) {
		for (th = 0; th < N_HRS_TYPES; ++th)
			s -> _heures [th][i] = heures [th][i];
		if (volume [i] != 0.) {
			// le formulaire est non vide.
			form_vide = false;
			// ne tenir compte de la priorit que si le volume est non nul.
			s -> _prioritaire [i] = prioritaire [i];
			// ne pas comptabiliser les heures de charges choisies sous forme de prime.
			if (!s -> _prime)
				s -> _heures [hc [i]][i] += volume [i];
			// retirer ce volume du manque de cet enseignement (meme dans le cas d'une prime).
			ens -> _manque [i] -= volume [i];
		}
	}
	// mettre la date de maintenant  ce service.
	s -> _date = time (0);
	// plus du tiers du service statutaire d'un titulaire du dept. Info. fait  Polytech?
	if (prof -> _employeur -> _type == UNIVERSITE &&
		prof -> _dept -> _a_nous && prof -> _serv_annuel > 0 &&
		0 == strncmp ("POLYTECH", s -> _ens -> _groupe -> _filiere -> _identification, 8))
	{
		float	total_polytech = 0;

		if (nouv_serv)
			total_polytech = s -> eq_td (HEURE_STAT_A) + s -> eq_td (HEURE_STAT_B)
						   + s -> eq_td (HEURE_STAT_EXT);
		// calculer le total des heures faites  Polytech.
		for (int i = 0; i < prof -> _service.length (); ++i) {
			Service*	srv = (Service*) prof -> _service [i];

			if (0 == strncmp ("POLYTECH", srv -> _ens -> _groupe -> _filiere -> _identification, 8)
			||	0 == strncmp ("CH5", srv -> _ens -> _groupe -> _identification, 3))
			{
				total_polytech += srv -> eq_td (HEURE_STAT_A) + srv -> eq_td (HEURE_STAT_B)
								+ srv -> eq_td (HEURE_STAT_EXT);
			}
		}
		// pas glop: raler, mais ne pas interdir.
		if (total_polytech > prof -> _serv_annuel / 3.)
			_outp_ += sprintf (_outp_, "\n<script>alert('Attention\\: vous faites plus du tiers de votre\\nservice statutaire hors du dept. Informatique.');\n</script>\n");
	}
	// n'ajouter qu'un service non nul.
	if (nouv_serv) {
		if (!form_vide)	prof -> add_service (s);
	} else
	if (s -> vide ())
		prof -> remove_service (s);	// virer le service devenu vide.
	// montrer les changements.
	_outp_ += sprintf (_outp_, fmt_service, prof -> nospaces ());
	// sauvegarde automatique.
	auto_save ();
}

/*
 *	Retourne un fichier postscript reprsentant la fiche d'engagement de paiement d'heures complmentaires.
 */

static void fiche_engagement (char* ident, char* user_passwd, FILE* sock)
{
	Enseignant*	prof;
	int			n;
	char*		nom;

#if FICHE_LATEX
	for (nom = ident; *nom != '_'; ++nom)
		;
	*nom++ = 0;
	n = atoi (ident);	// numero du service.
	prof = Enseignant::name2prof (nom);
	if (prof != 0)
		prof -> fiche_engagement (n);
	else
		no_data (sock);
#else
	bool	with_date = true;

	if (ident [0] == 'V') {
		with_date = false;
		++ident;
	}

	for (nom = ident; *nom != '_'; ++nom)
		;
	*nom++ = 0;
	n = atoi (ident);
	prof = Enseignant::name2prof (nom);
	// seul le mandator a le droit.
	if (prof != 0 && MANDATOR)
		prof -> fiche_engagement (n, with_date);
	else
		no_data (sock);
#endif
}

/*
 *	Retourne un fichier postscript reprsentant l'attestation de service fait.
 */

static void attestation_service_fait (char* ident, char* user_passwd, FILE* sock)
{
	Enseignant*	prof = Enseignant::name2prof (ident);

	// ont le droit le mandator, le superuser et le prof en question.
	if (prof != 0 && ((user_passwd [0] != 0 && MANDATOR) || get_ident (sock, user_passwd, prof)))
		prof -> attestations_service (1, 2, true);
}

#ifdef STAT_ET_COMP

/*
 *	Retourne un fichier postscript reprsentant l'attestation de service avec les hstat et hcc.
 */

static void service_stat_comp (char* ident, char* user_passwd, FILE* sock)
{
	Enseignant*	prof = Enseignant::name2prof (ident);

	// ont le droit le mandator, le superuser et le prof en question.
	if (prof != 0 && ((user_passwd [0] != 0 && MANDATOR) || get_ident (sock, user_passwd, prof)))
		prof -> etat_statutaire_complementaire ();
}

/*
 *	La mme avec les hcc de Polytech.
 */

static void service_stat_comp_ext (char* ident, char* user_passwd, FILE* sock)
{
	Enseignant*	prof = Enseignant::name2prof (ident);

	// ont le droit le mandator, le superuser et le prof en question.
	if (prof != 0 && ((user_passwd [0] != 0 && MANDATOR) || get_ident (sock, user_passwd, prof)))
		prof -> etat_statutaire_complementaire (true);
}

#else

/*
 *	Retourne un fichier postscript reprsentant l'tat d'heures statutaires rempli.
 */

static void etat_statutaire (char* ident, char* user_passwd, FILE* sock)
{
	Enseignant*	prof = Enseignant::name2prof (ident);

	// ont le droit le mandator, le superuser et le prof en question.
	if (prof != 0 && ((user_passwd [0] != 0 && MANDATOR) || get_ident (sock, user_passwd, prof)))
		prof -> etat_statutaire ();
}
#endif

/*
 *	Fabrique un fichier postscript reprsentant l'attestation de service fait pour tous les enseignants.
 */

static void attestation_tous_enseignants (char** av, char* user_passwd, FILE* sock)
{
	// ont le droit le mandator, et le superuser.
	if (! (SUPERUSER || MANDATOR)) {
		get_user_passwd (admin_title, sock);
		// le client refera la requte avec l'authentification,
		return;
	}
	Enseignant::attestations_service (1);
}

/*
 *	Fabrique un fichier postscript reprsentant l'tat prvisionnel de service.
 */

static void etat_prev (char* ident, char* user_passwd, FILE* sock)
{
	Enseignant*	prof = Enseignant::name2prof (ident);

	// ont le droit le mandator, le superuser et le prof en question.
	if (prof != 0 && ((user_passwd [0] != 0 && MANDATOR) || get_ident (sock, user_passwd, prof)))
		prof -> attestations_service (1, 2, false);
}

/*
 *	Fabrique un fichier postscript reprsentant l'tat prvisionnel de service pour tous les enseignants.
 */

static void previsionnel_tous_enseignants (char** av, char* user_passwd, FILE* sock)
{
	// ont le droit le mandator, et le superuser.
	if (! (SUPERUSER || MANDATOR)) {
		get_user_passwd (admin_title, sock);
		// le client refera la requte avec l'authentification,
		return;
	}
	Enseignant::attestations_service (0);
}

/*
 *	Fabrique un fichier postscript reprsentant l'tat hybride de service.
 */

static void etat_hybride (char* ident, char* user_passwd, FILE* sock)
{
	Enseignant*	prof = Enseignant::name2prof (ident);

	// ont le droit le mandator, le superuser et le prof en question.
	if (prof != 0 && ((user_passwd [0] != 0 && MANDATOR) || get_ident (sock, user_passwd, prof)))
	{
		prof -> attestations_hybride (1, 2);
	}
}

/*
 *	Fabrique un fichier postscript reprsentant l'tat hybride de service pour tous les enseignants.
 */

static void hybride_tous_enseignants (char** av, char* user_passwd, FILE* sock)
{
	// ont le droit le mandator, et le superuser.
	if (! (SUPERUSER || MANDATOR)) {
		get_user_passwd (admin_title, sock);
		// le client refera la requte avec l'authentification,
		return;
	}
	Enseignant::attestations_service (2);
}

/*
 *	Affiche la liste des enseignements (pour ajouter un nouveau cours).
 */

static void ajoute_cours (char** av, char* user_passwd, FILE* sock)
{
	static char	aide [] =
		"<hr><b>&nbsp;&nbsp;Choisissez un module dans une filire, vous obtiendrez la "
		"liste des cours de ce module. Il vous suffira alors de slectionner l'enseignement "
		"qui vous intresse pour avoir le formulaire de saisie.</b><br><hr><br>";

	static char msg [] = "<script>alert('Probleme de jeu de caracteres: essayez ISO8859-1')</script>\n";
	Enseignant*	prof = Enseignant::name2prof (av [0]+2);

	// probleme de codage de caractres: le nom n'est pas reconnu.
	if (prof == 0) {
		_outp_ += sprintf (_outp_, "%s", msg);
		zlog ("Probleme de jeu de caracteres: '%s'\n", av [0]+2);
		return;
	}
	// pas encore identifi, ou pas la bonne personne.
	if (! get_ident (sock, user_passwd, prof))
		return;

#ifdef DEBUG
	zlog ("ajoute_cours:	");
	for (char** v = av; *v; ++v)
		zlog ("%s ", *v);
	zlog ("\n");
#endif

	begin_html ("Filires (saisie service)");
	_outp_ += sprintf (_outp_, "%s", aide);
	for (Filiere** p = Filiere::filieres; *p; ++p)
		(*p) -> affiche (0, av [0]+2);
	end_html ();
}

/*
 *	Retire des cours d'un service.
 */

static void retire_cours (char** av, char* user_passwd, FILE* sock) {
	Enseignant*		prof;
	Enseignement*	e = 0;
	Service*		s = 0;
	int				i, th;
	float			vol;

	prof = Enseignant::name2prof (av [0]+2);	// P=Patrick Amar.
	if (prof == 0) {
		zlog ("Prof inconnu: %s\n", av [0]+2);
		return;
	}

	// Si on n'est pas en mode dition, ne rien modifier.
	if (! (Enseignant::_edit_mode || SUPERUSER)) {
		_outp_ += sprintf (_outp_, fmt_service, prof -> nospaces ());
		return;
	}

	if (! get_ident (sock, user_passwd, prof))
		return;		// pas encore identifi, ou pas la bonne personne.

#ifdef DEBUG
	zlog ("retrait: %s\n", Host);
	for (char** v = av; *v; ++v)
		zlog ("%s ", *v);
	zlog ("\n");
#endif

	for (++av ; *av; ++av)
		switch (av [0][0]) {
			case 'E':	// identification d'un cours.
				e = Filiere::name2ens (av [0]+2);
				if (e == 0) {
					zlog ("Cours inconnu: %s\n", av [0]+2);
					return;
				}
				s = prof -> get_service (e);
				// probablement un 'repost' du formulaire.
				if (s == 0) {
					_outp_ += sprintf (_outp_, fmt_service, prof -> nospaces ());
					return;
				}
				continue;

			case 'T':	// cours  retirer.
				i = atoi (av [0]+2);
				++av;	// regarder le volume  retirer.
				if (av [0][0] != 'V') {
					zlog ("Pb de syntaxe.\n");
					return;
				}
				vol = atof (av [0]+3);	// V1=12.8
				th = av [0][1] - '0';
				if (th == HEURE_COMP_A) {
					for (int n = 0; n < prof -> _mandatement.length (); ++n) {
						Mandatement* m = (Mandatement*) prof -> _mandatement [n];

						if (m -> _s == s && s -> _heures [th][i]-vol < m -> _vol [i])
						{
							zlog ("Il faut d'abord retirer le mandatement avant les heures...\n");
							break;
						}
					}
				}
#ifdef CHARGES_EN_HC
				if (s -> _ens -> _charge_np)
					for (th = HEURE_STAT_A; th <= HEURE_GRAT; th += 2)
						if (s -> _heures [th][TD] == vol)
							break;	// on DOIT trouver...
#endif
				// accessibles que par le superuser (exemple: reliquats et avances)
				if ((s -> _ens -> _groupe -> _su_only || s -> _ens -> _su_only)
					&& !SUPERUSER)
				{
					_outp_ += sprintf (_outp_,
						"<script>alert('Seuls les administrateurs peuvent supprimer ces heures.');\ntop.history.back();</script>\n");
					continue;
				}
				// journaliser les modifications.
				log_infos (prof, e, i, th, -vol, s -> _ens -> _charge_np && i == TD && vol == 0.);
				s -> _heures [th][i] -= vol;
				// mettre la date de maintenant  ce service.
				s -> _date = time (0);
				// dans le cas d'une charge sous forme de prime mettre
				// artificiellement le bon nombre d'heures.
				if (s -> _ens -> _charge_np && i == TD && vol == 0.)
					vol = s -> _ens -> _volume [i];
				// ajouter au manque de cet enseignement.
				s -> _ens -> _manque [i] += vol;
				// le service est-il devenu vide ?
				for (i = 0; i < N_ENS_TYPES; ++i)
					for (th = 0; th < N_HRS_TYPES; ++th)
						if (s -> _heures [th][i] != 0)
							goto encore;	// non, alors on continue.
				// retirer ce service de ceux du prof.
				prof -> remove_service (s);
		encore:	continue;

			case 'V':	// volume de l'enseignement (inutile ici).
				continue;
		}
	// montrer les changements.
	_outp_ += sprintf (_outp_, fmt_service, prof -> nospaces ());
	// sauvegarde automatique.
	auto_save ();
}

/*
 *	Donne l'ensemble des enseignements.
 */

static void index_enseignements (char* user_passwd, FILE* sock) {
	begin_html ("Index des enseignements");
	static char	fmt [] =
		"	<td align=center height=40 bgcolor=\"%s\">\n"
			"<table height=40 width=100%% cellspacing=0 cellpadding=0 border=0>\n"
			"	<tr><td align=center><font size=2 color=#000000 face=Helvetica><a href=total_trous.cmd><b>%s</b></a></font></td></tr>\n"
			"	<tr><td align=center><font size=2 color=#000000 face=Helvetica><a href=maquette_excel.csv><b>%s</b></a></font></td></tr>\n"
			"</table>\n"
			"</td>\n"
		"	<td align=center height=40 bgcolor=\"%s\"><font size=5 color=#FFFF00 face=Helvetica>%s</font></td>\n"
		"	<td align=center height=40 bgcolor=\"%s\">\n"
			"<table height=40 width=100%% cellspacing=0 cellpadding=0 border=0>\n"
			"	<tr><td align=center><font size=2 color=#000000 face=Helvetica><a href=total_bosses.cmd><b>%s</b></a></font></td></tr>\n"
			"	<tr><td align=center><font size=2 color=#000000 face=Helvetica><a href=seconds_choix.cmd><b>%s</b></a></font></td></tr>\n"
			"</table>\n"
			"</td>\n";
	TNG_round_title_begin ();
	_outp_ += sprintf (_outp_, fmt,
		page_colors [TitreEmphase], "Trous", "Maquette",
		page_colors [TitreEmphase], "INDEX DES ENSEIGNEMENTS",
		page_colors [TitreEmphase], "Bosses", "Second");
	TNG_round_title_end ();
	// afficher le bouton pour avoir le tableau des mandatements.
	if (MANDATOR)
		_outp_ += sprintf (_outp_,
				"<form method=post action=recap_mandatements>"
				"<input type=hidden name=P value=0>"
				"<input type=hidden name=F value=0>"
				"<input type=hidden name=D value=6>"
				"<b><input type=button onClick='submit()' value=Rcapitulatif> des "
				"mandatements.</b></form><p>\n"

				"<form method=post action=reste_a_mandater>"
				"<b><input type=button onClick='submit()' value=Heures> pas "
				"encore mandates.</b></form><p>\n"

				"<hr>Rcuprer les <b><a href=\"detail_excel.csv\">services dtaills</a></b> (\"clic droit\" puis \"Save link as...\") "
				"et mettre dans un fichier .csv  ouvrir avec Excel (en laissant la tabulation "
				"comme sparateur de colonnes)<hr>\n"

				"<form method=post action=att_tous_ens>"
				"<b><input type=button onClick='submit()' value=Attestations> de "
				"service fait pour tous les enseignants.</b></form><p>\n"

				"<form method=post action=prev_tous_ens>"
				"<b><input type=button onClick='submit()' value='tat Prvisionnel'> des "
				"services (tous les enseignants).</b></form><p>\n"
				);
	else {
		// lien pour rcuprer les services en Excel.
		_outp_ += sprintf (_outp_,
				"<hr>Rcuprer les <b><a href=\"detail_excel.csv\">services dtaills</a></b> (\"clic droit\" puis \"Save link as...\") "
				"et mettre dans un fichier .csv  ouvrir avec Excel (en laissant la tabulation "
				"comme sparateur de colonnes)<hr>\n");
	}

	for (Filiere** p = Filiere::filieres; *p; ++p)
		(*p) -> affiche (0);
	end_html ();
}

/*
 *	Donne la liste des enseignants qui participent  une filire.
 */

static void enseignants_filiere (char** av, char* user_passwd, FILE* sock)
{
	static char	fmt_selection [] =
		"<form method=post action=\"select\">\n"
		"<input type=hidden name=F value=\"%s\">"
		"<b>Heures "
		"<select name=H onChange='submit()'>\n"
		"<option %s value=0>Statutaires</option>"
		"<option %s value=1>Complmentaires</option>"
		"<option %s value=2>Statutaires Extrieures</option>"
		"<option %s value=3>Complmentaires Extrieures</option>"
		"<option %s value=4>Bnvoles</option>"
		"</select>"
		"Priode "
		"<select name=P onChange='submit()'>\n"
		"<option %s value=0>Toute l'anne</option>"
		"<option %s value=1>Semestre 1</option>"
		"<option %s value=2>Semestre 2</option>"
		"</select> "
		"<input name=T %s type=checkbox onClick='submit()'> Tout afficher</b>"
		"</form>\n";

	static char	fmt_title [] =
		"<tr bgcolor=#004070>"
		"<td align=center><b>Nom</b></td>"
		"<td align=center><b>Statut</b></td>"
		"<td align=center><b>HStat.</b></td>"
		"<td align=center><b>HComp.</b></td>"
		"<td align=center><b>HStat-Ext.</b></td>"
		"<td align=center><b>HComp-Ext</b></td>"
		"<td align=center><b>HGrat</b></td></tr>\n";

	static const char*	fmt_tot [5] = {
		"<tr bgcolor=#004070><td align=right><b>Total %s</b></td><td>&nbsp;</td><td>%g</td><td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td></tr>\n",
		"<tr bgcolor=#004070><td align=right><b>Total %s</b></td><td>&nbsp;</td><td>&nbsp;</td><td>%g</td><td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td></tr>\n",
		"<tr bgcolor=#004070><td align=right><b>Total %s</b></td><td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td><td>%g</td><td>&nbsp;</td><td>&nbsp;</td></tr>\n",
		"<tr bgcolor=#004070><td align=right><b>Total %s</b></td><td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td><td>%g</td><td>&nbsp;</td></tr>\n",
		"<tr bgcolor=#004070><td align=right><b>Total %s</b></td><td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td><td>%g</td></tr>\n",
	};
	
	Filiere*	f = Filiere::name2filiere (av [0]+2);
	static const char	selected [] = "selected";
	static const char	empty [] = "";
	char		tmp [SMALLBUFLEN];
	int			th = av [1][2] - '0';		// type d'heures.
	Periode		periode = (Periode) (av [2][2] - '0');	// semestre dans l'anne.
	bool		tous = (av [3] != 0);		// faut-il afficher tous les enseignants?

	if (f == 0) {
		zlog ("Filire inconnue: %s\n", av [0]+2);
		return;
	}

	if (!MANDATOR && !SUPERUSER && !FILADMIN) {
		get_user_passwd (admin_title, sock);
		// le client refera la requte avec l'authentification,
		return;
	}

	(void) sprintf (tmp, "Enseignants en %s", f -> _nom);
	begin_html ("Enseignants");
	TNG_title (tmp);

	_outp_ += sprintf (_outp_, fmt_selection, av [0]+2,
					   th == 0 ? selected : empty,
					   th == 1 ? selected : empty,
					   th == 2 ? selected : empty,
					   th == 3 ? selected : empty,
					   th == 4 ? selected : empty,
					   periode == ANNEE ? selected : empty,
					   periode == SEMESTRE_UN ? selected : empty,
					   periode == SEMESTRE_DEUX ? selected : empty,
					   tous ? "checked" : empty);

	Enseignant* prof = Enseignant::_head;
	float	vol = 0.;
	Genre	genre = INCONNU;

	_outp_ += sprintf (_outp_, "<table cellspacing=1 border=2 bgcolor=#004070 cellpadding=2 width=\"100%%\">\n");
	_outp_ += sprintf (_outp_, "%s", fmt_title);
	do {
		for (int i = 0; i < prof -> _service.length (); ++i)
			if (((Service*) (prof -> _service [i])) -> _ens -> _groupe -> _filiere == f) {
				if (genre == INCONNU)	// le premier.
					genre = prof -> get_type ();

				if (genre != prof -> get_type ()) {
					if (vol > 0.)
						_outp_ += sprintf (_outp_, fmt_tot [th], Enseignant::genre2str [genre], vol);
					genre = prof -> get_type ();
					vol = 0.;
				}
				vol += prof -> service_filiere (f, th, tous, periode);
				break;
			}
	} while ((prof = prof -> _suivant) != Enseignant::_head);
	if (vol > 0.)
		_outp_ += sprintf (_outp_, fmt_tot [th], Enseignant::genre2str [genre], vol);
	_outp_ += sprintf (_outp_, "</table><p>\n");
	end_html ();
}

/*
 *	Donne les informations demandes par l'UFR sous forme d'un fichier .csv
 */

#if HEURES_FLOAT
static void potentiel_efficace (char** av, char* user_passwd, FILE* sock)
{
	Enseignant* prof = Enseignant::_head;
	Genre	genre;
	Genre	prev_genre = INCONNU;
	float	tot_stat_th = 0;
	float	tot_dech = 0;
	float	tot_serv_du = 0;
	float	tot_serv_fait = 0;
	float	tot_hcc_ufr = 0;

	// demander l'identification du client.
	if (! (SUPERUSER || MANDATOR || FILADMIN)) {
		get_user_passwd (admin_title, sock);
		// le client refera la requte avec l'authentification,
		ignore_ret = write (fileno (sock), cancel_get_passwd, sizeof (cancel_get_passwd) - 1);
		return;
	}
	// afficher l'entte.
	_outp_ += sprintf (_outp_, "Nom-Prnom	Origine	Grade	Service statutaire thorique	Dcharges statutaires	heures de dcharge	Service d	Heures prsentielles faites dans la composante	Heures prsentielles hors composante	Autre composante	Heures Rfrentiel pdagogie	Heures Rfrentiel Recherche et Administration	Service fait	HCC payes\n");
	// parcourir la liste des profs.
	do {
		char orig [128];
		float sstat_th = 0;
		float dech_st = 0;
		char* intit_dech = (char*)"";
		float serv_du = 0;
		float hpres_ufr = 0;
		float hpres_aut = 0;
		const char* aut_comp = "";
		float hpedag = 0;
		float hadm = 0;
		float sfait = 0;
		float hcc_ufr = 0;

		if (prof -> _service.length () == 0)
			continue;

		genre = prof -> get_type ();
		if (genre != prev_genre) {
			// afficher le bas de la liste sauf au dbut.
			if (prev_genre != INCONNU) {
				// imprimer les totaux.
				_outp_ += sprintf (_outp_, "TOTAL\t\t\t%g\t\t%g\t%g\t\t\t\t\t\t%g\t%g\n\n",
								tot_stat_th, tot_dech, tot_serv_du,
								tot_serv_fait, tot_hcc_ufr);
				tot_stat_th = 0;
				tot_dech = 0;
				tot_serv_du = 0;
				tot_serv_fait = 0;
				tot_hcc_ufr = 0;
			}
			prev_genre = genre;
			_outp_ += sprintf (_outp_, "%s\n", (char*)Enseignant::genre2str [genre]);
		}
		prof -> potentiel (sstat_th, dech_st, intit_dech, serv_du,
							hpres_ufr, hpres_aut, aut_comp,
							hpedag, hadm, sfait, hcc_ufr, orig);
		char	nom [128];
		char	prenom [128];

		prof -> nom_prenom (nom, prenom);
		_outp_ += sprintf (_outp_, "\"%s %s\"	%s	%s	%g	%s	%g	%g	%g	%g	%s	%g	%g	%g	%g\n",
							nom, prenom, orig, prof -> cat2str (),
							sstat_th, intit_dech, dech_st,
							serv_du, hpres_ufr, hpres_aut, aut_comp,
							hpedag, hadm, sfait, hcc_ufr);
							
		tot_stat_th += sstat_th;
		tot_dech += dech_st;
		tot_serv_du += serv_du;
		tot_serv_fait += sfait;
		tot_hcc_ufr += hcc_ufr;
	} while ((prof = prof -> _suivant) != Enseignant::_head);
}

#else

static void potentiel_efficace (char** av, char* user_passwd, FILE* sock)
{
	Enseignant* prof = Enseignant::_head;
	Genre	genre;
	Genre	prev_genre = INCONNU;
	int	tot_stat_th = 0;
	int	tot_dech = 0;
	int	tot_serv_du = 0;
	int	tot_serv_fait = 0;
	int	tot_hcc_ufr = 0;

	// demander l'identification du client.
	if (! (SUPERUSER || MANDATOR || FILADMIN)) {
		get_user_passwd (admin_title, sock);
		// le client refera la requte avec l'authentification,
		ignore_ret = write (fileno (sock), cancel_get_passwd, sizeof (cancel_get_passwd) - 1);
		return;
	}
	// afficher l'entte.
	_outp_ += sprintf (_outp_, "Nom-Prnom	Origine	Grade	Service statutaire thorique	Dcharges statutaires	heures de dcharge	Service d	Heures prsentielles faites dans la composante	Heures prsentielles hors composante	Autre composante	Heures Rfrentiel pdagogie	Heures Rfrentiel Recherche et Administration	Service fait	HCC payes\n");
	// parcourir la liste des profs.
	do {
		char orig [128];
		int sstat_th = 0;
		int dech_st = 0;
		char* intit_dech = (char*)"";
		int serv_du = 0;
		int hpres_ufr = 0;
		int hpres_aut = 0;
		const char* aut_comp = "";
		int hpedag = 0;
		int hadm = 0;
		int sfait = 0;
		int hcc_ufr = 0;

		if (prof -> _service.length () == 0)
			continue;

		genre = prof -> get_type ();
		if (genre != prev_genre) {
			// afficher le bas de la liste sauf au dbut.
			if (prev_genre != INCONNU) {
				// imprimer les totaux.
				_outp_ += sprintf (_outp_, "TOTAL\t\t\t%d\t\t%d\t%d\t\t\t\t\t\t%d\t%d\n\n",
								tot_stat_th, tot_dech, tot_serv_du,
								tot_serv_fait, tot_hcc_ufr);
				tot_stat_th = 0;
				tot_dech = 0;
				tot_serv_du = 0;
				tot_serv_fait = 0;
				tot_hcc_ufr = 0;
			}
			prev_genre = genre;
			_outp_ += sprintf (_outp_, "%s\n", (char*)Enseignant::genre2str [genre]);
		}
		prof -> potentiel (sstat_th, dech_st, intit_dech, serv_du,
							hpres_ufr, hpres_aut, aut_comp,
							hpedag, hadm, sfait, hcc_ufr, orig);
		char	nom [128];
		char	prenom [128];

		prof -> nom_prenom (nom, prenom);
		_outp_ += sprintf (_outp_, "\"%s %s\"	%s	%s	%d	%s	%d	%d	%d	%d	%s	%d	%d	%d	%d\n",
							nom, prenom, orig, prof -> cat2str (),
							sstat_th, intit_dech, dech_st,
							serv_du, hpres_ufr, hpres_aut, aut_comp,
							hpedag, hadm, sfait, hcc_ufr);
							
		tot_stat_th += sstat_th;
		tot_dech += dech_st;
		tot_serv_du += serv_du;
		tot_serv_fait += sfait;
		tot_hcc_ufr += hcc_ufr;
	} while ((prof = prof -> _suivant) != Enseignant::_head);
}
#endif

/*
 *	Donne TOUTES les informations sous forme d'un fichier texte qui sera mang par EXCEL ensuite.
 */

static void detail_excel (char** av, char* user_passwd, FILE* sock)
{
	// parcourir la liste des profs.
	Enseignant* prof = Enseignant::_head;
	bool		il_glande;
	char		tc [SMALLBUFLEN];
	char*		nom = 0;
	char*		prenom = 0;
	char		orig [256];

	// demander l'identification du client.
	if (! (SUPERUSER || MANDATOR || FILADMIN)) {
		get_user_passwd (admin_title, sock);
		// le client refera la requte avec l'authentification,
		ignore_ret = write (fileno (sock), cancel_get_passwd, sizeof (cancel_get_passwd) - 1);
		return;
	}
	// afficher l'entte.
	_outp_ += sprintf (_outp_, "Filire\tGroupe\tEnseignement\tDescriptif\tEffectifs\tCodePedago\tType Heures\tSemestre\tCours\tTD\tTP\tMP\tDemi-TP\tSoutien TD\tSoutien 2e Session\tNom Prnom\tOrigine\tCatgorie\tSection CNU\n");
	// et la suite.
	do {
		il_glande = true;
		for (int i = 0; i < prof -> _service.length (); ++i) {
			if (il_glande) {
				il_glande = false;
				// l'origine de l'enseignant.
				switch (prof -> _employeur -> _type) {
					case UNIVERSITE:
						sprintf (orig, "Dept. %s", prof -> _dept -> _nom);
						break;
					case FONCTIONNAIRE:
						sprintf (orig, "Fonct. %s", prof -> _employeur -> _nom);
						break;
					case PRIVE:
						sprintf (orig, "Priv. %s", prof -> _employeur -> _nom);
						break;
					case BOURSIER:
						sprintf (orig, "Boursier %s", prof -> _employeur -> _nom);
						break;
				}
				strcpy (tc, prof -> _nom);
				nom = index (tc, ' ');
				// sparer le nom du prnom.
				if (nom == 0) {
					nom = tc;
					prenom = (char*)"";
				} else {
					*nom++ = 0;
					prenom = tc;
				}
				// mettre le nom en capitales et mapper l'espace en soulign.
				for (unsigned char* cp = (unsigned char*) nom; *cp != 0; ++cp)
					if (('a' <= *cp && *cp <= 'z') || 0340 <= *cp)
						*cp -= 'a' - 'A';
					else if (*cp == ' ')
						*cp = '_';
				// mapper l'espace en soulign dans le prnom aussi.
				for (unsigned char* cp = (unsigned char*) prenom; *cp != 0; ++cp)
					if (*cp == ' ')
						*cp = '_';
			}
			Service*	s = (Service*) (prof -> _service [i]);
			float		volumes [N_HRS_TYPES] [N_ENS_TYPES];	// volumes horaires dans chaque type d'enseignement.
			char*		intitule [N_HRS_TYPES];
			char*		filiere [N_HRS_TYPES];
			static const char*	th [N_HRS_TYPES] = {
				"STAT", "STAT",
				"COMP", "COMP",
				"COMP_EXT", "COMP_EXT",
				"STAT_EXT", "STAT_EXT",
				"BENEVOLE", "BENEVOLE"
			};
			// numro de semestre
			int sem = 1 + Per2Semestre (s -> _ens -> _periode);
			bzero ((void*) volumes, sizeof (volumes));
			/*
			 *	Afficher tous les types d'heures.
			 */
			for (int sc = 0; sc < N_HRS_TYPES; ++sc) {
				if (0 == s -> affiche_service (sc, &filiere [sc], &intitule [sc], volumes [sc]))
					continue;

				_outp_ += sprintf (_outp_, "%s\t%s\t%s",
						filiere [sc],
						s -> _ens -> _groupe -> _nom,
						intitule [sc]);
				// le descriptif du cours.
				char*	ds = s -> _ens -> _verbatim;
				char	c;
				// vide?.
				if (ds == 0)
					_outp_ += sprintf (_outp_, "\t-");
				// non: remplacer les \t et \n par un espace et mettre la chaine entre guillemets.
				else {
					*_outp_++ = '\t';
					*_outp_++ = '\"';
					for (;;) {
						switch (c = *ds++) {
						case 0:
							*_outp_++ = '\"';
							break;
						// remplacer les guillemets par des apostrophes.
						case '\"':
							*_outp_++ = '\'';
							continue;

						case '\t':
						case '\n':
						case ' ':
							while (*ds == '\t' || *ds == '\n' || *ds == ' ') ++ds;
							*_outp_++ = ' ';
							continue;
#if HEURES_FLOAT
						case ';':
							*_outp_++ = ',';
							continue;

						case '&':
							*_outp_++ = 'e';
							continue;
#endif
						default:
							*_outp_++ = c;
							continue;
						}
						break;
					}
				}
				_outp_ += sprintf (_outp_, "\t%d\t%s\t%s\t%d\t",
						s -> _ens -> _effectifs,
						s -> _ens -> _activite_pedag,
						th [sc],
						sem);

				for (int n = 0; n < N_ENS_TYPES; ++n) {
#if HEURES_FLOAT
					_outp_ += sprintf (_outp_, "%g\t", volumes [sc][n]);
#else
					char	tc [64];

					(void) sprintf (tc, "%g", volumes [sc][n]);
					// remplacer le point dcimal par la virgule.
					for (char* p = tc; *p != 0; ++p)
						if (*p == '.')
							*p = ',';

					_outp_ += sprintf (_outp_, "%s\t", tc);
#endif
				}
				_outp_ += sprintf (_outp_, "%s %s\t%s\t%s\t%d\n", nom, prenom, orig, prof -> cat2str (), prof -> _cnu);
			}
		}
	} while ((prof = prof -> _suivant) != Enseignant::_head);
}

/*
 *	Donne le dtail d'une filire.
 */

static void detail_filiere (char* ident, char* user_passwd, FILE* sock)
{
	Filiere*	f = Filiere::name2filiere (ident);

	if (f == 0) {
		zlog ("Filire inconnue: %s\n", ident);
		return;
	}
	// demander l'identification du client.
	if (! (SUPERUSER || MANDATOR || FILADMIN)) {
		char	tc [256];

		sprintf (tc, "Administration du %s", f -> _nom);
		get_user_passwd (tc, sock);
		// le client refera la requte avec l'authentification,
		return;
	}
	begin_html (f -> _nom);
	f -> table ();
	end_html ();
}

/*
 *	Donne le dtail de tous les trous.
 */

static void trous_total (char* user_passwd, FILE* sock)
{
	begin_html ("Enseignements non pourvus");
	for (Filiere** pf = Filiere::filieres; *pf != 0; ++pf)
		(*pf) -> affiche (2, (char*)"", 3);	// 3 pour avoir le dtail maximum.
	end_html ();
}

/*
 *	Donne le dtail de toutes les bosses.
 */

static void bosses_total (char* user_passwd, FILE* sock)
{
	begin_html ("Enseignements sur-pourvus");
	for (Filiere** pf = Filiere::filieres; *pf != 0; ++pf)
		(*pf) -> affiche (3, (char*)"", 3);	// 3 pour avoir le dtail maximum.
	end_html ();
}

/*
 *	Donne le dtail de toutes les bosses.
 */

static void seconds_choix (char* user_passwd, FILE* sock)
{
	Enseignant*	prof = Enseignant::_head;	// parcourir la liste des enseignants.
	Enseignant*	prev = 0;

	if (! SUPERUSER) {
		get_user_passwd (admin_title, sock);
		// le client refera la requte avec l'authentification,
		return;
	}

	begin_html ("Enseignements pourvus en second choix");

	TNG_round_title_begin ();
	_outp_ += sprintf (_outp_,
		"<td align=center height=40 bgcolor=\"%s\"><font size=5 color=#000000 face=Helvetica>%s</font></td>\n",
		page_colors [TitreEmphase], "Enseignements pourvus en second choix");
	TNG_round_title_end ();

	_outp_ += sprintf (_outp_, "<center><table width=90%%>\n");
	do {
		for (int i = 0; i < prof -> _service.length (); ++i) {
			Service* s = (Service*) prof -> _service [i];

			for (int enst = 0; enst < N_ENS_TYPES; ++enst)
				if (s -> _heures [HEURE_STAT_B][enst] != 0 ||
					s -> _heures [HEURE_COMP_B][enst] != 0 ||
					s -> _heures [HEURE_COMP_EXT_B][enst] != 0 ||
					s -> _heures [HEURE_STAT_EXT_B][enst] != 0 ||
					s -> _heures [HEURE_GRAT_B][enst] != 0)
				{
					// le nom de l'enseignant.
					if (prof == prev)
						_outp_ += sprintf (_outp_, "<tr><td align=left>&nbsp;</td>");
					else
						_outp_ += sprintf (_outp_,
							"<tr><td align=left><a href=\"service_%s.cmd\">%s</a></td>",
							prof -> nospaces (), prof -> _nom);
					// o il enseigne.
					_outp_ += sprintf (_outp_,
						"<td align=left><a href=\"cours_%s.cmd\">%s - %s</a></td></tr>\n",
							s -> _ens -> _identification,
							s -> _ens -> _identification,
							s -> _ens -> get_intitule (false));
					prev = prof;
					break;
				}
		}
		prof = prof -> _suivant;
	} while (prof != Enseignant::_head);	// jusqu'au dbut exclu.

	_outp_ += sprintf (_outp_, "</table></center>\n");
	end_html ();
}

/*
 *	Reconstitue la maquette
 */

static void maquette_excel (char* user_passwd, FILE* sock)
{
	// demander l'identification du client.
	if (! (SUPERUSER || MANDATOR)) {
		get_user_passwd (admin_title, sock);
		// le client refera la requte avec l'authentification,
		ignore_ret = write (fileno (sock), cancel_get_passwd, sizeof (cancel_get_passwd) - 1);
		return;
	}
	for (Filiere** pf = Filiere::filieres; *pf != 0; ++pf) {
		for (int g = 0; g < (*pf) -> _groupes.length (); ++g) {
			Groupe*	groupe = (Groupe*)  (*pf) -> _groupes [g];

			_outp_ += sprintf (_outp_, "%s", groupe -> _nom);
			_outp_ += sprintf (_outp_,
				"	code nomenclature UE	intitul UE	TC ou option	Code APOGEE	Nombre IP	Nb heures cours	nb tudiants par groupe de cours	Nb heures TD	nb tudiants par groupe de TD	Nb heures TP	nb tudiants par groupe de TP	Total EQTD\n");
			for (int semestre = 1; semestre <= 2; ++semestre) {
				bool	s_done = false;	// pour n'afficher le semestre que s'il est non vide.
				int		UE_num = 0;		// numro d'UE.

				for (int n = 0; n < groupe -> _groupes.length (); ++n) {
					Enseignement*	ens = (Enseignement*) groupe -> _groupes [n];
					// numro de semestre
					int sem = 1 + Per2Semestre (ens -> _periode);
					if (sem != semestre)
						continue;
					// indiquer le semestre avant la premire UE.
					if (!s_done) {
						s_done = true;
						_outp_ += sprintf (_outp_, "Semestre %d\n", semestre);
					}
					// enseignement commun?
					bool			commun = (ens -> _base != 0 ||
											  ens -> _total_grp > ens -> _ngrp [TD] + ens -> _ngrp [TP]);

					float	cours = ens -> _volume [COURS];
					float	td = ens -> _volume [TD];
					float	tp = ens -> _volume [TP];
					// corriger pour les Cours/TD.
					if (ens -> _cours_td) {
						cours /= 2.;
						td += cours;
					}
					float	eq_td = cours * Service::taux [COURS] +
									td * Service::taux [TD] +
									tp * Service::taux [TP];

					_outp_ += sprintf (_outp_, "UE%d	%s	%s%s		%s		%g		%g		%g		%g\n",
						UE_num++, ens -> _identification,
						ens -> get_intitule (), commun ? " (commun)" : "",
						ens -> _apogee_id == 0 ? "-" : ens -> _apogee_id, 
						cours, td, tp, eq_td);
				}
			}
		}
	}
}

/*
 *	Donne le dtail des trous d'une filire.
 */

static void trous_filiere (char* ident, char* user_passwd, FILE* sock)
{
	Filiere*	f = Filiere::name2filiere (ident);
	char		tmp [SMALLBUFLEN];

	if (f == 0) {
		zlog ("Filire inconnue: %s\n", ident);
		return;
	}
	sprintf (tmp, "Enseignements non pourvus en: %s", f -> _nom);
	begin_html (tmp);
	f -> affiche (2, (char*)"", 3);	// 3 pour avoir le dtail maximum.
	end_html ();
}

/*
 *	Donne le dtail des 'bosses' d'une filire.
 */

static void bosses_filiere (char* ident, char* user_passwd, FILE* sock)
{
	Filiere*	f = Filiere::name2filiere (ident);
	char		tmp [SMALLBUFLEN];

	if (f == 0) {
		zlog ("Filire inconnue: %s\n", ident);
		return;
	}
	sprintf (tmp, "Enseignements sur-pourvus en: %s", f -> _nom);
	begin_html (tmp);
	f -> affiche (3, (char*)"", 3);	// 3 pour avoir le dtail maximum.
	end_html ();
}

/*
 *	Donne l'ensemble des enseignants.
 */

static void index_enseignants (char* user_passwd)
{
	static char fmt_lambda [] =
		"<tr>\n"
		"	<td width=%d>\n"
		"		<table bgcolor=\"%s\" width=%d cellspacing=0 cellpadding=0 border=0>\n"
		"		<tr><td valign=top align=center>&nbsp;</td></tr>\n"
		"		</table>\n"
		"	</td>\n"
		"	<td bgcolor=#CCCCCC align=center><font color=#000>%s</td>\n"
		"	<td bgcolor=#CCCCCC align=center><font color=#000>Statut</td>\n"
		"	<td bgcolor=#CCCCCC align=center><font color=#000>S.Stat.</td>\n"
		"	<td bgcolor=#CCCCCC align=center><font color=#000>S.Fait</td>\n"
		"	<td bgcolor=#CCCCCC align=center><font color=#000>Diff.</td>\n"
		"	<td width=12></td>\n"
		"</tr>\n";

	static char fmt_total_lambda [] =
		"<tr>\n"
		"	<td width=%d>\n"
		"		<table bgcolor=\"%s\" width=%d cellspacing=0 cellpadding=0 border=0>\n"
		"		<tr><td valign=top align=center>&nbsp;</td></tr>\n"
		"		</table>\n"
		"	</td>\n"
		"	<td bgcolor=#CCCCCC align=center><font color=#000>&nbsp;</td>\n"
		"	<td bgcolor=#CCCCCC align=center><font color=#000><b>TOTAL</b></td>\n"
		"	<td bgcolor=#CCCCCC align=right><font color=#000><b>%d</b>&nbsp;</td>\n"
		"	<td bgcolor=#CCCCCC align=right><font color=#000><b>%d</b>&nbsp;</td>\n"
		"	<td bgcolor=#CCCCCC align=right><font color=#000><b>%d</b>&nbsp;</td>\n"
		"	<td width=12></td>\n"
		"</tr>\n";

	// formats avec colonnes supplmentaires (Polytech et CFA).

	static char fmt [] =
		"<tr>\n"
		"	<td width=%d>\n"
		"		<table bgcolor=\"%s\" width=%d cellspacing=0 cellpadding=0 border=0>\n"
		"		<tr><td valign=top align=center>&nbsp;</td></tr>\n"
		"		</table>\n"
		"	</td>\n"
		"	<td bgcolor=#CCCCCC align=center><font color=#000>%s</td>\n"
		"	<td bgcolor=#CCCCCC align=center><font color=#000>Statut</td>\n"
		"	<td bgcolor=#CCCCCC align=center><font color=#000>S.Stat.</td>\n"
		"	<td bgcolor=#CCCCCC align=center><font color=#000>S.Fait</td>\n"
		"	<td bgcolor=#CCCCCC align=center><font color=#000>Diff.</td>\n"
		"	<td bgcolor=#CCCCCC align=center><font color=#000>Ptech</td>\n"
		"	<td bgcolor=#CCCCCC align=center><font color=#000>CFA</td>\n"
		"	<td width=12></td>\n"
		"</tr>\n";

	static char fmt_total [] =
		"<tr>\n"
		"	<td width=%d>\n"
		"		<table bgcolor=\"%s\" width=%d cellspacing=0 cellpadding=0 border=0>\n"
		"		<tr><td valign=top align=center>&nbsp;</td></tr>\n"
		"		</table>\n"
		"	</td>\n"
		"	<td bgcolor=#CCCCCC align=center><font color=#000>&nbsp;</td>\n"
		"	<td bgcolor=#CCCCCC align=center><font color=#000><b>TOTAL</b></td>\n"
		"	<td bgcolor=#CCCCCC align=right><font color=#000><b>%d</b>&nbsp;</td>\n"
		"	<td bgcolor=#CCCCCC align=right><font color=#000><b>%d</b>&nbsp;</td>\n"
		"	<td bgcolor=#CCCCCC align=right><font color=#000><b>%d</b>&nbsp;</td>\n"
		"	<td bgcolor=#CCCCCC align=right><font color=#000><b>%d</b>&nbsp;</td>\n"
		"	<td bgcolor=#CCCCCC align=right><font color=#000><b>%d</b>&nbsp;</td>\n"
		"	<td width=12></td>\n"
		"</tr>\n";

	static char fmt_head [256] = { 0 };

	if (fmt_head [0] == 0) {
		sprintf (fmt_head, 
					"LISTE DES ENSEIGNANTS</font></td>\n"
					"<td align=center bgcolor=\"%s\">"
					"<a href=detail_enseignants.cmd><font color=\"%s\">Dtail</a>",
				page_colors [TitreEmphase], page_colors [Detail]);
	}

	Enseignant* prof = Enseignant::_head;
	Genre	genre;
	Genre	prev_genre = INCONNU;
	static const int	largeur = 200;
	int		tot_stat = 0;
	int		tot_fait = 0;
	int		tot_polytech = 0;
	int		tot_cfa = 0;
	bool	avec_polytech_et_cfa = SUPERUSER || MANDATOR;

	begin_html ("Liste des enseignants");
	TNG_title (fmt_head);

	if (avec_polytech_et_cfa) {
		_outp_ += sprintf (_outp_,
				"Rcuprer la <b><a href=\"index_enseignants_excel.csv\">liste des services des enseignants</a></b> (\"clic droit\" puis \"Save link as...\") "
				"et mettre dans un fichier .csv  ouvrir avec Excel (en laissant la tabulation "
				"comme sparateur de colonnes)<hr>\n");
	}

	bool	ligne_claire = false;
	do {
		int	stat;
		int	fait;
		int	polytech = 0;
		int	cfa = 0;

		genre = prof -> get_type ();
		if (genre != prev_genre) {
			ligne_claire = false;
			// afficher le bas de la liste sauf au dbut.
			if (prev_genre != INCONNU) {
				_outp_ += sprintf (_outp_,
							avec_polytech_et_cfa ? fmt_total : fmt_total_lambda,
							largeur+50, page_colors [Liste], largeur+36,
							tot_stat, tot_fait, tot_fait-tot_stat,
							tot_polytech, tot_cfa);
				tot_stat = 0;
				tot_fait = 0;
				tot_polytech = 0;
				tot_cfa = 0;
				TNG_bot_list (largeur);
				_outp_ += sprintf (_outp_, "<p>\n");
			}
			prev_genre = genre;
			TNG_top_list (largeur, (char*)Enseignant::genre2str [genre], (char*)"98%", 1);
			_outp_ += sprintf (_outp_,
						avec_polytech_et_cfa ? fmt : fmt_lambda,
						largeur+50,
						page_colors [Liste],
						largeur+36, Enseignant::genre2orig [genre]);
		}
		if (avec_polytech_et_cfa)
			prof -> affiche_liste (largeur, &stat, &fait, ligne_claire, &polytech, &cfa);
		else
			prof -> affiche_liste (largeur, &stat, &fait, ligne_claire);

		ligne_claire = !ligne_claire;
		tot_stat += stat;
		tot_fait += fait;
		tot_polytech += polytech;
		tot_cfa += cfa;
	} while ((prof = prof -> _suivant) != Enseignant::_head);

	_outp_ += sprintf (_outp_,
					avec_polytech_et_cfa ? fmt_total : fmt_total_lambda,
					largeur+50, page_colors [Liste], largeur+36,
					tot_stat,
					tot_fait,
					tot_fait-tot_stat,
					tot_polytech,
					tot_cfa);

	TNG_bot_list (largeur);
	end_html ();
}

/*
 *	Donne la liste des enseignants pour affecter un cours.
 */

static void liste_enseignants_cours (char* ident, char* user_passwd, FILE* sock)
{
	Enseignant* 	prof = Enseignant::_head;
	Enseignement*	ens;
	Genre			genre;
	Genre			prev_genre = INCONNU;
	int				ncol = 0;
	char			tmp [1024];

	// demander l'identification du client.
	if (!SUPERUSER) {
		get_user_passwd (admin_title, sock);
		// le client refera la requte avec l'authentification,
		return;
	}

	ens = Filiere::name2ens (ident+1);
	if (ens == 0)
		return;

	begin_html ("Liste des enseignants");
	sprintf (tmp, "Enseignant pour %s</font></td>\n", ident+1);
	TNG_title (tmp);
	do {
		genre = prof -> get_type ();
		if (genre != prev_genre) {
			// afficher le bas de la liste sauf au dbut.
			if (prev_genre == INCONNU) {
				_outp_ += sprintf (_outp_, "<table width=100%% cellspacing=4 cellpadding=0 border=0>\n");
			} else {
				_outp_ += sprintf (_outp_, "</td></tr>\n");
			}
			ncol = 0;
			prev_genre = genre;
			_outp_ += sprintf (_outp_, "<tr bgcolor=\"%s\"><td colspan=3 align=center>%s</td></tr><tr>\n",
						page_colors [Liste],
						Enseignant::genre2str [genre]);
		}
		// Extraire le prnom de l'enseignant.
		char	prenom [128];
		char	nom [128];

		prof -> nom_prenom (nom, prenom);
		// mettre le nom en capitales.
		for (unsigned char* cp = (unsigned char*) nom; *cp != 0; ++cp)
			if (('a' <= *cp && *cp <= 'z') || 0340 <= *cp)
				*cp -= 'a' - 'A';

		_outp_ += sprintf (_outp_, "<td><a href=\"cours_%s.%s.cmd\">%s %s</a></td>\n",
									ident+1, prof -> nospaces (), prenom, nom);
		if (++ncol == 3) {
			ncol = 0;
			_outp_ += sprintf (_outp_, "</tr><tr>\n");
		}
	} while ((prof = prof -> _suivant) != Enseignant::_head);
	_outp_ += sprintf (_outp_, "</table>\n");
	end_html ();
}

/*
 *	Donne l'index des enseignants sous forme d'un fichier texte qui sera mang par EXCEL ensuite.
 */

static void index_enseignants_excel (char** av, char* user_passwd, FILE* sock)
{
	Enseignant* prof = Enseignant::_head;

	// demander l'identification du client.
	if (! (SUPERUSER || MANDATOR)) {
		get_user_passwd (admin_title, sock);
		// le client refera la requte avec l'authentification,
		ignore_ret = write (fileno (sock), cancel_get_passwd, sizeof (cancel_get_passwd) - 1);
		return;
	}
	// afficher l'entte.
	_outp_ += sprintf (_outp_, "NOM	DPARTEMENT	STATUT	SERVICE STATUTAIRE	SERVICE  FAIT	DIFFRENCE	SERVICE  POLYTECH	SERVICE EN CFA\n");
	// et la suite.
	do {
		float	total = 0.;
		float	total_stat = 0.;
		float	total_polytech = 0.;
		float	total_cfa = 0.;
		char	tc [SMALLBUFLEN];
		char*	nom = 0;
		char*	prenom = 0;
		char	orig [256];

		if (prof -> _service.length () == 0)
			continue;

		// l'origine de l'enseignant.
		switch (prof -> _employeur -> _type) {
			case UNIVERSITE:
				sprintf (orig, "%s", prof -> _dept -> _nom);
				break;
			case FONCTIONNAIRE:
				sprintf (orig, "%s", prof -> _employeur -> _nom);
				break;
			case PRIVE:
				sprintf (orig, "%s", prof -> _employeur -> _nom);
				break;
			case BOURSIER:
				sprintf (orig, "Boursier %s",
					prof -> _employeur -> _nom [0] == '_'
					 ? prof -> _employeur -> _nom + 1		// ne pas afficher le soulign.
					 : prof -> _employeur -> _nom );
				break;
		}
		strcpy (tc, prof -> _nom);
		nom = index (tc, ' ');
		// sparer le nom du prnom.
		if (nom == 0) {
			nom = tc;
			prenom = (char*)"";
		} else {
			*nom++ = 0;
			prenom = tc;
		}
		// mettre le nom en capitales et mapper l'espace en soulign.
		for (unsigned char* cp = (unsigned char*) nom; *cp != 0; ++cp)
			if (('a' <= *cp && *cp <= 'z') || 0340 <= *cp)
				*cp -= 'a' - 'A';
			else if (*cp == ' ')
				*cp = '_';
		// mapper l'espace en soulign dans le prnom aussi.
		for (unsigned char* cp = (unsigned char*) prenom; *cp != 0; ++cp)
			if (*cp == ' ')
				*cp = '_';
		// calculer le service de cet enseignant.
		for (int i = 0; i < prof -> _service.length (); ++i) {
			Service*	s = (Service*) prof -> _service [i];
			bool	moniteur_polytech = false;
			char	nom_doctorant [128];
			// est-ce un tutorat de doctorant Polytech?
			if (0 == strncmp (s -> _ens -> _groupe -> _identification, "Tutorat", 7) &&
				s -> _ens -> _intitule != 0 &&
				0 == strncmp (s -> _ens -> _intitule, "Tutorat de ", 11))
			{
				strcpy (nom_doctorant, s -> _ens -> _intitule+11);
				Enseignant*	moniteur = Enseignant::name2prof (nom_doctorant);

				moniteur_polytech = (moniteur != 0) && moniteur -> _dept -> search ("Polytech");
				}
			// accumuler les heures faites  Polytech.
			if (0 == strncmp ("POLYTECH", s -> _ens -> _groupe -> _filiere -> _identification, 8)
			||	0 == strncmp ("CH5", s -> _ens -> _groupe -> _identification, 3)
			||	moniteur_polytech)
			{
				total_polytech += s -> eq_td (HEURE_STAT_A) + s -> eq_td (HEURE_STAT_B)
								+ s -> eq_td (HEURE_COMP_A) + s -> eq_td (HEURE_COMP_B)
								+ s -> eq_td (HEURE_COMP_EXT) + s -> eq_td (HEURE_STAT_EXT)
								+ s -> eq_td (HEURE_GRAT);
			}
			// accumuler les heures faites en CFA.
			if (0 == strncmp ("_CFA",
					s -> _ens -> _groupe -> _filiere -> _identification - 4 +
					strlen (s -> _ens -> _groupe -> _filiere -> _identification), 4)
			||	0 == strncmp ("CHARGES-CFA-MIAGE", s -> _ens -> _groupe -> _identification, 17))
			{
				total_cfa += s -> eq_td (HEURE_STAT_A) + s -> eq_td (HEURE_STAT_B)
						   + s -> eq_td (HEURE_COMP_A) + s -> eq_td (HEURE_COMP_B)
						   + s -> eq_td (HEURE_COMP_EXT) + s -> eq_td (HEURE_STAT_EXT)
						   + s -> eq_td (HEURE_GRAT);
			}
			// accumuler les heures statutaires effectue.
			total_stat += s -> eq_td (HEURE_STAT_A) + s -> eq_td (HEURE_STAT_B)
				   		+ s -> eq_td (HEURE_STAT_EXT);
			// et le total gnral.
			total += s -> eq_td (HEURE_STAT_A) + s -> eq_td (HEURE_STAT_B)
				   + s -> eq_td (HEURE_COMP_A) + s -> eq_td (HEURE_COMP_B)
				   + s -> eq_td (HEURE_COMP_EXT) + s -> eq_td (HEURE_STAT_EXT)
				   + s -> eq_td (HEURE_GRAT);
		}
#if HEURES_FLOAT
		char	buffer [128];
		char*	s = Enseignant::disponibilite [(int)(prof -> _disponibilite)]._intitule;

		for (char* d = buffer; 0 != (*d = *s); ++s, ++d)
			if (*s == '&') {
				*d = '<';	//  faire pour tous les codes HTML...
				while (*++s != ';')
					;
			}

		_outp_ += sprintf (_outp_, "%s %s\t%s\t%s %s\t%d\t%g\t%g\t%g\t%g\n",
					nom, prenom, orig, prof -> cat2str (),
					buffer,
					prof -> _serv_annuel,
					total, (total - prof -> _serv_annuel),
					total_polytech, total_cfa);
#else
		_outp_ += sprintf (_outp_, "%s %s\t%s\t%s %s\t%d\t%d\t%d\t%d\t%d\n",
					nom, prenom, orig, prof -> cat2str (),
					Enseignant::disponibilite [(int)(prof -> _disponibilite)]._intitule,
					prof -> _serv_annuel,
					(int) (total+0.5), ((int) (total+0.5) - prof -> _serv_annuel),
					(int) (total_polytech+0.5), (int) (total_cfa+0.5));
#endif
	} while ((prof = prof -> _suivant) != Enseignant::_head);
}

/*
 *	Donne le dtail des enseignements de tous les enseignants.
 */

static void detail_enseignants (char* user_passwd, FILE* sock)
{
	Enseignant* prof = Enseignant::_head;

	// demander l'identification du client.
	if (! (SUPERUSER || MANDATOR)) {
		get_user_passwd (admin_title, sock);
		// le client refera la requte avec l'authentification,
		return;
	}

	begin_html ("Dtail des service des enseignants");
	do {
		if (prof -> _service.length () > 0)
			prof -> affiche_detail ();
	} while ((prof = prof -> _suivant) != Enseignant::_head);
	end_html ();
}

/*
 *	Donne l'ensemble des cours d'un groupe.
 */

static void index_group (char* ident, char* user_passwd, FILE* sock)
{
	Groupe*	g;
	char*	nom_ens = index (ident, Service::_sep [0]);

	if (nom_ens)
		*nom_ens++ = 0;

	g = Filiere::name2group (ident);
	if (g == 0) {
		zlog ("Groupe inconnu: %s\n", ident);
		return;
	}
	begin_html (g -> _nom);
	// accessibles que par le superuser (exemple: reliquats et avances)
	if (nom_ens != 0 && !SUPERUSER && g -> _su_only) {
		_outp_ += sprintf (_outp_,
				"<script>alert('Seuls les administrateurs ont acces a cette page.');\ntop.history.back();</script>\n");
	} else
		g -> affiche (0, nom_ens, 1);
	end_html ();
}

/*
 *	Envoie un mail aux enseignants de ce cours.
 */

static void mail_enseignants_UE (char* ident, char* user_passwd, FILE* sock)
{
	Enseignement*	ens = Filiere::name2ens (ident);

	// enseignement inconnu
	if (ens == 0) {
		zlog ("Cours inconnu: %s\n", ident);
		return;
	}

	char		email_list [2048];
	Enseignant*	liste_ens [1024];
	int			n_ens = 0;

	// calculer la liste des emails des enseignants de cette UE.
	char*	ep = email_list;
	for (Enseignant* prof = Enseignant::_head;;) {
		Service*	s = prof -> get_service (ens);

		if (s != 0 && prof -> _email != 0) {
			char	real_mail [256];
			// ne le mettre qu'une fois!
			for (int i = 0; i < n_ens; ++i)
				if (liste_ens [i] == prof)
					goto suivant;

			(void) prof -> get_email (real_mail);
			ep += sprintf (ep, "%s,", real_mail);
			liste_ens [n_ens++] = prof;
			// peu probable, mais ne pas se planter btement.
			if (n_ens > 1024)
				break;
		}
	suivant:
		prof = prof -> _suivant;
		if (prof == Enseignant::_head)
			break;
	}
	ep [-1] = 0;	// virer la derniere virgule.

	_outp_ += sprintf (_outp_,
				"<script>window.location.href='mailto:%s'; window.history.back(); </script>\n",
				email_list);
}

/*
 *	Envoie un mail aux enseignants de ce groupe de cours.
 */

static void mail_enseignants_GR (char* ident, char* user_passwd, FILE* sock)
{
	Groupe*	group = Filiere::name2group (ident);

	// enseignement inconnu
	if (group == 0) {
		zlog ("Groupe inconnu: %s\n", ident);
		return;
	}

	// demander l'identification du client.
	if (! (SUPERUSER || MANDATOR || FILADMIN)) {
		get_user_passwd (admin_title, sock);
		// le client refera la requte avec l'authentification,
		return;
	}

	char		email_list [10240];
	char*		ep = email_list;
	char		real_mail [256];
	Enseignant*	liste_ens [1024];
	int			n_ens = 0;

	n_ens = group -> liste_enseignants (liste_ens, 1024);
	// calculer la liste des emails des enseignants de ce groupe.
	for (int n = 0; n < n_ens; ++n) {
		(void) liste_ens [n] -> get_email (real_mail);
		ep += sprintf (ep, "%s,", real_mail);
	}
	ep [-1] = 0;	// virer la derniere virgule.
	_outp_ += sprintf (_outp_,
				"<script>window.location.href='mailto:%s'; window.history.back(); </script>\n",
				email_list);
}

/*
 *	Affiche le formulaire html pour ajouter un nouveau cours.
 */

static void saisie_cours (char* ident, char* user_passwd, FILE* sock)
{
	Enseignant*		prof = 0;
	char*			nom_ens = index (ident, Service::_sep [0]);
	if (nom_ens)
		*nom_ens++ = 0;
	Enseignement*	ens = Filiere::name2ens (ident);
	bool			avec_saisie = false;

	// enseignement inconnu
	if (ens == 0) {
		zlog ("Cours inconnu: %s\n", ident);
		return;
	}
	// si le nom du prof n'est pas fourni, utiliser l'identification.
	if (nom_ens == 0) {
		if (user_passwd [0] != 0 && !(SUPERUSER||MANDATOR)) {
			static char	tmp [64];
			int			i;

			for (i = 0; (tmp [i] = user_passwd [i]) != ':'; ++i)
				;
			tmp [i] = 0;
			if (0 != (prof = Enseignant::name2prof (tmp)))
				nom_ens = tmp;	// voila le nom du prof!
		}
	} else {
	// enseignant inconnu
		prof = Enseignant::name2prof (nom_ens);
		if (prof == 0) {
			_outp_ += sprintf (_outp_, "<h3>Enseignant inconnu: %s\n", nom_ens);
			return;
		}
	// sommes nous identifi ?
		if (Enseignant::_edit_mode && ! get_ident (sock, user_passwd, prof))
			return;

		avec_saisie = true;
	}
	// est-ce une charge?
	if ((Enseignant::_edit_mode || SUPERUSER) && nom_ens && ens -> _charge_np && avec_saisie) {
		// chercher si l'enseignant n'a pas dj une charge de ce type
		Service* s = prof -> get_service (ens);
		if (s != 0 || ens -> _manque [TD] == 0.) {
			_outp_ += sprintf (_outp_,
					"<script>top.location='service_%s.cmd'; confirm('Cette charge%s est deja attribuee !!')</script>\n",
					prof -> nospaces (),
					s != 0 ? " vous" : "");
			return;
		}
		begin_html ("Saisie service");
		ens -> formulaire (prof);
		end_html ();
		return;
	}
	// non, c'est un cours standard.
	is_superuser = SUPERUSER;
	if ((Enseignant::_edit_mode || SUPERUSER) && nom_ens && avec_saisie) {
		begin_html ("Saisie service");
		// accessibles que par le superuser (exemple: reliquats et avances)
		if ((ens -> _groupe -> _su_only || ens -> _su_only) && !SUPERUSER) {
			_outp_ += sprintf (_outp_,
				"<script>alert('Seuls les administrateurs ont acces a cette page.');\ntop.history.back();</script>\n");
		} else
			ens -> formulaire (prof);
	} else {
		char	tmp [2014];

		sprintf (tmp, "%s %s\n", ens -> _groupe -> _identification, ens -> _intitule);
		begin_html (tmp);
		ens -> affiche (1);
	}
	end_html ();
}

/*
 *	Excute une requte du client.
 */

void exec_get (char* cmd, char* user_passwd, FILE* sock)
{
	time_t	now = time (0);
	struct tm*	t;

	t = gmtime (&now);
	zlog ("%30.30s: %-29.29s => %02d/%02d - %02d:%02d\n",
			Host, cmd,
			t -> tm_mday, t -> tm_mon+1,
			(t -> tm_mon < 3 || t -> tm_mon > 9)
				? t -> tm_hour+1
				: t -> tm_hour+2,
			t -> tm_min);

	if (!strncmp (cmd, "index_enseignants_excel", sizeof ("index_enseignants_excel")-1)) {
		index_enseignants_excel (0, user_passwd, sock);
		return;
	}
	if (!strncmp (cmd, "index_enseignants", sizeof "index_enseignants" -1)) {
		index_enseignants (user_passwd);
		return;
	}
#if 1
	if (!strncmp (cmd, "liste_enseignants_cours", sizeof "liste_enseignants_cours" -1)) {
		liste_enseignants_cours (cmd+sizeof "liste_enseignants_cours"-1, user_passwd, sock);
		return;
	}
#endif
	if (!strncmp (cmd, "detail_enseignants", sizeof "detail_enseignants" -1)) {
		detail_enseignants (user_passwd, sock);
		return;
	}
	if (!strcmp (cmd, "index_enseignements")) {
		index_enseignements (user_passwd, sock);
		return;
	}
	if (!strncmp (cmd, "cours_", 6)) {
		saisie_cours (cmd+6, user_passwd, sock);
		return;
	}
	if (!strncmp (cmd, "groupe_", 7)) {
		index_group (cmd+7, user_passwd, sock);
		return;
	}
	if (!strncmp (cmd, "service_", 8)) {
		service_enseignant (cmd+8, user_passwd, sock);
		return;
	}
#if WITH_CHG_PASSWD
	if (!strncmp (cmd, "passwd_", 7)) {
		change_passwd (cmd+7, user_passwd, sock);
		return;
	}
#endif
	if (!strncmp (cmd, "filiere_", 8)) {
		detail_filiere (cmd+8, user_passwd, sock);
		return;
	}
	if (!strncmp (cmd, "trous_", 6)) {
		trous_filiere (cmd+6, user_passwd, sock);
		return;
	}
	if (!strncmp (cmd, "total_trous", 11)) {
		trous_total (user_passwd, sock);
		return;
	}
	if (!strncmp (cmd, "bosses_", 7)) {
		bosses_filiere (cmd+7, user_passwd, sock);
		return;
	}
	if (!strncmp (cmd, "total_bosses", 12)) {
		bosses_total (user_passwd, sock);
		return;
	}
	if (!strncmp (cmd, "seconds_choix", 13)) {
		seconds_choix (user_passwd, sock);
		return;
	}
	if (!strncmp (cmd, "maquette_excel", 14)) {
		maquette_excel (user_passwd, sock);
		return;
	}
#ifdef STAT_ET_COMP
	if (!strncmp (cmd, "etat_stat_ext_", 14)) {
		service_stat_comp_ext (cmd+14, user_passwd, sock);
		return;
	}
#endif
	if (!strncmp (cmd, "etat_stat_", 10)) {
#ifdef STAT_ET_COMP
		service_stat_comp (cmd+10, user_passwd, sock);
#else
		etat_statutaire (cmd+10, user_passwd, sock);
#endif
		return;
	}
	if (!strncmp (cmd, "etat_prev_", 10)) {
		etat_prev (cmd+10, user_passwd, sock);
		return;
	}
	if (!strncmp (cmd, "etat_hybride_", 13)) {
		etat_hybride (cmd+13, user_passwd, sock);
		return;
	}
	if (!strncmp (cmd, "att_service_", 12)) {
		attestation_service_fait (cmd+12, user_passwd, sock);
		return;
	}
	if (!strncmp (cmd, "engagement_", 11)) {
		fiche_engagement (cmd+11, user_passwd, sock);
		return;
	}
	if (!strncmp (cmd, "enseignants_", 12)) {
		static char	_fil [32];
		static const char* av [] = {
			_fil, "H=0", "P=0", 0
		};

		(void) sprintf (_fil, "F=%s", cmd+12);
		enseignants_filiere ((char**)av, user_passwd, sock);
		return;
	}
	if (!strncmp (cmd, "detail_excel", sizeof ("detail_excel")-1)) {
		detail_excel (0, user_passwd, sock);
		return;
	}
	if (!strncmp (cmd, "potentiel_efficace", sizeof ("potentiel_efficace")-1)) {
		potentiel_efficace (0, user_passwd, sock);
		return;
	}
	if (!strncmp (cmd, "mail_ens_UE_", sizeof ("mail_ens_UE_")-1)) {
		mail_enseignants_UE (cmd+sizeof ("mail_ens_UE_")-1, user_passwd, sock);
		return;
	}
	if (!strncmp (cmd, "mail_ens_GR_", sizeof ("mail_ens_GR_")-1)) {
		mail_enseignants_GR (cmd+sizeof ("mail_ens_GR_")-1, user_passwd, sock);
		return;
	}
	// rien n'a t reconnu alors...
	_outp_ += sprintf (_outp_, "%s", CommandeInconnue);
}

/*
 *	Excute une commande de type 'post'
 */

const char* exec_post (char* cmd, char** av, char* user_passwd, FILE* sock)
{
	const char*	mime_type = "text/html";
	time_t	now = time (0);
	struct tm*	t;

	t = gmtime (&now);
	zlog ("%30.30s: %-29.29s => %02d/%02d - %02d:%02d\n",
			Host, cmd,
			t -> tm_mday, t -> tm_mon+1,
			(t -> tm_mon < 3 || t -> tm_mon > 9)
				? t -> tm_hour+1
				: t -> tm_hour+2,
			t -> tm_min);

	if (!strcmp (cmd, "add_return"))
		add_service (av, user_passwd, sock);
	else if (!strcmp (cmd, "add_request"))
		ajoute_cours (av, user_passwd, sock);
	else if (!strcmp (cmd, "remove_request"))
		retire_cours (av, user_passwd, sock);
	else if (!strcmp (cmd, "edit_mode"))
		set_edit_mode (av, user_passwd, sock);
	else if (!strcmp (cmd, "stop"))
		exit_server (av, user_passwd, sock);
	else if (!strcmp (cmd, "restart"))
		restart_server (av, user_passwd, sock);
	else if (!strcmp (cmd, "dump"))
		dump_server (av, user_passwd, sock);
	else if (!strcmp (cmd, "mandator"))
		mandatorize (av, user_passwd, sock);
	else if (!strcmp (cmd, "director"))
		directorize (av, user_passwd, sock);
	else if (!strcmp (cmd, "admin"))
		admin (user_passwd, sock);
	else if (!strcmp (cmd, "select"))
		enseignants_filiere (av, user_passwd, sock);
#if WITH_CHG_PASSWD
	else if (!strcmp (cmd, "chg_passwd"))
		chg_passwd (av, user_passwd, sock);
#endif
	else if (!strcmp (cmd, "select_mandatement"))
		select_mandatement (av, user_passwd, sock);
	else if (!strcmp (cmd, "mandatement_return"))
		add_mandatement (av, user_passwd, sock);
	else if (!strcmp (cmd, "remove_mandatement_return"))
		remove_mandatement (av, user_passwd, sock);
	else if (!strcmp (cmd, "mandatement_retour"))
		retour_mandatement (av, user_passwd, sock);
	else if (!strcmp (cmd, "mandatement_request"))
		modif_mandatement (av, user_passwd, sock);
	else if (!strcmp (cmd, "tous_enseignants"))
		set_tous (av, user_passwd, sock);
	else if (!strcmp (cmd, "inscription_restreinte"))
		inscription_restreinte (av, user_passwd, sock);
	else if (!strcmp (cmd, "recap_mandatements"))
		recap_mandatements (av, user_passwd, sock);
	else if (!strcmp (cmd, "total_filiere"))
		affiche_total_filieres (av, user_passwd, sock);
	else if (!strcmp (cmd, "att_tous_ens")) {
		attestation_tous_enseignants (av, user_passwd, sock);
		mime_type = "application/postscript";
	}
	else if (!strcmp (cmd, "prev_tous_ens")) {
		previsionnel_tous_enseignants (av, user_passwd, sock);
		mime_type = "application/postscript";
	}
	else if (!strcmp (cmd, "hybride_tous_ens")) {
		hybride_tous_enseignants (av, user_passwd, sock);
		mime_type = "application/postscript";
	}
	else if (!strcmp (cmd, "reste_a_mandater"))
		reste_a_mandater (av, user_passwd, sock);
	else
		_outp_ += sprintf (_outp_, "%s", CommandeInconnue);

	return mime_type;
}

/*
 *	Excute une commande de type 'post transfer'
 */

void exec_post_transfer (char* cmd, char** av, char* file_buffer, int file_length, char* user_passwd, FILE* sock)
{
	time_t	now = time (0);
	zlog ("%20.20s: %-29.29s => %s", Host, cmd, ctime (&now));

	if (file_buffer != 0)
		free (file_buffer);
	_outp_ += sprintf (_outp_, "<html><body><h3>File transfer ?? </h3></body></html>\n");
}
