/*****************************************************************

  Copyright (c) 1999 - 2003 by Patrick Amar - pa@lri.fr

    http://www.lri.fr/~pa

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

#include <stdio.h>
#include "Lexer.h"		// doit tre avant Employeur.h
#include "Enseignant.h"	// Employeur.h est inclu dedans.
#include "display.h"

/*
 *	Initialise la liste des enseignants, des employeurs, des dpartements, et des taux horaires.
 */

#define T_PR			(T_VOID+1)
#define T_MC			(T_VOID+2)
#define T_MON			(T_VOID+3)
#define T_AMN			(T_VOID+4)
#define T_ATER			(T_VOID+5)
#define T_DEMI_ATER		(T_VOID+6)

#define T_PRAG			(T_VOID+7)
#define T_PAST			(T_VOID+8)
#define T_CONTR			(T_VOID+9)
#define T_PRCE			(T_VOID+10)
#define T_CR			(T_VOID+11)
#define T_DR			(T_VOID+12)
#define T_IR			(T_VOID+13)
#define T_IE			(T_VOID+14)
#define T_TECH			(T_VOID+15)
#define T_VAC			(T_VOID+16)
#define T_ATE			(T_VOID+17)
#define T_PCA			(T_VOID+18)

#define T_LAST_ENS		T_PCA

#define T_DEUG			(T_LAST_ENS+1)
#define T_NFI			(T_LAST_ENS+2)
#define T_IUP			(T_LAST_ENS+3)
#define T_DEUG_NFI		(T_LAST_ENS+4)

#define T_LAST_AFF		T_DEUG_NFI

#define T_ENSEIGNANT	(T_LAST_AFF+1)
#define T_EMPLOYEUR		(T_LAST_AFF+2)
#define T_DEPARTEMENT	(T_LAST_AFF+3)
#define T_TAUX			(T_LAST_AFF+4)
#define T_PRIVE			(T_LAST_AFF+5)
#define T_FONCT			(T_LAST_AFF+6)
#define T_UNIV			(T_LAST_AFF+7)
#define T_BOURSIER		(T_LAST_AFF+8)
#define T_PROVENANCE	(T_LAST_AFF+9)
#define T_HOME			(T_LAST_AFF+10)

#define T_LAST_EMPL		T_HOME

#define T_CELIBATAIRE	(T_LAST_EMPL+1)
#define T_MARIE			(T_LAST_EMPL+2)
#define T_VEUF			(T_LAST_EMPL+3)
#define T_DIVORCE		(T_LAST_EMPL+4)
#define T_YEAR			(T_LAST_EMPL+5)
#define T_DISPONIBILITE	(T_LAST_EMPL+6)
#define T_ANNEE			(T_LAST_EMPL+7)
#define T_COLORS		(T_LAST_EMPL+8)

#define T_LAST_SITFAM	T_COLORS

#define T_LBRA			(T_LAST_SITFAM+1)
#define T_RBRA			(T_LAST_SITFAM+2)
#define T_COMMA			(T_LAST_SITFAM+3)
#define T_SEMI			(T_LAST_SITFAM+4)
#define T_COLON			(T_LAST_SITFAM+5)
#define T_SLASH			(T_LAST_SITFAM+6)

#define T_LAST_TOK		T_SLASH



#define KEYWORDS "PR", "MC", "MON", "AMN", \
				 "ATER", "DEMI_ATER", "PRAG", "PAST", "CONTR", "PRCE", \
				 "CR", "DR", "IR", "IE", "TECH", "VAC", "ATE", "PCA", \
				 "DEUG", "NFI", "IUP", "DEUG-NFI", \
				 "Enseignant", "Employeur", "Departement", "Taux", \
				 "Prive", "Fonctionnaire", "Universite", "Boursier", "Provenance", \
				 "le_notre", "celibataire", "marie", "veuf", "divorce", \
				 "Year", "Disponibilite", "Annee", "Couleurs"

static const char*	keywords [] = { KEYWORDS, 0 };

static const char*	err [] = {	"a character string", "the end of file", "a number", "????",
							KEYWORDS,
							"left brace", "right brace", "comma", "semicolon", "slash"
						 };
#undef KEYWORDS

static Special	ctype [] = {
	{ '{',	T_LBRA },
	{ '}',	T_RBRA },
	{ ',' ,	T_COMMA },
	{ '/' ,	T_SLASH },
	{ ';' ,	T_SEMI },
	{ ':' ,	T_COLON },
	{ 0, 0 }
};

const char* page_colors [MAX_COLORS] = {
//	jaune		orange		violet		violet vif
	"#f9dc76",	"#f8bb51",	"#8579c5",	"#8000ff",
//	jaune clair		violet clair
	"#ffec86",	"#9589d5",
//	jaune vif		orange vif
	"#f0f000",	"#ffc000",
//	blanc		noir
	"#ffffff",		"#000000"
};


/*
 *	Rcupre les infos personnelles d'un enseignant.
 */

Perso::Perso (Lexer& lx)
{
	Lexeme	l;

	(void) lx.expect (T_LBRA);
	l = lx.expect (T_STRING); _njf = l._string;	// nom de jeune fille.
	l = lx.expect (T_NUMBER); _jdn = (int) (long) l._string;	// jour de naissance.
	if (_jdn > 31) {
		lx.unlex (l);
		(void) lx.expect (0, "a day number <= 31");
	}
	(void) lx.expect (T_SLASH);
	l = lx.expect (T_NUMBER); _mdn = (int) (long) l._string;	// mois de naissance.
	if (_mdn > 12) {
		lx.unlex (l);
		(void) lx.expect (0, "a month number <= 12");
	}
	(void) lx.expect (T_SLASH);
	l = lx.expect (T_NUMBER); _adn = (int) (long) l._string;	// anne de naissance.
	if (_adn < 1930) {
		lx.unlex (l);
		(void) lx.expect (0, "a birth year >= 1930");
	}
	l = lx.expect (T_STRING); _ldn = l._string;	// lieu de naissance.
	// la situation familiale.
	l = lx.lex ();
	switch (l._type) {
		case T_CELIBATAIRE:	_sf = CELIBATAIRE; break;
		case T_MARIE:		_sf = MARIE; break;
		case T_VEUF:		_sf = VEUF; break;
		case T_DIVORCE:		_sf = DIVORCE; break;
		default:
			lx.unlex (l);
			(void) lx.expect (0, "'celibataire', 'marie', 'veuf', 'divorce'");
	}
	l = lx.expect (T_STRING); _secu = l._string;	// numro de scurit sociale.
	// vrifier sa longueur.
	if (strlen (_secu) != 13) {
		lx.unlex (l);
		(void) lx.expect (0, " a13 digits number");
	}
	// cohrence entre le numro de scu, la situation familiale, et le nom de jeune fille.
	switch (_secu [0]) {
		case '1':
			if (_njf [0] != 0) {
				lx.unlex (l);
				lx.warning ("un homme n'a pas de nom de jeune fille");
				exit (1);
			}
			break;

		case '2':
			if (_sf == MARIE && _njf [0] == 0) {
				lx.unlex (l);
				lx.warning ("un femme marie a un nom de jeune fille");
				exit (1);
			}
			if (_sf == CELIBATAIRE && _njf [0] != 0) {
				lx.unlex (l);
				lx.warning ("un femme clibataire n'a pas nom de jeune fille");
				exit (1);
			}
			break;

		default:
				lx.unlex (l);
				(void) lx.expect (0, "a number beginning with 1 or 2");
	}
	if ((_adn % 100) != ((10 * (_secu [1] - '0')) + (_secu [2] - '0'))) {
		lx.unlex (l);
		lx.warning ("incohrence entre l'anne de naissance et le numro de scurit sociale.");
		exit (1);
	}
	if (_mdn != ((10 * (_secu [3] - '0')) + (_secu [4] - '0'))) {
		lx.unlex (l);
		lx.warning ("incohrence entre le mois de naissance et le numro de scurit sociale.");
		exit (1);
	}
	l = lx.expect (T_STRING); _clef = l._string;	// clef qui va avec.
	if (strlen (_clef) != 2) {
		lx.unlex (l);
		(void) lx.expect (0, "a 2 digits number");
	}
	// les trois lignes d'adresse.
	_ad1 = (char*)""; _ad2= (char*)""; _ad3 = (char*)"";		// vides a priori.
	(void) lx.expect (T_LBRA);
	l = lx.lex ();
	switch (l._type) {
		case T_STRING:	_ad1 = l._string;		// premire lign de l'adresse.
						break;
		case T_RBRA:	goto phone_number;
		default:		lx.unlex (l);
						(void) lx.expect (0, "first line of address, or right brace");
	}
	l = lx.lex ();
	switch (l._type) {
		case T_STRING:	_ad2 = l._string;		// deuxime ligne de l'adresse.
						break;
		case T_RBRA:	goto phone_number;
		default:		lx.unlex (l);
						(void) lx.expect (0, "second line of address, or right brace");
	}
	l = lx.lex ();
	switch (l._type) {
		case T_STRING:	_ad3 = l._string;		// troisime ligne de l'adresse.
						(void) lx.expect (T_RBRA);
		case T_RBRA:	goto phone_number;
		default:		lx.unlex (l);
						(void) lx.expect (0, "third line of address, or right brace");
	}
phone_number:
	l = lx.expect (T_STRING); _ntel = l._string;	// numro de tlphone perso.
	l = lx.expect (T_STRING); _ntelpro = l._string;	// numro de tlphone pro.
	// traitement pay par le tpg des Yvelines ?
	l = lx.lex ();
	switch (l._type) {
		case T_STRING:
			if (!strcmp (l._string, "tpg_yvelines")) {
				_tpg_yv = true;
				(void) lx.expect (T_RBRA);
			} else {
				lx.unlex (l);
				lx.expect (0, "'tpg_yvelines' or empty string");
			}
			break;

		case T_RBRA:
			break;

		default:
			lx.unlex (l);
			(void) lx.expect (T_RBRA);
	}
#ifdef DEBUG
	fprintf (stderr, "njf = %s\n", _njf);
	fprintf (stderr, "ddn = %d / %d / %d\n", _jdn, _mdn, _adn);
	fprintf (stderr, "lieu = %s\n", _ldn);
	fprintf (stderr, "sitfam = %d\n", _sf);
	fprintf (stderr, "secu = %s %s\n", _secu, _clef);
	fprintf (stderr, "%s\n%s\n%s\n", _ad1, _ad2, _ad3);
	fprintf (stderr, "tl perso = %s\n", _ntel);
	fprintf (stderr, "tl pro = %s\n", _ntelpro);
	fprintf (stderr, "tpg yvelines = %s\n", _tpg_yv ? "true" : "false");
#endif
}

/*
 *	Lit et analyse le fichier des infos personnelles.
 */

void Perso::init (const char* file)
{
	Enseignant*	prof;
	Lexer		lx (file, keywords, err, ctype);
	Lexeme		l;

	for (;;) {
		l = lx.lex ();
		switch (l._type) {
			case T_STRING:
				// rcuprer le nom de l'enseignant.
				prof = Enseignant::name2prof (l._string);
				if (prof == 0) {
					lx.unlex (l);
					(void) lx.expect (0, "Enseignant inconnu");
				}
				// pour y coller les infos personnelles.
				prof -> _perso = new Perso (lx);
				continue;

			default:
				lx.unlex (l);
				(void) lx.expect (0, "'nom d'enseignant'");
				return;

			case T_EOF:
				return;
		}
	}
}

/*
 *	Initialise un enseignant.
 *	
 *	{ "Pas-Cette Anne" --- PCA }
 *	{ "Jean Dupond" UPS Informatique MC }
 *	{ "Jean Dupond" UPS Informatique MC jd@ici.fr }
 *	{ "Jean Durand" CEA VAC "ingnieur de recherches" jd@cea.fr }
 *	{ "Jean Dupond" UPS Informatique MC jd@ici.fr 192 Present }
 *	{ "Jean Dupuis" UPS Informatique MON { DEUG 1 } jd@ici.fr 64 Present }
 */

static Enseignant* p_enseignant (Lexer& lx)
{
	Lexeme			l;
	char*			nom;
	int				cnu = DEF_CNU;
	Employeur*		empl;
	Departement*	dept = 0;			// non universitaire.
	Categorie		categorie;
	int				affect = DEUG;
	int				annee = 1;
	char*			email = (char*)"";
	char*			poste = (char*)"";
	int				service = 192;
	int				d = 0;				// prsent.

	// son nom.
	(void) lx.expect (T_LBRA);
	l = lx.expect (T_STRING); nom = l._string;

	// la section CNU (27 par dfaut).
	l = lx.lex ();
	if (l._type == T_NUMBER) {
		cnu = (int) (long) l._string;
		l = lx.lex ();
	}
	// l'employeur.
	if (l._type != T_STRING ||
		0 == (empl = Employeur::name2empl (l._string, 0)))
	{
		lx.unlex (l);
		(void) lx.expect (0, "'employeur valide'");
		return 0;
	}

	// le dpartement est facultatif.
	if (empl -> _type != PRIVE) {
		l = lx.lex ();
		if (l._type != T_STRING ||
			0 == (dept = Departement::name2dept(l._string, 0)))
		{
			lx.unlex (l);
		}
	}
	// le type de poste.
	l = lx.lex ();
	if (T_PR > l._type || l._type > T_LAST_ENS) {
		lx.unlex (l);
		(void) lx.expect (0, "'PR', 'MC', 'AS', 'MON', 'AMN', 'ATER', 'ATE', 'PRAG', "
							 "'PAST', 'CONTR', 'CR', 'DR', 'IR', 'IE', 'TECH', 'VAC', 'PCA'");
		return 0;
	}
	categorie = (Categorie) (l._type - T_PR);	// ca se suit exprs!
	// pour les vacataires extrieurs on indique le type de leur poste dans leur entreprise.
	if (categorie == VAC) {
		l = lx.expect (T_STRING);
		poste = l._string;
	}
	else if (categorie == MON || categorie == AMN) {
		// pour les moniteurs.
		(void) lx.expect (T_LBRA);
		// il faut l'affectation.
		l = lx.lex ();
		switch (l._type) {
			case T_DEUG:		affect = DEUG; break;
			case T_NFI:			affect = NFI; break;
			case T_IUP:			affect = IUP; break;
			case T_DEUG_NFI:	affect = DEUG+NFI; break;
			default:
				lx.unlex (l);
				(void) lx.expect (0, "'DEUG', 'NFI', 'IUP', 'DEUG-NFI'");
				return 0;
		}
		// et l'anne.
		l = lx.expect (T_NUMBER);
		annee = (int) (long) l._string;
		(void) lx.expect (T_RBRA);
	}
	// Pas cette anne.
	else if (categorie == PCA) {
		service = 0;
		switch (l = lx.lex (), l._type) {
			case T_RBRA:
				return new Enseignant (empl, dept, categorie, nom, email, service, d, affect, annee, poste, cnu);

			default:
				lx.unlex (l);
				(void) lx.expect (0, "a right brace (not this year...)");
				return 0;
		}
	}
	// avec email.
	switch (l = lx.lex (), l._type) {
		case T_RBRA:
						return new Enseignant (empl, dept, categorie, nom, email, service, d, affect, annee, poste, cnu);
		case T_STRING:	email = l._string;
						break;

		default:		lx.unlex (l);
						(void) lx.expect (0, "an email address");
						return 0;
	}
	// avec service
	switch (l = lx.lex (), l._type) {
		case T_RBRA:
						return new Enseignant (empl, dept, categorie, nom, email, service, d, affect, annee, poste, cnu);
		case T_NUMBER:	service = (int) (long) l._string;
						break;

		default:		lx.unlex (l);
						(void) lx.expect (T_NUMBER);
						return 0;
	}
	// avec disponibilit.
	switch (l = lx.lex (), l._type) {
		case T_RBRA:
						return new Enseignant (empl, dept, categorie, nom, email, service, d, affect, annee, poste, cnu);

		case T_STRING:
						for (d = 0; Enseignant::disponibilite [d]._motclef != 0; ++d)
							if (!strcmp (Enseignant::disponibilite [d]._motclef, l._string))
								break;
						if (Enseignant::disponibilite [d]._motclef == 0)
							goto def;

						(void) lx.expect (T_RBRA);
						return new Enseignant (empl, dept, categorie, nom, email, service, d, affect, annee, poste, cnu);

		default: def:	lx.unlex (l);
						(void) lx.expect (0, "'Present', 'Detachement', 'Sabbatique', 'Maternite', 'Maladie', 'Echange', 'Mission', 'Partiel'");
						return 0;
	}
	return 0;
}

/*
 *	Initialise la liste des employeurs et des dpartements.
 *
 *	Taux {
 *		"1/9/1999" 244,18
 *		"1/1/2000" 246,13
 *	}
 *	Year 2004/2005;
 *	Disponibilite {
 *		"Detachement", "(dtach)", "En dtachement",
 *	}
 *	Employeur {
 *		"SYSECA" "prive" "SYSECA, 38 rue des bleuets",
 *		"UPS" "Universite" "Universit PARIS SUD",
 *		"CNRS" "Fonctionnaire" "CNRS"
 *	}
 *	Departement {
 *		"Informatique",
 *		"IUT Chimie"
 *	}
 *	Enseignant {
 *		{ "Jean Dupond" UPS Informatique MC jd@ici.fr 192 Present }
 *		...
 *	}
 */

void Employeur::init (const char* file)
{
	Lexer		lx(file, keywords, err, ctype);
	Lexeme		l;
	char*		nom;
	const char*		ad1;
	const char*		ad2;
	const char*		ad3;
	Statut		statut;
	bool		a_nous;
	Departement*	dept;
	Enseignant*	prof;
	Enseignant*	tous [MAX_PROF];
	int			n = 0;
	bool		employeurs_ok = false;
	bool		enseignants_ok = false;
	bool		departements_ok = false;
	bool		dispo_ok = false;

	for (;;) switch (l = lx.lex (), l._type) {
		default:
			lx.unlex (l);
			(void) lx.expect (0, "'Provenance', Employeur', 'Departement', 'Taux', 'Annee', 'Couleurs', 'Disponibilite' or 'Enseignant'");
			return;

		case T_PROVENANCE:
			(void) lx.expect (T_LBRA);
			for (int i = (int)INCONNU; i <= (int)VAC_PUB; ++i) {
				switch (l = lx.lex (), l._type) {
				case T_STRING:
					Enseignant::genre2str [i] = l._string;
					if (i < (int)VAC_PUB) {
						(void) lx.expect (T_COMMA);
						continue;
					} else {
						(void) lx.expect (T_RBRA);
						break;
					}

				case T_COMMA:
					continue;	// rien => valeur par dfaut.

				case T_RBRA:
					break;		// le reste est  la valeur par dfaut.

				default:
					(void) lx.expect (0, "Syntax: Provenance { \"inconnu\", \"titulaire\", ..., \"vac. public\" }");
					return;
				}
				break;
			}
			continue;

		case T_EOF:
			if (Taux::_year1 * Taux::_year2 == 0)
				(void) lx.expect (0, "Must specify the year");
			return;

		case T_YEAR:
		case T_ANNEE:
			if (Taux::_year1 * Taux::_year2 != 0) {
				(void) lx.expect (0, "Must specify exactly one 'Year'");
				return;
			}
			Taux::_year1 = (int) (long) (lx.expect (T_NUMBER)._string);
			(void) lx.expect (T_SLASH);
			Taux::_year2 = (int) (long) (lx.expect (T_NUMBER)._string);
			l = lx.lex ();
			if (l._type == T_COLON) {
				Taux::_position = (int) (long) (lx.expect (T_NUMBER)._string);
				Taux::_url_format = lx.expect (T_STRING)._string;
			} else
				lx.unlex (l);
			(void) lx.expect (T_SEMI);
			if (Taux::_year2 != Taux::_year1 + 1) {
				(void) lx.expect (0, "Syntax: year 2003/2004");
				return;
			}
			continue;

		case T_COLORS:
			(void) lx.expect (T_LBRA);
			for (int i = 0; i < MAX_COLORS; ++i) {
				switch (l = lx.lex (), l._type) {
				case T_STRING:
					page_colors [i] = l._string;
					(void) lx.expect (T_COMMA);
					continue;

				case T_COMMA:
					continue;	// rien => valeur par dfaut.

				case T_RBRA:
					break;		// le reste est  la valeur par dfaut.

				default:
					(void) lx.expect (0, "Syntax: Couleurs { \"yellow\", \"#ffc000\",, \"black\" }");
					return;
				}
				break;
			}
			continue;

		case T_ENSEIGNANT:
			if (enseignants_ok) {
				(void) lx.expect (0, "Must specify exactly one 'Enseignant' list");
				return;
			}
			if (!departements_ok || !employeurs_ok || !dispo_ok) {
				(void) lx.expect (0, "Must specify 'Employeur', 'Disponibilite' and 'Departement' before 'Enseignant'");
				return;
			}
			(void) lx.expect (T_LBRA);
			for (;;) {
				tous [n++] = prof = p_enseignant (lx);
				switch (l = lx.lex (), l._type) {
					case T_LBRA:
						lx.unlex (l);
						continue;

					case T_RBRA:
						break;

					default:
						lx.unlex (l);
						(void) lx.expect (0, "left or right brace");
						return;
				}
				break;
			}
			// constituer la liste
			Enseignant::_head = tous [0];
			for (int i = 1; i < n; ++i) {
				tous [i] -> _precedent = tous [i-1];
				tous [i] -> _suivant = tous [i+1];
			}
			// replier la liste.
			tous [n-1] -> _suivant = tous [0];		// le suivant du dernier est le premier.
			tous [0] -> _precedent = tous [n-1];	// le prcdent du premier est le dernier.
			tous [0] -> _suivant = tous [1];		// le suivant du premier est le deuxime!

			enseignants_ok = true;		// on a bien vu les enseignants.
			zlog ("%d enseignants.\n", n);
			continue;

		case T_DISPONIBILITE:
			(void) lx.expect (T_LBRA);
			for (int i = 0;;) {
				Enseignant::disponibilite [i]._motclef = (lx.expect (T_STRING))._string;
				Enseignant::disponibilite [i]._intitule = (lx.expect (T_STRING))._string;
				Enseignant::disponibilite [i]._intitule_ps = (lx.expect (T_STRING))._string;
				++i;
				l = lx.lex ();
				if (l._type == T_RBRA) {
					Enseignant::disponibilite [i]._motclef = 0;
					break;
				}
				lx.unlex (l);
			}
			dispo_ok = true;
			continue;

		case T_EMPLOYEUR:
			(void) lx.expect (T_LBRA);
			for (;;) {
				l = lx.expect (T_STRING); nom = l._string;
				l = lx.lex ();
				if (l._type < T_PRIVE || l._type > T_BOURSIER) {
					lx.unlex (l);
					(void) lx.expect (0, "'Prive', 'Fonctionnaire', 'Universite, 'Boursier'");
					return;
				}
				statut = (Statut) (l._type - T_PRIVE);
				// les trois lignes d'adresse.
				ad1 = ""; ad2= ""; ad3 = "";		// vides a priori.
				(void) lx.expect (T_LBRA);
				switch (l = lx.lex (), l._type) {
					case T_STRING:	ad1 = l._string;		// premire lign de l'adresse.
									break;
					case T_RBRA:	goto ok;
					default:		lx.unlex (l);
									(void) lx.expect (0, "first line of address, or right brace");
				}
				switch (l = lx.lex (), l._type) {
					case T_STRING:	ad2 = l._string;		// deuxime ligne de l'adresse.
									break;
					case T_RBRA:	goto ok;
					default:		lx.unlex (l);
									(void) lx.expect (0, "second line of address, or right brace");
				}
				switch (l = lx.lex (), l._type) {
					case T_STRING:	ad3 = l._string;		// troisime ligne de l'adresse.
									(void) lx.expect (T_RBRA);
					case T_RBRA:	goto ok;
					default:		lx.unlex (l);
									(void) lx.expect (0, "third line of address, or right brace");
				}
			ok:
				// crer le nouvel employeur.
				(void) new Employeur (nom, ad1, ad2, ad3, statut);
				// passer  la suite.
				switch (l = lx.lex (), l._type) {
					case T_STRING:		// au suivant.
						lx.unlex (l);
						continue;

					case T_RBRA:		// c'est termin.
						break;

					default:
						lx.unlex (l);
						(void) lx.expect (0, "right brace or character string");
						return;
				}
				break;
			}
			employeurs_ok = true;	// on a vu une liste d'employeurs.
			continue;

		case T_DEPARTEMENT:
			(void) lx.expect (T_LBRA);
			for (;;) {
				l = lx.expect (T_STRING); nom = l._string;
				a_nous = false;
				if ((l = lx.lex (), l._type) == T_HOME) {
					a_nous = true;		// c'est un de nos dpartements.
				} else
					lx.unlex (l);
				// crer le nouveau dpartement.
				dept = new Departement (nom, a_nous);
				// vp enseignement.
				if ((l = lx.lex (), l._type) == T_STRING) {
					dept -> _vp_enseignement = l._string;
					l = lx.lex ();
				}
				switch (l._type) {
					case T_COMMA:	continue;
					case T_RBRA:	break;
					default:
						lx.unlex (l);
						(void) lx.expect (0, "comma or right brace");
						return;
				}
				break;
			}
			departements_ok = true;		// on a vu une liste de dpartements.
			continue;

		case T_TAUX:
			(void) lx.expect (T_LBRA);
			for (;;) {
				time_t	date;
				int		e, f;

				l = lx.expect (T_STRING); date = String2Time (l._string);
				if (date == -1) {
					lx.unlex (l);
					(void) lx.expect (0, "date invalide");
					return;
				}
				l = lx.expect (T_NUMBER); e = (int) (long) l._string;
				l = lx.lex ();
				if (l._type == T_COMMA) {
					l = lx.expect (T_NUMBER);
					f = (int) (long) l._string;
					l = lx.lex ();
				} else
					f = 0;
				// crer le nouveau taux.
				(void) new Taux ((float) (e + ((float) f) / 100), date);
				switch (l._type) {
					case T_STRING:
						lx.unlex (l);
						continue;

					case T_RBRA:
						break;

					default:
						lx.unlex (l);
						(void) lx.expect (0, "comma or right brace");
						return;
				}
				break;
			}
			continue;
	}
}

/*
 *	Les taux.
 */

Taux*	Taux::_les_taux [MAX_TAUX];
int		Taux::_year1 = 0;
int		Taux::_year2 = 0;
int		Taux::_position = 0;
char*	Taux::_url_format = (char*) "http://voeux%d-%d.lri.fr";

Taux::Taux (float t, time_t date)
{
	_taux = t;
	_date = date;
	// insrer le nouveau taux dans la table.
	for (int i = 0; i < MAX_TAUX; ++i)
		if (_les_taux [i] == 0) {
			_les_taux [i] = this;
			return;
		}
	// plus de place dans la table.
}

/*
 *	Recherche un taux  partir d'une date.
 */

float Taux::get (time_t date)
{
	if (date < _les_taux [0] -> _date)
		return _les_taux [0] -> _taux;

	for (int i = 1; i < MAX_TAUX; ++i) {
		if (_les_taux [i] == 0 ||
			(_les_taux [i-1] -> _date <= date && date < _les_taux [i] -> _date))
		{
			return _les_taux [i-1] -> _taux;
		}
	}
	return _les_taux [MAX_TAUX-1] -> _taux;	// taux le plus rcent utilis.
}

/*
 *	Les dpartements.
 */

Departement*	Departement::_departements [MAX_DEPT];

Departement::Departement (char* nom, bool a_nous)
{
	_nom = nom;
	_a_nous = a_nous;
	_vp_enseignement = (char*) "";
	(void) name2dept (nom, this);	// l'insrer dans la table.
}

/*
 *	Indique si un dpartement est un IUT, ou POLYTECH ou...
 */

bool Departement::search (const char* nom)
{
	int	len = strlen (nom);
	// se borne  chercher la chaine dans le nom.
	for (char* p = _nom; *p != 0; ++p)
		if (strncmp (p, nom, len) == 0)
			return true;
	return false;
}

/*
 *	Recherche (squentiellement) un dpartement par son nom.
 */

Departement* Departement::name2dept (char* name, Departement* dept)
{
	for (int i = 0; i < MAX_DEPT; ++i) {
		if (_departements [i] == 0) {
			if (dept == 0)
				return 0;	// pas trouv.
			// on le met dans la table.
			_departements [i] = dept;
			return dept;
		}
		if (!strcmp (_departements [i] -> _nom, name))
			return _departements [i];
	}
	return 0;	/* plus de place dans la table */
}

/*
 *	Les employeurs.
 */

Employeur*	Employeur::_employeurs [MAX_EMPL];

Employeur::Employeur (char* nom, const char* ad1, const char* ad2, const char* ad3, Statut type)
{
	_nom = nom;
	_ad1 = ad1;
	_ad2 = ad2;
	_ad3 = ad3;
	_type = type;
	(void) name2empl (nom, this);	// l'insrer dans la table.
}

/*
 *	Recherche un employeur par son nom.
 */

Employeur* Employeur::name2empl (char* name, Employeur* empl)
{
	int	h = Enseignant::hash (name, MAX_EMPL);

	for (int i = MAX_EMPL; --i >= 0;) {
		if (_employeurs [h] == 0) {
			if (empl == 0)
				return 0;	/* pas trouv */
			/* on le met dans la table */
			_employeurs [h] = empl;
			return empl;
		}
		if (!strcmp (_employeurs [h] -> _nom, name))
			return _employeurs [h];

		h = (h + 41) % MAX_EMPL;	/* collision */
	}
	return 0;	/* plus de place dans la table */
}
