/*****************************************************************

  Copyright (c) 1999 - 2007 by Patrick Amar - pa@lri.fr

    http://www.lri.fr/~pa

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

#ifndef EMPLOYEUR_H
#define EMPLOYEUR_H

// nombre max de dpartements.
#define MAX_DEPT	32
// nombre max d'employeurs.
#define MAX_EMPL	256
// nombre max de taux.
#define MAX_TAUX	32

#define DEF_CNU		27	// section CNU par dfaut.

//
//	Les taux horaires d'une heure complmentaire de TD.
//

class Taux {
private:
	static Taux*	_les_taux [MAX_TAUX];
public:
	static int		_year1, _year2;	// anne scolaire.
	static int		_position;		// 0: en bas, 1: au milieu, 2: en haut
	static char*	_url_format;	// URL d'un serveur de voeux.

	float	_taux;		// taux.
	time_t	_date;		// date de prise d'effet.

	Taux (float, time_t);
	static float get (time_t);	// retoure le taux associ  la date.
};

/*
 *	L'employeur de l'enseignant.
 */

class Departement {
public:
	char*	_nom;		// son nom.
	bool	_a_nous;	// est-il  nous?
	char*	_vp_enseignement;	// le VP enseignement pour les fiches.

public:
	Departement (char*, bool);
	bool search (const char*);
	bool iut (void) { return search ("IUT"); }

	static Departement*	_departements [MAX_DEPT];
	static Departement*	name2dept (char*, Departement* =0);
};

enum Statut { PRIVE, FONCTIONNAIRE, UNIVERSITE, BOURSIER };

class Employeur {
public:
	Statut	_type;	// type d'employeur.
	char*	_nom;	// son nom.
	const char*	_ad1;	// premire ligne de l'adresse.
	const char*	_ad2;	// deuxime ligne de l'adresse.
	const char*	_ad3;	// troisime ligne de l'adresse.

public:
	Employeur (char*, const char*, const char*, const char*, Statut);

	static Employeur*	_employeurs [MAX_EMPL];
	static void			init (const char*);
	static Employeur* name2empl (char*, Employeur* =0);
};

/*
 *	Les renseignements supplmentaires sur les enseignants (pour remplir la
 *	fiche d'engagement d'heures complmentaires).
 */

enum Sitfam { CELIBATAIRE, MARIE, VEUF, DIVORCE };

class Lexer;

class Perso {
public:
	char*	_njf;		// nom de jeune fille.
	int		_jdn;		// jour de naissance.
	int		_mdn;		// mois de naissance.
	int		_adn;		// anne de naissance.
	char*	_ldn;		// lieu de naissance.
	Sitfam	_sf;		// situation familiale.
	char*	_secu;		// numro de scurit sociale.
	char*	_clef;		// la clef qui va avec.
	char*	_ad1;		// premire ligne de l'adresse.
	char*	_ad2;		// deuxime ligne de l'adresse.
	char*	_ad3;		// troisime ligne de l'adresse.
	char*	_ntel;		// numro de tlphone perso.
	char*	_ntelpro;	// numro de tlphone pro.
	bool	_tpg_yv;	// traitement pay par le tpg des Yvelines.

	Perso (Lexer&);		// constructeur/analyseur.
	static void			init (const char*);
};

#endif
