/*****************************************************************

  Copyright (c) 1999 - 2014 by Patrick Amar - pa@lri.fr

    http://www.lri.fr/~pa

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <stdarg.h>
#include <time.h>
#include <string.h>

#if ZLOG
#include <zlib.h>
#endif

#include "Lexer.h"
#include "Enseignant.h"
#include "display.h"
#include "etat_statutaire_ps.h"
#if !FICHE_LATEX
#	include "fiche_engagement_ps.h"
#endif
#include "attestation_services_ps.h"
#include "attestation_hybride_ps.h"
#include "stat_comp_ps.h"

// Hack local pour diffrencier les enseignements de l'UFE de ceux de la Licence.
#define HACK_UFE	1

extern char* _outp_;

Dispo	Enseignant::disponibilite [128];

const char*	Enseignant::genre2str [] = {
	"Inconnu", "Titulaires", "Moniteurs", "A.T.E.R.", "IUT Paris-Sud", "Polytech",
	"Autres dpartements Paris-Sud", "Personnels CNRS", "Personnels INRIA", "Contractuels",
	"Vacataires secteur priv", "Vacataires secteur public"
};

const char*	Enseignant::genre2orig [] = {
	"Inconnu", "Dpartement", "Dpartement", "Dpartement", "Dpartement", "Dpartement",
	"Dpartement", "Origine", "Origine", "Dpartement", "Origine", "Origine"
};

unsigned char Enseignant::iso2ascii [] = {
	0, 1, 2, 3, 4, 5, 6, 7,
	'\b', '\t', '\n', '\r', 12, 13, 14, 15,
	16, 17, 18, 19, 20, 21, 22, 23,
	24, 25, 26, 27, 28, 29, 30, 31,
	'_', '!', '\"', '#', '$', '%', '&', '\'',	// mapper l'espace en soulign.
	'(', ')', '*', '+', ',', '-', '.', '/',
	'0', '1', '2', '3', '4', '5', '6', '7',
	'8', '9', ':', ';', '<', '=', '>', '?',
	'@', 'a', 'b', 'c', 'd', 'e', 'f', 'g',
	'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o',
	'p', 'q', 'r', 's', 't', 'u', 'v', 'w',
	'x', 'y', 'z', '[', '\\', ']', '^', '_',
	'`', 'a', 'b', 'c', 'd', 'e', 'f', 'g',
	'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o',
	'p', 'q', 'r', 's', 't', 'u', 'v', 'w',
	'x', 'y', 'z', '{', '|', '}', '~', '',
	128, 129, 130, 131, 132, 133, 134, 135,
	136, 137, 138, 139, 140, 141, 142, 143,
	144, 145, 146, 147, 148, 149, 150, 151,
	152, 153, 154, 155, 156, 157, 158, 159,
	160, 161, 163, 163, 164, 165, 166, 167,
	168, 169, 170, 171, 172, 173, 174, 175,
	176, 177, 178, 179, 180, 181, 182, 183,
	184, 185, 186, 187, 188, 189, 190, 191,
	'a', 'a', 'a', 0303, 'a', 'a', 'a', 'c',
	'e', 'e', 'e', 'e', 'i', 'i', 'i', 'i',
	'd', 'n', 'o', 'o', 'o', 'o', 'o', 'x',
	'o', 'u', 'u', 'u', 'u', 'y', 'b', 'b',
	'a', 'a', 'a', 'a', 'a', 'a', 'a', 'c',
	'e', 'e', 'e', 'e', 'i', 'i', 'i', 'i',
	'd', 'n', 'o', 'o', 'o', 'o', 'o', 'x',
	'o', 'u', 'u', 'u', 'u', 'y', 'b', 'b'
};

bool			Enseignant::_tous = true;		// remettre faux quand tout sera ok.
bool			Enseignant::_edit_mode = false;
Enseignant* 	Enseignant::_head = 0;					// tte de liste.
Enseignant* 	Enseignant::_enseignants [MAX_PROF];	// table de hashcode.

enum LMD { L, M, D };
enum FORM { FI, FC, PERS };

/*
 *	Pour la journalisation.
 */

void zlog (const char* fmt, ...)
{
	va_list args;
#if ZLOG
	static int		cnt = 10;
	extern gzFile	log_fp;
#endif

	va_start (args, fmt);
#if ZLOG
	if (log_fp != 0) {
#if __APPLE__
		char	buffer [4096];
		int 	n = vsprintf (buffer, fmt, args);

		gzwrite (log_fp, buffer, n);
#else
		gzvprintf (log_fp, fmt, args);
#endif
		if (--cnt <= 0) {
			gzflush (log_fp, Z_FINISH);
			cnt = 10;
		}
	} else
#endif
		vfprintf (stderr, fmt, args);
	va_end (args);
}

/*
 *	Cre un nouvel enseignant (si a pouvait tre aussi facile en vrai!)
 */

Enseignant::Enseignant (Employeur* employeur, Departement* departement,
						Categorie categorie,
						char* nom, char* email, int serv,
						int disp, int affectation, int annee,
						char* poste, int cnu)
: _service (0), _mandatement (0)
{
	_cnu = cnu;
	_suivant = 0;
	_precedent = 0;
	_employeur = employeur;
	_dept = departement;
	_categorie = categorie;
	_nom = nom;
	_email = email;
	_serv_annuel = serv;
	_disponibilite = disp;
	_affectation = affectation;
	_annee = annee;
	_poste = poste;
	_perso = 0;						// donnes personnelles vides pour l'instant.
	(void) name2prof (nom, this);	// l'insrer dans la table de hashcode.
	strcpy (_passwd, STD_PASSWD);	// passwd par dfaut.
}

/*
 *	Retourne la "vraie" adresse mail dans la chaine passe en argument.
 */

bool Enseignant::get_email (char* real_mail) {
	if (_email == 0)
		return false;

	for (char* p = _email; *p != 0; ++p, ++real_mail)
		if ((*real_mail = *p) == '(' && 0 == strncmp (p, "(at)", 4)) {
			*real_mail = '@';
			p += 3;
		}
	*real_mail = 0;
	return true;
}

/*
 *	Retourne le genre d'enseignant.
 */

Genre Enseignant::get_type (void)
{
	switch (_employeur -> _type) {
		case UNIVERSITE:
			if (_dept -> _a_nous) {
				switch (_categorie) {
					case MON:
					case AMN:
						return MONITEUR;
					case ATE:
					case ATER:
					case DEMI_ATER:
						return ATERS;
					case PAST:
					case CONTR:
						return CONTR_PAST;
					default:
						return TITULAIRE;
				}
			}
			if (_dept -> search ("IUT"))
				return IUT;
			if (_dept -> search ("Polytech") || _dept -> search ("IFIPS"))
				return POLYTECH;
			return AUTRE_UPS;

		case PRIVE:
			return VAC_PRIV;

		case FONCTIONNAIRE:
			switch (_categorie) {
				case CR:
				case DR:
				case IR:
				case IE:
					if (strcmp ("INRIA", _employeur -> _nom) == 0)
						return CH_INRIA;
					if (strcmp ("CNRS", _employeur -> _nom) == 0)
						return CH_CNRS;
				default:
				case VAC:
					return VAC_PUB;
			}
			return INCONNU;
		case BOURSIER:
			if (_categorie == MON || _categorie == AMN)
				return MONITEUR;
			return VAC_PUB;
		default:
			return INCONNU;		// non atteint.
	}
}

/*
 *	Extrait le nom et le prnom de l'enseignant.
 */

void Enseignant::nom_prenom (char* nom, char* prenom)
{
	char*	p;

	strcpy (prenom, _nom);
	// le _ spare les prnoms multiples.
	for (p = prenom;;) {
		switch (*p++) {
			default:
				continue;

			case 0:
				p = 0;
				break;

			case ' ':
				--p;
				break;

			case '_':
				p [-1] = ' ';	// remplacer le _ par un espace.
				continue;
		}
		break;
	}
	if (p != 0) {
		// avec prnom.
		*p++ = 0;
		for (int n = 0; (nom [n] = *p++) != 0; ++n)
			if (nom [n] != ' ' && nom [n] != '-')
				nom [n] &= ~040;
	} else {
		// sans prnom.
		for (int n = 0; (nom [n] = *p++) != 0; ++n)
			if (nom [n] != ' ' && nom [n] != '-')
				nom [n] &= ~040;
		prenom [0] = 0;
	}
}

/*
 *	Retourne l'enseignant prcdent (selon qu'on veut tous les
 *	enseignants ou seulement ceux qui ont un service non vide)
 */

Enseignant* Enseignant::precedent (void)
{
	Enseignant*	prof;

	if (_tous)
		return _precedent;		// facile!

	prof = this;
	do
		prof = prof -> _precedent;
		while (prof != this && prof -> _service.length () == 0);

	return prof;
}

/*
 *	Retourne l'enseignant suivant (selon qu'on veut tous les
 *	enseignants ou seulement ceux qui ont un service non vide)
 */

Enseignant* Enseignant::suivant (void)
{
	Enseignant*	prof;

	if (_tous)
		return _suivant;		// facile!

	prof = this;
	do
		prof = prof -> _suivant;
		while (prof != this && prof -> _service.length () == 0);

	return prof;
}

/*
 *	Remplace les espaces par des _ pour faire une url.
 */

char* Enseignant::nospaces (void)
{
	return nospaces (_nom);
}

char* Enseignant::nospaces (char* name)
{
	static char	result [128];	// gaffe  ne pas appeller plusieurs fois
								// sans recopier le rsultat!
	if (name != result) {		// ventuellement dj fait!
		char*	p;

		if (name == 0) {
			result [0] = 0;
			return result;
		}
		for (p = result; *name != 0;) {
			int	c = (int) (unsigned char) *name++;

			if (c == 0303)	// UTF8 -> ISO8859-1 approximatif!
				c = ((int) (unsigned char) *name++) | 0100;

			*p++ = iso2ascii [c];
		}
		*p = 0;
	}
	return result;
}

/*
 *	Retourne l'enseignant qui est prsident de notre dpartement d'enseignement.
 */

Enseignant* Enseignant::president_dept (void)
{
	Enseignant*	prof;

	prof = _head;	// parcourir la liste des enseignants.
	do {
		if (prof -> _employeur -> _type == UNIVERSITE &&
			prof -> _dept -> _a_nous)
		{
			for (int i = 0; i < prof -> _service.length (); ++i) {
				Service* s = (Service*) prof -> _service [i];

				if (! s -> _ens -> _charge || ! s -> _ens -> _charge_np)
					continue;	// ce doit tre une charge!

				if (!strcmp ("VPEnsDpt-A", s -> _ens -> _identification))
					return prof;	// c'est elle/lui !
			}
		}
		prof = prof -> _suivant;
	} while (prof != _head);	// jusqu'au dbut exclu.
	return 0;		// dpartement sans prsident!
}

#define NSEM		2
#define PREMIER		0
#define DEUXIEME	1

static char* float_to_string (float f)
{
	static char	tmp [32];
	int n = sprintf (tmp, "%.1f", f);

	for (int i = 0; i < n; ++i)
		if (tmp [i] == '.' && tmp [i+1] == '0') {
			tmp [i] = 0;
			break;
		}
	return tmp;
}

#ifdef STAT_ET_COMP
/*
 *	Retourne un fichier postscript reprsentant l'tat d'heures statutaires et complmentaires rempli.
 */

#define HSTAT	0
#define HCOMP	1

static inline float round (float x)
{
	return (float) (((int) (x*10.)) / 10.);
}

/*
 *	Trouve la composante de l'universit.
 */

const char*	Enseignant::find_comp (void)
{
	if (_employeur -> _type == UNIVERSITE) {
		const char*	dp;

		if (0 == strncmp ("Polytech", _dept -> _nom, 8))
			return "Polytech Paris-Sud";

		for (dp = _dept -> _nom; *dp != 0; ++dp)
			// "- IUT d'Orsay".
			if (dp [0] == 'I' && dp [1] == 'U' && dp [2] == 'T' && dp [6] == 'O')
				return dp;

		if (*dp == 0)
			return "UFR Sciences";
	}
	return "";
}

void Enseignant::etat_statutaire_complementaire (int page_no, int tot_page, bool hcc_ext)
{
	Service*	s;
	int			y [2];			// position verticale selon le type d'heure.
	int			x [3];			// position horizontale selon le type d'enseignement.
	float		total [2];		// total de chaque type d'heure.
	char		ps_intitule [128];	// l'intitul avec les parenthses backslashifies.
	float		volumes [2] [N_ENS_TYPES];	// volumes horaires dans chaque type d'enseignement.
	char*		intitule [2];
	char*		filiere [2];
	char*		fmt;
	char		fmt_int [] = "%d (.5) stringwidth pop sub %d moveto (%g) rshow\n";
	char		fmt_frac [] = "%d %d moveto (%g) rshow\n";

	// initialisation des coordonnes.
	y [HSTAT] = 380;
	y [HCOMP] = 217;

	x [COURS] = 410+50;
	x [TD] = 490+50;
	x [TP] = 560+50;
	// totaux  zro.
	total [HSTAT] = 0.;
	total [HCOMP] = 0.;
	// afficher l'entte.
	_outp_ += sprintf (_outp_, page_stat_comp_ps,
				page_no, tot_page,
				Taux::_year1, Taux::_year2,
				find_comp (),		// UFR ou IUT ou Polytech.
				_nom,				// nom de l'heureux propritaire.
				cat2str (true),		// son grade.
				// le prsident du dpartement d'enseignement.
				hcc_ext ? "" : _dept -> _vp_enseignement
				);
	// 'remplir' la page de services.
	_outp_ += sprintf (_outp_, "9 f0\n");	// helvetica 9.
	for (int i = 0; i < _service.length (); ++i) {
		float	tot [2];

		s = (Service*) _service [i];

		tot [HSTAT] = s -> affiche_stat (HEURE_STAT_A, &filiere [HSTAT], &intitule [HSTAT], volumes [HSTAT]);
		// avec les heures complmentaires extrieures.
		if (hcc_ext)
			tot [HCOMP] = s -> affiche_stat (HEURE_COMP_EXT, &filiere [HCOMP], &intitule [HCOMP], volumes [HCOMP]);
		else
			tot [HCOMP] = s -> affiche_stat (HEURE_COMP_A, &filiere [HCOMP], &intitule [HCOMP], volumes [HCOMP]);

		for (int sc = 0; sc < 2 ; ++sc) {
			if (tot [sc] == 0.)
				continue;

			// protger les parenthses pour la sortie postscript.
			int k = 0;
			for (int n = 0; intitule [sc] [n] != 0; ++n, ++k) {
				if (intitule [sc] [n] == '(' || intitule [sc] [n] == ')')
					ps_intitule [k++] = '\\';
				ps_intitule [k] = intitule [sc] [n];
			}
			ps_intitule [k] = 0;
			// afficher l'intitul.
#if HACK_UFE
			// Hack local: indiquer explicitement les enseignements UFE (du reste de la Licence) sur l'tat.
			if (0 == strncmp ("Formation UFE", s -> _ens -> _groupe -> _nom, sizeof ("Formation UFE") - 1))
			{
				_outp_ += sprintf (_outp_, "5 %d moveto (%s \\(%s\\)) show ",
									y [sc], ps_intitule, s -> _ens -> _groupe -> _nom);
			} else
#endif
			_outp_ += sprintf (_outp_, "5 %d moveto (%s \\(%s\\)) show ",
								y [sc], ps_intitule, filiere [sc]);
			// type de formation.
			_outp_ += sprintf (_outp_, "%d %d moveto (F%c) cshow ",
										// dcaler FC/FI  droite pour laisser plus de place (tait 355).
										385, y [sc],
										s -> _ens -> _form_initiale
											? 'I' : 'C');

			for (int th = COURS; th <= TP; ++th) {
				if (volumes [sc][th] == 0.)
					continue;

				fmt = (volumes [sc][th] - ((int) volumes [sc][th]) == 0.)
							 ? fmt_int
							 : fmt_frac;
				_outp_ += sprintf (_outp_, fmt,
										x [th], y [sc],
										volumes [sc][th]);
			}
			// afficher le total en heures eqTD arrondi  la demie heure.
			tot [sc] = round (tot [sc]);
			fmt = (tot [sc] - (int) tot [sc] != 0.) ? fmt_frac : fmt_int;
			_outp_ += sprintf (_outp_, fmt, 630+50, y [sc], tot [sc]);
			y [sc] -= 10;
			total [sc] += tot [sc];
		}
	}
	// afficher les totaux pour chaque type d'heures.
	_outp_ += sprintf (_outp_, "9 f1\n");	// helvetica-bold 9.
	for (int sc = 0; sc < 2; ++sc)
		if (total [sc] > 0.) {
			_outp_ += sprintf (_outp_,
					"%d %d moveto 40 0 rlineto stroke ", 630+10, y [sc]+5);

			fmt = (total [sc] - (int) total [sc] == 0.5) ? fmt_frac : fmt_int;
			_outp_ += sprintf (_outp_, fmt, 630+50, y [sc]-5, total [sc]);
		}
	_outp_ += sprintf (_outp_, "%d y 268 sub moveto (Total :     %g) show\n",
									630+20, total [HSTAT] + total [HCOMP]);
	// afficher l'intitul de la dcharge s'il y a lieu.
	if ((_categorie == MC || _categorie == PR) &&
		(192 > _serv_annuel && _serv_annuel >= 0))
	{
		_outp_ += sprintf (_outp_, "5 %d moveto (%s) show ",
							y [0]-15, disponibilite [_disponibilite]._intitule_ps);
		_outp_ += sprintf (_outp_, "5 %d moveto (service statutaire : %d heures eq. TD) show ",
							y [0]-25, _serv_annuel);
	}
	// afficher la fin du code postscript.
	_outp_ += sprintf (_outp_, "%s", fin_page_stat_comp_ps);
}

/*
 *	Pour les HCC de Polytech (HCC_EXT) appeler avec 'true'
 */

void Enseignant::etat_statutaire_complementaire (bool hcc_ext)
{
	_outp_ += sprintf (_outp_, entete_stat_comp_ps, 1);
	etat_statutaire_complementaire (1, 1, hcc_ext);
	_outp_ += sprintf (_outp_, "%s", fin_stat_comp_ps);
}
#endif

/*
 *	Retourne un fichier postscript reprsentant l'tat d'heures statutaires rempli.
 */

void Enseignant::etat_statutaire (void)
{
	Service*	s;
	int			y [NSEM];		// position verticale selon le semestre.
	int			x [3];			// position horizontale selon le type d'enseignement.
	float		volumes [N_ENS_TYPES];	// volumes horaires dans chaque type d'enseignement.
	float		total [NSEM];	// total de chaque semestre.
	char		ps_intitule [256];	// l'intitul avec les parenthses backslashifies.
	char*		intitule;
	char*		filiere;
	char*		fmt;
	char		fmt_int [] = "%d (.5) stringwidth pop sub %d moveto (%s) rshow\n";
	char		fmt_frac [] = "%d %d moveto (%s) rshow\n";

	// initialisation des coordonnes.
	y [PREMIER] = 380;
	y [DEUXIEME] = 247;

	x [COURS] = 410+50;
	x [TD] = 490+50;
	x [TP] = 560+50;
	// totaux  zro.
	total [PREMIER] = 0.;
	total [DEUXIEME] = 0.;
	// afficher l'entte.
	_outp_ += sprintf (_outp_, entete_stat_ps,
				Taux::_year1, Taux::_year2,
				find_comp (),		// UFR ou IUT ou Polytech.
				_nom,				// nom de l'heureux propritaire.
				cat2str (true),		// son grade.
				_dept -> _vp_enseignement);
	// 'remplir' la page de services.
	_outp_ += sprintf (_outp_, "9 f0\n");	// helvetica 9.
	for (int i = 0; i < _service.length (); ++i) {
		int		semestre;
		float	tots;

		s = (Service*) _service [i];
		// tester la priode pour avoir le semestre
		semestre = Per2Semestre (s -> _ens -> _periode);
		tots = s -> affiche_stat (HEURE_STAT_A, &filiere, &intitule, volumes);
		if (tots == 0.0)
			continue;	// service vide ???

		// protger les parenthses pour la sortie postscript.
		int k = 0;
		for (int n = 0; intitule [n] != 0; ++n, ++k) {
			if (intitule [n] == '(' || intitule [n] == ')')
				ps_intitule [k++] = '\\';
			ps_intitule [k] = intitule [n];
		}
		ps_intitule [k] = 0;
		// afficher l'intitul.
		_outp_ += sprintf (_outp_, "5 %d moveto (%s \\(%s\\)) show ",
							y [semestre], ps_intitule, filiere);
		// type de formation.
		_outp_ += sprintf (_outp_, "%d %d moveto (F%c) cshow ",
									355, y [semestre],
									s -> _ens -> _form_initiale
										? 'I' : 'C');

		for (int th = COURS; th <= TP; ++th) {
			if (volumes [th] == 0.)
				continue;

			fmt = (volumes [th] - ((int) volumes [th]) == 0.)
						 ? fmt_int
						 : fmt_frac;
			_outp_ += sprintf (_outp_, fmt,
									x [th], y [semestre],
									float_to_string (volumes [th]));
		}
		// afficher le total en heures eqTD arrondi  la demie heure.
		fmt = (tots - (int) tots == 0.) ? fmt_int : fmt_frac;
		_outp_ += sprintf (_outp_, fmt, 630+50, y [semestre],
							float_to_string (tots));
		y [semestre] -= 10;
		total [semestre] += tots;
	}
	// afficher les totaux de chaque semestre.
	_outp_ += sprintf (_outp_, "9 f1\n");	// helvetica-bold 9.
	for (int sem = PREMIER; sem < NSEM; ++sem)
		if (total [sem] > 0.) {
			_outp_ += sprintf (_outp_,
					"%d %d moveto 40 0 rlineto stroke ", 630+10, y [sem]+5);
			fmt = (total [sem] - (int) total [sem] == 0.) ? fmt_int : fmt_frac;
			_outp_ += sprintf (_outp_, fmt, 630+50, y [sem]-5,
								float_to_string (total [sem]));
		}
	_outp_ += sprintf (_outp_, "%d y 268 sub moveto (Total :     %d) show\n",
									630+20, (int) (0.9 + total [PREMIER] + total [DEUXIEME]));
	// afficher l'intitul de la dcharge s'il y a lieu.
	if ((_categorie == MC || _categorie == PR) &&
		(192 > _serv_annuel && _serv_annuel > 0))
	{
		_outp_ += sprintf (_outp_, "5 %d moveto (%s) show ",
							y [0]-15, disponibilite [_disponibilite]._intitule_ps);
		_outp_ += sprintf (_outp_, "5 %d moveto (service statutaire : %d heures eq. TD) show ",
							y [0]-25, _serv_annuel);
	}
	// afficher la fin du code postscript.
	_outp_ += sprintf (_outp_, "%s", fin_stat_ps);
}

/*
 *	Retourne un fichier postscript reprsentant les services faits ou prvisionnel selon.
 */

void Enseignant::attestations_service (int type)
{
	int			npages = 0;
	Enseignant*	prof;

	prof = _head;	// parcourir la liste des enseignants pour les compter!
	do {
		Genre	g = prof -> get_type ();
		if (g == ATERS || g == TITULAIRE || g == CONTR_PAST)
		{
			npages += 2;	// 2 pages par tat.
		}
		prof = prof -> _suivant;
	} while (prof != _head);	// jusqu'au dbut exclu.
	// parcourir la liste des enseignants pour les imprimer cette fois-ci!
	int	n = 0;
	do {
		Genre	g = prof -> get_type ();
		if (g == ATERS || g == TITULAIRE || g == CONTR_PAST)
		{
			switch (type) {
			case 0:
				prof -> attestations_service (++n, npages, false);
				break;
			case 1:
				prof -> attestations_service (++n, npages, true);
				break;
			case 2:
				prof -> attestations_hybride (++n, npages);
				break;
			}
			++n;
		}
		prof = prof -> _suivant;
	} while (prof != _head);	// jusqu'au dbut exclu.
}

/*
 *	Renvoie un tat de services faits ou prvisionnel selon le cas.
 */

void Enseignant::attestations_service (int page_no, int tot_page, bool faits)
{
	Service*	s;
	int			y [NSEM];		// position verticale selon le semestre.
	int			x [2][3];		// position horizontale selon les type d'heures et d'enseignement.
	float		volumes [2][N_ENS_TYPES];	// volumes horaires dans chaque type d'enseignement.
	int			lmd [3];		// position horizontale selon le niveau d'enseignement.
	int			form [3];		// position horizontale du type de formation.
	int			xtot [2] = { 745, 792 };		// position horizontale du total.
	float		total [2][NSEM];	// total de chaque semestre.
	char		ps_intitule [128];	// l'intitul avec les parenthses backslashifies.
	char*		intitule;
	char*		filiere;
	char*		fmt;
	char		fmt_int [] = "%d (.5) stringwidth pop sub %d moveto (%s) rshow\n";
	char		fmt_frac [] = "%d %d moveto (%s) rshow\n";
	bool		polytech = (get_type () == POLYTECH);

	// initialisation des coordonnes.
	y [PREMIER] = 374;
	y [DEUXIEME] = 254;
#define XCMS	568
#define XCMC	XCMS+75
	x [HSTAT][COURS] = XCMS;	x [HSTAT][TD] = XCMS+28;	x [HSTAT][TP] = XCMS+28+28;
	x [HCOMP][COURS] = XCMC;	x [HCOMP][TD] = XCMC+28;	x [HCOMP][TP] = XCMC+28+28;
	lmd [L] = 360;			lmd [M] = 384;			lmd [D] = 408;
	form [FI] = 436;		form [FC] = 484;		form [PERS] = 540;
	// totaux  zro.
	total [HSTAT][PREMIER] = 0.; total [HCOMP][PREMIER] = 0.;
	total [HSTAT][DEUXIEME] = 0.; total [HCOMP][DEUXIEME] = 0.;
	// afficher l'entte.
	if (page_no == 1)
		_outp_ += sprintf (_outp_, etat_prev_entete, tot_page);
	// Extraire le prnom de l'enseignant.
	char	prenom [128];
	char	nom [128];

	nom_prenom (nom, prenom);
	// afficher les informations d'identit.
	_outp_ += sprintf (_outp_, etat_prev_form,
				page_no, tot_page,
				faits ? "DES SERVICES EFFECTUS" : "PRVISIONNEL DES SERVICES",
				Taux::_year1, Taux::_year2,
				nom, prenom,		// nom de l'heureux propritaire.
				cat2str (true),		// son grade.
				find_comp (), // la composante.
				_cnu,				// sa section CNU.
				Taux::_year1,		// PES au 01/10/YEAR1.
				Taux::_year1);		// PEDR au 01/10/YEAR1.
	// 'remplir' la page de services.
	for (int i = 0; i < _service.length (); ++i) {
		int		semestre;
		float	tots [2];

		s = (Service*) _service [i];
		// tester la priode pour avoir le semestre
		semestre = Per2Semestre (s -> _ens -> _periode);
		tots [HSTAT] = s -> affiche_stat (HEURE_STAT_A, &filiere, &intitule, volumes [HSTAT]);
		tots [HCOMP] = s -> affiche_stat (HEURE_COMP_A, &filiere, &intitule, volumes [HCOMP]);
		if (tots [HSTAT] == 0.0 && tots [HCOMP] == 0.0)
			continue;	// service vide ???
		// protger les parenthses pour la sortie postscript.
		int k = 0;
		for (int n = 0; intitule [n] != 0; ++n, ++k) {
			if (intitule [n] == '(' || intitule [n] == ')')
				ps_intitule [k++] = '\\';
			ps_intitule [k] = intitule [n];
		}
		ps_intitule [k] = 0;
		// afficher l'intitul.
#if HACK_UFE
		// Hack local: indiquer explicitement les enseignements UFE (du reste de la Licence) sur l'tat.
		if (0 == strncmp ("Formation UFE", s -> _ens -> _groupe -> _nom, sizeof ("Formation UFE") - 1))
		{
			_outp_ += sprintf (_outp_, "7.5 f0\n6 %d moveto (%s \\(%s\\)) show\n9 f0\n",
								y [semestre], ps_intitule, s -> _ens -> _groupe -> _nom);
		} else
#endif
		_outp_ += sprintf (_outp_, "7.5 f0\n6 %d moveto (%s \\(%s\\)) show\n9 f0\n",
							y [semestre], ps_intitule, filiere);
		// L M ou D?
		// trop "hack", mettre un flag dans la classe Filiere.
		if (!strncmp ("Licence", filiere, sizeof ("Licence") - 1) ||
			!strcmp ("Polytech", s -> _ens -> _groupe -> _filiere -> _identification) ||
			!strcmp ("IFIPS", s -> _ens -> _groupe -> _filiere -> _identification))
			_outp_ += sprintf (_outp_, "%d %d moveto (x) show\n", lmd [L], y [semestre]);
		else if (!strncmp ("Master", filiere, sizeof ("Master") - 1) ||
				 !strcmp ("FIIFO4", s -> _ens -> _groupe -> _identification) ||
				 !strcmp ("FIIFO5", s -> _ens -> _groupe -> _identification))
			_outp_ += sprintf (_outp_, "%d %d moveto (x) show\n", lmd [M], y [semestre]);
		// type de formation.
		_outp_ += sprintf (_outp_, "%d %d moveto (x) show ",
									s -> _ens -> _form_initiale
										? form [FI] : form [FC],
									y [semestre]);

		for (int sc = HSTAT; sc <= HCOMP; ++sc) {
			if (tots [sc] == 0.0)
				continue;

			for (int th = COURS; th <= TP; ++th) {
				if (volumes [sc][th] == 0.)
					continue;

				fmt = (volumes [sc][th] - ((int) volumes [sc][th]) == 0.) ? fmt_int : fmt_frac;
				_outp_ += sprintf (_outp_, fmt,
										x [sc][th], y [semestre],
										float_to_string (volumes [sc][th]));
			}
			// afficher le total en heures eqTD arrondi  la demie heure.
			fmt = (tots [sc] - (int) tots [sc] == 0.) ? fmt_int : fmt_frac;
			_outp_ += sprintf (_outp_, fmt, xtot [sc], y [semestre], float_to_string (tots [sc]));
			total [sc][semestre] += tots [sc];
		}
		y [semestre] -= 10;
	}
	// afficher les totaux de chaque semestre.
	_outp_ += sprintf (_outp_, "9 f1\n");	// helvetica-bold 9.
	for (int sc = HSTAT; sc <= HCOMP; ++sc) {
		for (int sem = PREMIER; sem < NSEM; ++sem)
			if (total [sc][sem] > 0.) {
				_outp_ += sprintf (_outp_,
						"%d %d moveto 36 0 rlineto stroke ", xtot [sc]-32, y [sem]+5);

				fmt = (total [sc][sem] - (int) total [sc][sem] == 0.) ? fmt_int : fmt_frac;
				_outp_ += sprintf (_outp_, fmt, xtot [sc], y [sem]-5, float_to_string (total [sc][sem]));
			}
		// afficher l'intitul de la dcharge s'il y a lieu.
		if ((_categorie == MC || _categorie == PR) &&
			(192 > _serv_annuel && _serv_annuel >= 0))
		{
			float	decharge = 192. - _serv_annuel;

			if (sc == HSTAT) {
				if (0 == strcmp ("Delegation", disponibilite [_disponibilite]._motclef))
					_outp_ += sprintf (_outp_, "%d %d moveto cocher\n", 120, 480);
				else if (0 == strcmp ("Sabbatique", disponibilite [_disponibilite]._motclef))
					_outp_ += sprintf (_outp_, "%d %d moveto cocher\n", 300, 480);
				else if (0 == strcmp ("Disponibilite", disponibilite [_disponibilite]._motclef) ||
						 0 == strcmp ("Detachement", disponibilite [_disponibilite]._motclef))
					_outp_ += sprintf (_outp_, "%d %d moveto cocher\n", 520, 480);
				else if (0 == strcmp ("Partiel", disponibilite [_disponibilite]._motclef))
					_outp_ += sprintf (_outp_, "%d %d moveto cocher %d %d moveto 13 f5 (%d) show\n",
										726, 480, 756, 480, (_serv_annuel*100) / 192);
				if (decharge > 0.) {
					_outp_ += sprintf (_outp_, "5 %d moveto 12 t0 (Dcharge: ) show 9 f1 (%s) show ",
										170, disponibilite [_disponibilite]._intitule_ps);
					_outp_ += sprintf (_outp_, "10 f1 %d 170 moveto (%g) rshow\n",
												xtot [sc]-8, decharge);
				}
				_outp_ += sprintf (_outp_, "%d 146 moveto (%s) rshow 9 f1\n",
											xtot [sc]-8, float_to_string (decharge + total [sc][PREMIER] + total [sc][DEUXIEME]));
			} else
				if (total [sc][PREMIER] + total [sc][DEUXIEME] > 0.)
					_outp_ += sprintf (_outp_, "%d 146 moveto (%s) rshow 9 f1\n",
												xtot [sc]-8, float_to_string (total [sc][PREMIER] + total [sc][DEUXIEME]));
		}
		else {
			if (total [sc][PREMIER] + total [sc][DEUXIEME] > 0.)
				_outp_ += sprintf (_outp_, "10 f1 %d 146 moveto (%s) rshow 9 f1\n",
											xtot [sc]-8, float_to_string (total [sc][PREMIER] + total [sc][DEUXIEME]));
		}
	}
	// afficher la fin du code postscript.
	_outp_ += sprintf (_outp_, etat_prev_fin,
							// le directeur de Polytech.
							polytech ? _dept -> _vp_enseignement : "",
							// le prsident du dpartement d'enseignement.
							polytech ? "" : _dept -> _vp_enseignement,
							//  le numro de page et leur nombre total.
							page_no+1, tot_page,
							_cnu == 27 ? "" : "%");
	// EOF si c'est la dernire page.
	if (page_no+1 == tot_page)
		_outp_ += sprintf (_outp_, "%%%%EOF\n");
}

/*
 *	Renvoie un tat de services faits ou prvisionnel selon le cas.
 */

void Enseignant::attestations_hybride (int page_no, int tot_page)
{
	Service*	s;
	int			y [NSEM];		// position verticale selon le semestre.
	int			x [3];			// position horizontale selon le type d'enseignement.
	int			lmd [3];		// position horizontale selon le niveau d'enseignement.
	int			form [3];		// position horizontale du type de formation.
	int			xtot = 780;		// position horizontale du total.
	float		volumes [N_ENS_TYPES];	// volumes horaires dans chaque type d'enseignement.
	float		total [NSEM];	// total de chaque semestre.
	char		ps_intitule [128];	// l'intitul avec les parenthses backslashifies.
	char*		intitule;
	char*		filiere;
	char*		fmt;
	char		fmt_int [] = "%d (.5) stringwidth pop sub %d moveto (%s) rshow\n";
	char		fmt_frac [] = "%d %d moveto (%s) rshow\n";
	int			ntut = 0;
	bool		tutorat_ok = false;
	// initialisation des coordonnes.
	y [PREMIER] = 374;	y [DEUXIEME] = 254;
	x [COURS] = 630;	x [TD] = 670;		x [TP] = 710;
	lmd [L] = 380;		lmd [M] = 404;		lmd [D] = 428;
	form [FI] = 466;	form [FC] = 514;	form [PERS] = 570;
	// totaux  zro.
	total [PREMIER] = 0.;
	total [DEUXIEME] = 0.;
	// afficher l'entte.
	if (page_no == 1)
		_outp_ += sprintf (_outp_, etat_hybride_entete, tot_page);
	// Extraire le prnom de l'enseignant.
	char	prenom [128];
	char	nom [128];

	nom_prenom (nom, prenom);
	// afficher les informations d'identit.
	_outp_ += sprintf (_outp_, etat_hybride_form,
				page_no, tot_page,
				Taux::_year1, Taux::_year2,
				nom, prenom,		// nom de l'heureux propritaire.
				cat2str (true),		// son grade.
				_cnu,				// sa section CNU.
				Taux::_year1);		// PEDR au 01/10/YEAR1.
	// 'remplir' la page de services.
	_outp_ += sprintf (_outp_, "9 f0\n");	// helvetica 9.
	// compter les tutorats de moniteurs.
	for (int i = 0; i < _service.length (); ++i) {
		s = (Service*) _service [i];
		if (0 != strcmp ("Tutorat", s -> _ens -> _groupe -> _identification))
			continue;
		++ ntut;
	}
	// allons-y.
	for (int i = 0; i < _service.length (); ++i) {
		int		semestre;
		float	tots;

		s = (Service*) _service [i];
		// tester la priode pour avoir le semestre
		semestre = Per2Semestre (s -> _ens -> _periode);
		if (ntut > 1 && 0 == strcmp ("Tutorat", s -> _ens -> _groupe -> _identification))
		{
			if (tutorat_ok)
				continue;	// ne traiter que le premier tutorat.

			tutorat_ok = true;
			tots = s -> affiche_stat (HEURE_STAT_A, &filiere, &intitule, volumes);
			intitule = (char*) "Tutorat de moniteurs";
			volumes [TD] *= ntut;
			tots *= ntut;
		} else
			tots = s -> affiche_stat (HEURE_STAT_A, &filiere, &intitule, volumes);
		if (tots == 0.0)
			continue;	// service vide ???

		// protger les parenthses pour la sortie postscript.
		int k = 0;
		for (int n = 0; intitule [n] != 0; ++n, ++k) {
			if (intitule [n] == '(' || intitule [n] == ')')
				ps_intitule [k++] = '\\';
			ps_intitule [k] = intitule [n];
		}
		ps_intitule [k] = 0;
		// afficher l'intitul.
#if HACK_UFE
		// Hack local: indiquer explicitement les enseignements UFE (du reste de la Licence) sur l'tat.
		if (0 == strncmp ("Formation UFE", s -> _ens -> _groupe -> _nom, sizeof ("Formation UFE") - 1))
		{
			_outp_ += sprintf (_outp_, "6 %d moveto (%s \\(%s\\)) show ",
								y [semestre], ps_intitule, s -> _ens -> _groupe -> _nom);
		} else
#endif
		_outp_ += sprintf (_outp_, "6 %d moveto (%s \\(%s\\)) show ",
							y [semestre], ps_intitule, filiere);
		// L M ou D?
		// trop "hack", mettre un flag dans la classe Filiere.
		if (!strncmp ("Licence", filiere, sizeof ("Licence") - 1) ||
			!strcmp ("L3-MIAGE-CFA", s -> _ens -> _groupe -> _identification) ||
			!strcmp ("Polytech", s -> _ens -> _groupe -> _filiere -> _identification) ||
			!strcmp ("IFIPS", s -> _ens -> _groupe -> _filiere -> _identification))
			_outp_ += sprintf (_outp_, "%d %d moveto (x) show\n", lmd [L], y [semestre]);
		else if (!strncmp ("Master", filiere, sizeof ("Master") - 1) ||
				 !strcmp ("FIIFO4", s -> _ens -> _groupe -> _identification) ||
				 !strcmp ("FIIFO5", s -> _ens -> _groupe -> _identification))
			_outp_ += sprintf (_outp_, "%d %d moveto (x) show\n", lmd [M], y [semestre]);
		// type de formation.
		_outp_ += sprintf (_outp_, "%d %d moveto (x) show ",
									s -> _ens -> _form_initiale
										? form [FI] : form [FC],
									y [semestre]);

		for (int th = COURS; th <= TP; ++th) {
			if (volumes [th] == 0.)
				continue;

			fmt = (volumes [th] - ((int) volumes [th]) == 0.) ? fmt_int : fmt_frac;
			_outp_ += sprintf (_outp_, fmt,
									x [th], y [semestre],
									float_to_string (volumes [th]));
		}
		// afficher le total en heures eqTD arrondi  la demie heure.
		fmt = (tots - (int) tots == 0.) ? fmt_int : fmt_frac;
		_outp_ += sprintf (_outp_, fmt, xtot, y [semestre], float_to_string (tots));
		y [semestre] -= 10;
		total [semestre] += tots;
	}
	// afficher les totaux de chaque semestre.
	_outp_ += sprintf (_outp_, "9 f1\n");	// helvetica-bold 9.
	for (int sem = PREMIER; sem < NSEM; ++sem)
		if (total [sem] > 0.) {
			_outp_ += sprintf (_outp_,
					"%d %d moveto 40 0 rlineto stroke ", xtot-40, y [sem]+5);

			fmt = (total [sem] - (int) total [sem] == 0.) ? fmt_int : fmt_frac;
			_outp_ += sprintf (_outp_, fmt, xtot, y [sem]-5, float_to_string (total [sem]));
		}
	// afficher l'intitul de la dcharge s'il y a lieu.
	float	decharge = 0;
	if ((_categorie == MC || _categorie == PR || _categorie == ATER) &&
		(192 > _serv_annuel && _serv_annuel >= 0))
	{
		decharge = 192. - _serv_annuel;

		if (0 == strcmp ("Delegation", disponibilite [_disponibilite]._motclef))
			_outp_ += sprintf (_outp_, "%d %d moveto cocher\n", 120, 480);
		else if (0 == strcmp ("Sabbatique", disponibilite [_disponibilite]._motclef))
			_outp_ += sprintf (_outp_, "%d %d moveto cocher\n", 300, 480);
		else if (0 == strcmp ("Disponibilite", disponibilite [_disponibilite]._motclef) ||
				 0 == strcmp ("Detachement", disponibilite [_disponibilite]._motclef))
			_outp_ += sprintf (_outp_, "%d %d moveto cocher\n", 520, 480);
		else if (0 == strcmp ("Partiel", disponibilite [_disponibilite]._motclef))
			_outp_ += sprintf (_outp_, "%d %d moveto cocher %d %d moveto 13 f5 (%d) show\n",
								726, 480, 756, 480, (_serv_annuel*100) / 192);
	}
	else if ((_categorie == AMN || _categorie == MON) && (64 > _serv_annuel && _serv_annuel >= 0))
		decharge = 64. - _serv_annuel;
	else if (_categorie == DEMI_ATER && (96 > _serv_annuel && _serv_annuel >= 0))
		decharge = 96. - _serv_annuel;
	else if (_categorie == ATE && (384 > _serv_annuel && _serv_annuel >= 0))
		decharge = 384. - _serv_annuel;

	if (decharge > 0.) {
		_outp_ += sprintf (_outp_, "5 %d moveto 12 t0 (Dcharge: ) show 9 f1 (%s) show ",
							170, disponibilite [_disponibilite]._intitule_ps);
		_outp_ += sprintf (_outp_, "10 f1 %d 170 moveto (%g) rshow\n",
									xtot-8, decharge);
	}
	_outp_ += sprintf (_outp_, "%d 146 moveto (%s) rshow 9 f1\n",
									xtot-8, float_to_string (decharge + total [PREMIER] + total [DEUXIEME]));
	// afficher la fin du code postscript. (identique  etat_prev_fin).
	_outp_ += sprintf (_outp_, etat_hybride_fin,
							// le prsident du dpartement d'enseignement.
							_dept -> _vp_enseignement,
							//  le numro de page et leur nombre total.
							page_no+1, tot_page,
							_cnu == 27 ? "" : "%");
	// EOF si c'est la dernire page.
	if (page_no+1 == tot_page)
		_outp_ += sprintf (_outp_, "%%%%EOF\n");
}

#undef NSEM
#undef PREMIER
#undef DEUXIEME

/*
 *	Retourne un fichier postscript reprsentant la fiche d'engagement de paiement d'HC.
 */

#if FICHE_LATEX

// nouvelle version en jachre...

void Enseignant::fiche_engagement (int index_service)
{
	// c'est quoi a, mmmh ??!?
	if (index_service < 0 || index_service >= _service.length ()) {
		_outp_ += sprintf (_outp_, "<script>top.history.back();</script>\n");
		zlog ("index_service = %d, _service.length () = %d\n",
						index_service, _service.length ());
		return;
	}

	Service*		sv = (Service*) _service [index_service];
	Enseignant*		resp;
	static const char*
				COURSTDTP [N_ENS_TYPES] = { "cours", "TD", "TP", "", "TD", "TD", "TD" };

	_outp_ += sprintf (_outp_, "\\annee{%d / %d}\n", Taux::_year1, Taux::_year2);
	// afficher les renseignements professionnels.
	switch (_employeur -> _type) {
		case UNIVERSITE:
			_outp_ += sprintf (_outp_, "\\ups\n\\tpgyvelinesO\n");
			break;

		case PRIVE:
			_outp_ += sprintf (_outp_, "\\tpgyvelinesN\n");	// surement pas le TPG des Yvelynes!
			break;

		case FONCTIONNAIRE:
		case BOURSIER:
			break;
	}
	// afficher les divers renseignements.
	resp = sv -> _ens -> _groupe -> _responsable;
	if (resp != 0)
		_outp_ += sprintf (_outp_, "\\respfil{%s}\n", resp -> _nom);

	// sparer le prnom du nom de l'enseignant.
	char	prenom [128];
	char	nom [128];

	nom_prenom (nom, prenom);
	_outp_ += sprintf (_outp_, "\\nom{%s}\n", nom);
	_outp_ += sprintf (_outp_, "\\prenom{%s}\n", prenom);
	_outp_ += sprintf (_outp_, "\\grade{%s}\n", cat2str (true));
	// son email.
	char	real_mail [128];
	if (get_email (real_mail))
		_outp_ += sprintf (_outp_, "\\email{%s}\n", real_mail);
	// date d'aujourd'hui.
	time_t		now = time (0);
	struct tm*	lt = localtime (&now);	// date de fabrication de la demande d'HCC.

	_outp_ += sprintf (_outp_, "\\fdate{%02d/%02d/%02d}\n",
				lt -> tm_mday, lt -> tm_mon+1, 1900 + lt -> tm_year);
	// regarder si un mandatement existe pour ce service.
	Mandatement*	m = 0;
	float			_vol_mand [N_ENS_TYPES] = { 0, 0, 0, 0 };

	for (int k = 0; k < _mandatement.length (); ++k) {
		// il peut y avoir plusieurs mandatements pour le mme service.
		m = (Mandatement*) _mandatement [k];
		if (m -> _s == sv) {
			_vol_mand [COURS] += m -> _vol [COURS];
			_vol_mand [TD] += m -> _vol [TD];
			_vol_mand [TP] += m -> _vol [TP];
			_vol_mand [MP] += m -> _vol [MP];
			_vol_mand [DTD] += m -> _vol [DTD];
			_vol_mand [STD] += m -> _vol [STD];
			_vol_mand [SSS] += m -> _vol [SSS];
		}
	}
	// l'enseignement en question.
	bool	td_fait = false;
	for (int i = COURS; i < COURS_TD; ++i) {
		float	volume;
		bool	accu = false;

		// mettre les TD, DTD, STD, et SSS dans le mme paquet.
		if ((i == TD || i == DTD || i == STD || i == SSS) && !td_fait) {
			accu = true;
			volume = sv -> _heures [HEURE_COMP_A][TD] +
					 sv -> _heures [HEURE_COMP_A][MP] +
					 sv -> _heures [HEURE_COMP_A][DTD] +
					 sv -> _heures [HEURE_COMP_A][STD] +
					 sv -> _heures [HEURE_COMP_A][SSS];
		} else
			volume = sv -> _heures [HEURE_COMP_A][i];

		if (volume > 0.) {
			if (_vol_mand [i] == volume || td_fait)
				continue;

			time_t	periode_deb;
			time_t	periode_fin;

			// pour ne pas sortir N fois la mme ligne, se rappeler qu'on vient de le faire!
			if (accu)
				td_fait = true;

			(void) Per2Time (sv -> _ens -> _periode, periode_deb, periode_fin);
			_outp_ += sprintf (_outp_, "\\%s{%g}{%s}",
								COURSTDTP [i],
								volume,
								AscTime (&periode_deb));
			// coup en deux car AscTime retourne une chaine statique...
			_outp_ += sprintf (_outp_, "{%s}\n", AscTime (&periode_fin));
		}
	}
	_outp_ += sprintf (_outp_, "\\formation{%s}\n",
		sv -> _ens -> _groupe -> _filiere -> _nom);
	_outp_ += sprintf (_outp_, "\\module{%s}\n", sv -> _ens -> get_intitule ());
}

#else

void Enseignant::fiche_engagement (int n, bool with_date)
{
	Mandatement*	m = (Mandatement*) _mandatement [n];
	Enseignant*		resp;
	bool			tpg_yv = false;
	int				type_emploi = 0;
	static const char*	mois [] = {
		"zero",
		"janvier", "fvrier", "mars", "avril", "mai", "juin",
		"juillet", "aot", "septembre", "octobre", "novembre", "dcembre"
	};

	// afficher le dbut du code postscript.
	_outp_ += sprintf (_outp_, engagement_ps_deb,
							Taux::_year1, Taux::_year2);
	// afficher les renseignements perso.
	if (_perso) {
		tpg_yv = _perso -> _tpg_yv;
		_outp_ += sprintf (_outp_, "(%s) nomjf\n", _perso -> _njf);
		_outp_ += sprintf (_outp_, "(%d %s %d) (%s) ddn\n",
							_perso -> _jdn, mois [_perso -> _mdn], _perso -> _adn,
							_perso -> _ldn);
		_outp_ += sprintf (_outp_, "%d situation\n", _perso -> _sf);
		_outp_ += sprintf (_outp_, "(%s) (%s) no_secu\n", _perso -> _secu, _perso -> _clef);
		_outp_ += sprintf (_outp_, "0 (%s) adresse\n", _perso -> _ad1);
		_outp_ += sprintf (_outp_, "1 (%s) adresse\n", _perso -> _ad2);
		_outp_ += sprintf (_outp_, "2 (%s) adresse\n", _perso -> _ad3);
		_outp_ += sprintf (_outp_, "(%s) tel\n", _perso -> _ntel);
		_outp_ += sprintf (_outp_, "(%s) tel_pro\n", _perso -> _ntelpro);
	}
	// afficher les renseignements professionnels.
	switch (_employeur -> _type) {
		case UNIVERSITE:
			tpg_yv = true;		// c'est le TPG de Yvelynes qui paie le personnel de l'UPS.
		case FONCTIONNAIRE:
			type_emploi = 3;	// titulaire.
			break;
		case PRIVE:
			type_emploi = (! strcmp (_employeur -> _nom, "INDEPENDANT"))
							? 4		// activit indpendante.
							: 2;	// priv.
			break;
		case BOURSIER:
			type_emploi = 5;	// boursier MRT.
			break;
	}
	_outp_ += sprintf (_outp_, "%d type_emploi\n",  type_emploi);
	_outp_ += sprintf (_outp_, "0 (%s) adresse_emp\n", _employeur -> _ad1);
	_outp_ += sprintf (_outp_, "1 (%s) adresse_emp\n", _employeur -> _ad2);
	_outp_ += sprintf (_outp_, "2 (%s) adresse_emp\n", _employeur -> _ad3);
	_outp_ += sprintf (_outp_, "%s tpg_yvelines\n",  tpg_yv ? "true" : "false");
	// afficher les divers renseignements.
	resp = m -> _s -> _ens -> _groupe -> _filiere -> _responsable;
	if (resp != 0)
		_outp_ += sprintf (_outp_, "(%s) responsable\n", resp -> _nom);
	// le prsident du dpartement.
	if (_dept -> _vp_enseignement [0] != 0)
		_outp_ += sprintf (_outp_, "(%s) presdeptens\n", _dept -> _vp_enseignement);
	_outp_ += sprintf (_outp_, "%d division\n", m -> _division);
	_outp_ += sprintf (_outp_, "(%d) nbetud\n", m -> _s -> _ens -> _groupe -> _effectifs);
	_outp_ += sprintf (_outp_, "(%s) nom\n", _nom);
	_outp_ += sprintf (_outp_, "%d formation\n", m -> _fi ? 0 : 1);
	_outp_ += sprintf (_outp_, "(%s) grade\n", cat2str (true));

/* A FAIRE AVEC LES DTD, STD et SSS... */

	for (int i = COURS; i <= TP; ++i) {
		float	volume = m -> _vol [i] + ((i == TD) ? m -> _vol [MP] : 0);

		if (volume > 0.) {
			if (with_date) {
				_outp_ += sprintf (_outp_, "(%g) (%s) ",
									volume, AscTime (&m -> _fin_periode));
				_outp_ += sprintf (_outp_, "(%s) %d acte\n",
									AscTime (&m -> _deb_periode), i);
			} else {
				_outp_ += sprintf (_outp_, "(%g) ( ) ", volume);
				_outp_ += sprintf (_outp_, "( ) %d acte\n", i);
			}
		}
	}
	_outp_ += sprintf (_outp_, "8 f1 (%s [%s]) intitule 11 f1\n",
						m -> _s -> _ens -> get_intitule (),
						m -> _s -> _ens -> _groupe -> _filiere -> _nom);
	// afficher la fin du code postscript.
	_outp_ += sprintf (_outp_, "%s", engagement_ps_fin);
}
#endif

/*
 *	Compare une chane au nom de l'enseignant en ignorant la casse et les accents et  son mot de passe.
 */

bool Enseignant::pw_compare (char* s)
{
	char*	p;

	for (p = _nom; *s != ':' && *s != 0; ++p, ++s)
		if (iso2ascii [(unsigned char) *s] != iso2ascii [(unsigned char) *p])
			return false;

	if (*p != 0)
		return false;		// mauvais nom.

	return (0 == strcmp (_passwd, ++s));
}

/*
 *	Compare une chane au nom de l'enseignant en ignorant la casse et les accents.
 */

bool Enseignant::compare (char* s)
{
	char*	p;

	for (p = _nom; *s != ':' && *s != 0; ++p, ++s)
		if (iso2ascii [(unsigned char) *s] != iso2ascii [(unsigned char) *p])
			return false;

	return (*p == 0);	// test final.
}

/*
 *	Fonction de hash-code pour les enseignants.
 */

int Enseignant::hash (char* name, int size)
{
	int	h = 11;

	for (int n = strlen (name); --n >= 0;)
		h = ((h * 17) ^ (int)(*name++)) % size;

	if (h < 0)
		h += size;
	return h;
}

/*
 *	Recherche un enseignant par son nom.
 */

Enseignant* Enseignant::name2prof (char* name, Enseignant* prof)
{
	int		h;

	name = nospaces (name);
	h = hash (name, MAX_PROF);
	for (int i = MAX_PROF; --i >= 0;) {
		if (_enseignants [h] == 0) {
			if (prof == 0)
				return 0;	/* pas trouv */
			/* on le met dans la table */
			_enseignants [h] = prof;
			return prof;
		}
		if (_enseignants [h] -> compare (name))
			return _enseignants [h];

		h = (h + 41) % MAX_PROF;	/* collision */
	}
	return 0;	/* plus de place dans la table */
}

/*
 *	Retourne la catgorie sous forme de chane.
 */

char* Enseignant::cat2str (bool format_long)
{
	static const char*	annee [] = {
		"premire", "deuxime", "troisime"
	};
	static char	tc [128];
	static char	demi_ater [] = "1/2 ATER";

	switch (_categorie) {
		case PCA:	return (char*)"Pas cette anne";
		case PR:	return format_long ? (char*) "Professeur" : (char*) "PR";
		case MC:	return format_long ? (char*) "Matre de Confrences" : (char*) "MC";
		case PRAG:	return (char*) "PRAG";
		case PAST:	return (char*) "PAST";
		case CONTR:	return (char*) "Contractuel";
		case PRCE:	return format_long ? (char*) "Professeur Certifi" : (char*) "PRCE";
		case VAC:	return format_long ? _poste : (char*) "Vacataire";
		case CR:	return format_long ? (char*) "Charg de Recherche" : (char*) "CR";
		case DR:	return format_long ? (char*) "Directeur de Recherche" : (char*) "DR";
		case IR:	return format_long ? (char*) "Ingnieur de Recherche" : (char*) "IR";
		case IE:	return format_long ? (char*) "Ingnieur d'tude" : (char*) "IE";
		case TECH:	return (char*) "Technicien";
		case MON:
			if (format_long)
				sprintf (tc, "Moniteur %s anne", annee [_annee-1]);
			else
				sprintf (tc, "MON-%1.1d", _annee);
			return tc;
		case AMN:
			if (format_long)
				sprintf (tc, "Moniteur %s anne", annee [_annee-1]);
			else
				sprintf (tc, "AMN-%1.1d", _annee);
			return tc;
		case DEMI_ATER:
			return demi_ater;
		case ATER:
			return demi_ater + 4;
		case ATE:
			return (char*) "ATE";
	}
	return (char*) "??";
}

/*
 *	Calcule le potentiel d'un enseignant...
 */

#if HEURES_FLOAT
void Enseignant::potentiel (
		float& sstat_th, float& dech_st, char*& intit_dech, float& serv_du,
		float& hpres_ufr, float& hpres_aut, const char*& aut_comp,
		float& hpedag, float& hadm, float& sfait, float& hcc_ufr, char* orig)
{
	// service statutaire thorique.
	sstat_th = 0;
	switch (_employeur -> _type) {
		case  UNIVERSITE:
			sprintf (orig, "%s", _dept -> _nom);
			if (_dept -> _a_nous || _dept -> search ("Polytech"))
				switch (_categorie) {
					case MON:
					case AMN:	sstat_th = 64; break;
					case ATER:	sstat_th = 192; break;
					case ATE:	sstat_th = 384; break;
					case DEMI_ATER:	sstat_th = 96; break;
					case PAST:	sstat_th = 96; break;
					case CONTR:	sstat_th = 384; break;
					default:	sstat_th = 192; break;
				}
			break;
		case BOURSIER:
			sprintf (orig, "Boursier %s",
				_employeur -> _nom [0] == '_'
				 ? _employeur -> _nom + 1		// ne pas afficher le soulign.
				 : _employeur -> _nom );
			if (_categorie == MON || _categorie == AMN)
				sstat_th = 64;
			break;
		case FONCTIONNAIRE:
		case PRIVE:
			sprintf (orig, "%s", _employeur -> _nom);
			break;
		default:
			break;
	}
	// dcharge statutaire.
	dech_st = sstat_th - _serv_annuel;
	// son intitul.
	if (dech_st > 0) {
		static char	buffer [128];
		char*	s = disponibilite [_disponibilite]._intitule;

		for (char* d = buffer; 0 != (*d = *s); ++s, ++d)
			if (*s == '&') {
				*d = '<';	//  faire pour tous les codes HTML...
				while (*++s != ';')
					;
			}

		intit_dech = buffer;
	}
	// service d.
	serv_du = _serv_annuel;
	// le service dtaiil.
	hpres_ufr = 0;	// heures prsentielles AP0, AP2A  l'UFR.
	hpres_aut = 0;	// heures prsentielles AP0, AP2A ailleurs (Polytech, IUT)
	hpedag = 0;		// heures rfrentiel lies  la pdagogie AP1, AP2B, AP3
	hadm = 0;		// heures rfrentiel lies  l'administration AE, AA
	sfait = 0;		// service fait.
	hcc_ufr = 0;	// HCC verses  l'enseignant.
	for (int i = 0; i < _service.length (); ++i) {
		Service*	s = (Service*) _service [i];
		// enseigement fait  Polytech.
		if (0 == strncmp ("POLYTECH", s -> _ens -> _groupe -> _filiere -> _identification, 8))
		{
			if (0 == strncmp (s -> _ens -> _activite_pedag, "AP2", 3) ||
				0 == strncmp (s -> _ens -> _activite_pedag, "AP0", 3))
			{
				hpres_aut += s -> eq_td (HEURE_STAT_A) + s -> eq_td (HEURE_STAT_B)
							+ s -> eq_td (HEURE_COMP_A) + s -> eq_td (HEURE_COMP_B)
							+ s -> eq_td (HEURE_COMP_EXT) + s -> eq_td (HEURE_STAT_EXT)
							+ s -> eq_td (HEURE_GRAT);
				aut_comp = "Polytech";
			}
		}
		// enseigement fait  l'IUT.
		else
		if (0 == strncmp ("IUT", s -> _ens -> _groupe -> _identification, 3))
		{
			if (0 == strncmp (s -> _ens -> _activite_pedag, "AP2", 3) ||
				0 == strncmp (s -> _ens -> _activite_pedag, "AP0", 3))
			{
				hpres_aut += s -> eq_td (HEURE_STAT_A) + s -> eq_td (HEURE_STAT_B)
							+ s -> eq_td (HEURE_COMP_A) + s -> eq_td (HEURE_COMP_B)
							+ s -> eq_td (HEURE_COMP_EXT) + s -> eq_td (HEURE_STAT_EXT)
							+ s -> eq_td (HEURE_GRAT);
				aut_comp = "IUT";
			}
		}
		// enseigement fait  l'UFR
		else {
			if (0 == strncmp (s -> _ens -> _activite_pedag, "AP2", 3) ||
				0 == strncmp (s -> _ens -> _activite_pedag, "AP0", 3))
			{
				hpres_ufr += s -> eq_td (HEURE_STAT_A) + s -> eq_td (HEURE_STAT_B)
							+ s -> eq_td (HEURE_COMP_A) + s -> eq_td (HEURE_COMP_B)
							+ s -> eq_td (HEURE_COMP_EXT) + s -> eq_td (HEURE_STAT_EXT)
							+ s -> eq_td (HEURE_GRAT);
			}
		}
		// heures rfrentiel lies  la pdagogie AP1, AP2B, AP3.
		if (0 == strncmp (s -> _ens -> _activite_pedag, "AP1", 3) ||
			0 == strncmp (s -> _ens -> _activite_pedag, "AP3", 3))
		{
			hpedag += s -> eq_td (HEURE_STAT_A) + s -> eq_td (HEURE_STAT_B)
					+ s -> eq_td (HEURE_COMP_A) + s -> eq_td (HEURE_COMP_B)
					+ s -> eq_td (HEURE_COMP_EXT) + s -> eq_td (HEURE_STAT_EXT)
					+ s -> eq_td (HEURE_GRAT);
		}
		// heures rfrentiel lies  l'administration AE, AA
		else
		if (0 == strncmp (s -> _ens -> _activite_pedag, "AA", 2) ||
			0 == strncmp (s -> _ens -> _activite_pedag, "AE", 2))
		{
			hadm += s -> eq_td (HEURE_STAT_A) + s -> eq_td (HEURE_STAT_B)
					+ s -> eq_td (HEURE_COMP_A) + s -> eq_td (HEURE_COMP_B)
					+ s -> eq_td (HEURE_COMP_EXT) + s -> eq_td (HEURE_STAT_EXT)
					+ s -> eq_td (HEURE_GRAT);
		}
		// accumuler les heures statutaires effectue.
		hcc_ufr += s -> eq_td (HEURE_COMP_A) + s -> eq_td (HEURE_COMP_B);
		// et le total gnral.
		sfait += s -> eq_td (HEURE_STAT_A) + s -> eq_td (HEURE_STAT_B)
			   + s -> eq_td (HEURE_COMP_A) + s -> eq_td (HEURE_COMP_B)
			   + s -> eq_td (HEURE_COMP_EXT) + s -> eq_td (HEURE_STAT_EXT)
			   + s -> eq_td (HEURE_GRAT);
	}
}

#else

void Enseignant::potentiel (
		int& sstat_th, int& dech_st, char*& intit_dech, int& serv_du,
		int& hpres_ufr, int& hpres_aut, const char*& aut_comp,
		int& hpedag, int& hadm, int& sfait, int& hcc_ufr, char* orig)
{
	// service statutaire thorique.
	sstat_th = 0;
	switch (_employeur -> _type) {
		case  UNIVERSITE:
			sprintf (orig, "%s", _dept -> _nom);
			if (_dept -> _a_nous || _dept -> search ("Polytech"))
				switch (_categorie) {
					case MON:
					case AMN:	sstat_th = 64; break;
					case ATER:	sstat_th = 192; break;
					case DEMI_ATER:	sstat_th = 96; break;
					case ATE:	sstat_th = 384; break;
					case PAST:	sstat_th = 96; break;
					case CONTR:	sstat_th = 384; break;
					default:	sstat_th = 192; break;
				}
			break;
		case BOURSIER:
			sprintf (orig, "Boursier %s",
				_employeur -> _nom [0] == '_'
				 ? _employeur -> _nom + 1		// ne pas afficher le soulign.
				 : _employeur -> _nom );
			if (_categorie == MON || _categorie == AMN)
				sstat_th = 64;
			break;
		case FONCTIONNAIRE:
		case PRIVE:
			sprintf (orig, "%s", _employeur -> _nom);
			break;
		default:
			break;
	}
	// dcharge statutaire.
	dech_st = sstat_th - _serv_annuel;
	// son intitul.
	if (dech_st > 0)
		intit_dech = disponibilite [_disponibilite]._intitule;
	// service d.
	serv_du = _serv_annuel;
	// le service dtaiil.
	hpres_ufr = 0;	// heures prsentielles AP0, AP2A  l'UFR.
	hpres_aut = 0;	// heures prsentielles AP0, AP2A ailleurs (Polytech, IUT)
	hpedag = 0;		// heures rfrentiel lies  la pdagogie AP1, AP2B, AP3
	hadm = 0;		// heures rfrentiel lies  l'administration AE, AA
	sfait = 0;		// service fait.
	hcc_ufr = 0;	// HCC verses  l'enseignant.
	for (int i = 0; i < _service.length (); ++i) {
		Service*	s = (Service*) _service [i];

		// enseigement fait  Polytech.
		if (0 == strncmp ("POLYTECH", s -> _ens -> _groupe -> _filiere -> _identification, 8))
		{
			if (0 == strncmp (s -> _ens -> _activite_pedag, "AP2", 3) ||
				0 == strncmp (s -> _ens -> _activite_pedag, "AP0", 3))
			{
				hpres_aut += s -> eq_td (HEURE_STAT_A) + s -> eq_td (HEURE_STAT_B)
							+ s -> eq_td (HEURE_COMP_A) + s -> eq_td (HEURE_COMP_B)
							+ s -> eq_td (HEURE_COMP_EXT) + s -> eq_td (HEURE_STAT_EXT)
							+ s -> eq_td (HEURE_GRAT);
				aut_comp = "Polytech";
			}
		}
		// enseigement fait  l'IUT.
		else
		if (0 == strncmp ("IUT", s -> _ens -> _groupe -> _identification, 3))
		{
			if (0 == strncmp (s -> _ens -> _activite_pedag, "AP2", 3) ||
				0 == strncmp (s -> _ens -> _activite_pedag, "AP0", 3))
			{
				hpres_aut += s -> eq_td (HEURE_STAT_A) + s -> eq_td (HEURE_STAT_B)
							+ s -> eq_td (HEURE_COMP_A) + s -> eq_td (HEURE_COMP_B)
							+ s -> eq_td (HEURE_COMP_EXT) + s -> eq_td (HEURE_STAT_EXT)
							+ s -> eq_td (HEURE_GRAT);
				aut_comp = "IUT";
			}
		}
		// enseigement fait  l'UFR
		else {
			if (0 == strncmp (s -> _ens -> _activite_pedag, "AP2", 3) ||
				0 == strncmp (s -> _ens -> _activite_pedag, "AP0", 3))
			{
				hpres_ufr += s -> eq_td (HEURE_STAT_A) + s -> eq_td (HEURE_STAT_B)
							+ s -> eq_td (HEURE_COMP_A) + s -> eq_td (HEURE_COMP_B)
							+ s -> eq_td (HEURE_COMP_EXT) + s -> eq_td (HEURE_STAT_EXT)
							+ s -> eq_td (HEURE_GRAT);
			}
		}
		// heures rfrentiel lies  la pdagogie AP1, AP2B, AP3.
		if (0 == strncmp (s -> _ens -> _activite_pedag, "AP1", 3) ||
			0 == strncmp (s -> _ens -> _activite_pedag, "AP3", 3))
		{
			hpedag += s -> eq_td (HEURE_STAT_A) + s -> eq_td (HEURE_STAT_B)
					+ s -> eq_td (HEURE_COMP_A) + s -> eq_td (HEURE_COMP_B)
					+ s -> eq_td (HEURE_COMP_EXT) + s -> eq_td (HEURE_STAT_EXT)
					+ s -> eq_td (HEURE_GRAT);
		}
		// heures rfrentiel lies  l'administration AE, AA
		else
		if (0 == strncmp (s -> _ens -> _activite_pedag, "AA", 2) ||
			0 == strncmp (s -> _ens -> _activite_pedag, "AE", 2))
		{
			hadm += s -> eq_td (HEURE_STAT_A) + s -> eq_td (HEURE_STAT_B)
					+ s -> eq_td (HEURE_COMP_A) + s -> eq_td (HEURE_COMP_B)
					+ s -> eq_td (HEURE_COMP_EXT) + s -> eq_td (HEURE_STAT_EXT)
					+ s -> eq_td (HEURE_GRAT);
		}
		// accumuler les heures statutaires effectue.
		hcc_ufr += s -> eq_td (HEURE_COMP_A) + s -> eq_td (HEURE_COMP_B);
		// et le total gnral.
		sfait += s -> eq_td (HEURE_STAT_A) + s -> eq_td (HEURE_STAT_B)
			   + s -> eq_td (HEURE_COMP_A) + s -> eq_td (HEURE_COMP_B)
			   + s -> eq_td (HEURE_COMP_EXT) + s -> eq_td (HEURE_STAT_EXT)
			   + s -> eq_td (HEURE_GRAT);
	}
}
#endif

/*
 *	Affiche un enseignant qui enseigne dans une filire donne
 *	(format table colore selon le type d'heure)
 */

float Enseignant::service_filiere (Filiere* f, int th, bool tous, Periode periode)
{
	float	vol [5];
	static char	format [] =
		"<tr bgcolor=\"%s\">"
			"<td width=\"40%%\"><a href=\"service_%s.cmd\">%s</a></td>"
			"<td width=\"12%%\">%s</td>"
			"<td bgcolor=\"%s\" width=\"10%%\">%g</td>"
			"<td bgcolor=\"%s\" width=\"10%%\">%g</td>"
			"<td bgcolor=\"%s\" width=\"10%%\">%g</td>"
			"<td bgcolor=\"%s\" width=\"10%%\">%g</td>"
			"<td bgcolor=\"%s\" width=\"10%%\">%g</td>"
		"</tr>\n";

	vol [0] = vol [1] = vol [2] = vol [3] = vol [4] = 0.;
	for (int i = 0; i < _service.length (); ++i) {
		Service*	s = (Service*) _service [i];

		if (s -> _ens -> _groupe -> _filiere != f)
			continue;	// pas la bonne filire.

		if (periode == SEMESTRE_UN && 0 != Per2Semestre (s -> _ens -> _periode))
				continue;	// pas la priode demande.

		if (periode == SEMESTRE_DEUX && 1 != Per2Semestre (s -> _ens -> _periode))
				continue;	// pas la priode demande.

		vol [0] += s -> eq_td (HEURE_STAT_A, false) + s -> eq_td (HEURE_STAT_B, false);
		vol [1] += s -> eq_td (HEURE_COMP_A, false) + s -> eq_td (HEURE_COMP_B, false);
		vol [2] += s -> eq_td (HEURE_STAT_EXT, false);
		vol [3] += s -> eq_td (HEURE_COMP_EXT, false);
		vol [4] += s -> eq_td (HEURE_GRAT, false);
	}
	// afficher ce service en coloriant le type d'heures slectionn.
	const char*	bg_col = vol [th] > 0. ? "#202020" : "#604060";

	if (tous || vol [th] != 0.)
		_outp_ += sprintf (_outp_, format,
								bg_col,
								nospaces (), _nom, cat2str (),
								(th == 0 && vol [0] != 0.) ? "#00aadd" : bg_col, vol [0],
								(th == 1 && vol [1] != 0.) ? "#00aadd" : bg_col, vol [1],
								(th == 2 && vol [2] != 0.) ? "#00aadd" : bg_col, vol [2],
								(th == 3 && vol [3] != 0.) ? "#00aadd" : bg_col, vol [3],
								(th == 4 && vol [4] != 0.) ? "#00aadd" : bg_col, vol [4]);
	return vol [th];
}

/*
 *	Affiche le service d'un enseignant:
 *		d'abord les enseignements,
 *		puis les charges.
 */

bool Enseignant::affiche_service (Editable editable, bool is_mandator)
{
	static char fmt_total_enseignement_1 [] =
		"<tr><td><b>Total :</b></td></tr>\n";
	static char fmt_total_enseignement_2 [] =
		"<tr>"
			"<td></td><td align=right><b>%g</b></td>"
			"<td width=20></td><td><b>heures %s</td></b>"
		"</tr>\n";
	static char fmt_total_ch [] =
		"<tr>"
			"<td></td><td></td>"
			"<td width=20></td><td><font color=#e0ffe0>(dont %g heures de charge)</font></td>"
		"</tr>\n";
	static char fmt_total_enseignement_3 [] =
		"Les lignes en <i>italique</i> sont les seconds choix.<br>\n";
	static char fmt_total_enseignement_4 [] =
		"L'astrisque indique que vous tes prioritaire sur cet enseignement.<p>\n";
	bool		avec_priorite = false;
	char		tc [128];
	float		total [N_HRS_TYPES/2];
	float&		total_stat = total [HEURE_STAT_A/2];
	float&		total_comp = total [HEURE_COMP_A/2];
	float&		total_comp_ext = total [HEURE_COMP_EXT/2];
	float&		total_stat_ext = total [HEURE_STAT_EXT/2];
	float&		total_grat = total [HEURE_GRAT/2];
	float		total_charge = 0.;
	float		f;
	bool		avec_second_choix = false;
	bool		avec_charges = false;
	bool		ret_val = false;
	Service*	s;

	for (int i = 0; i < N_HRS_TYPES/2; ++i)
		total [i] = 0.;
	// d'abord sans les charges.
	for (int i = 0; i < _service.length (); ++i) {
		s = (Service*) _service [i];
		if (s -> _ens -> _charge || s -> _ens -> _charge_np) {
			avec_charges = true;
			continue;
		}
		if (!avec_priorite && (s -> _prioritaire [COURS] || s -> _prioritaire [TD] ||
			s -> _prioritaire [TP] || s -> _prioritaire [MP] || s -> _prioritaire [DTD] ||
			s -> _prioritaire [STD] || s -> _prioritaire [SSS]))
		{
			avec_priorite = true;
		}
		s -> affiche (this, editable, is_mandator);
		for (int th = 0; th < N_HRS_TYPES; th += 2) {
			if ((f = s -> eq_td (th+1)) > 0.)
				avec_second_choix = true;
			total [th/2] += s -> eq_td (th) + f;
		}
	}
	// puis les charges  la fin.
	if (avec_charges) {
		_outp_ += sprintf (_outp_,
					"<tr><td></td><td></td><td></td><td><b><font color=#00d0d0>"
					"Charges spcifiques</font></b></td></tr>\n");
		for (int i = 0; i < _service.length (); ++i) {
			s = (Service*) _service [i];
			// accumuler les charges.
			if (!s -> _ens -> _charge_np && !s -> _ens -> _charge)
				continue;

			s -> affiche (this, editable, is_mandator);
			for (int th = HEURE_STAT_A; th <= HEURE_GRAT; th += 2) {
				total_charge += s -> _heures [th][TD];
				total [th/2] += s -> _heures [th][TD];
			}
		}
	}
	// n'afficher le total que s'il est non nul.
	if (total_stat + total_comp + total_comp_ext + total_stat_ext + total_grat != 0.) {
		_outp_ += sprintf (_outp_, "%s", fmt_total_enseignement_1);
		if (total_stat > 0.) {
			sprintf (tc, "statutaires");
			_outp_ += sprintf (_outp_, fmt_total_enseignement_2, total_stat, tc);
		}
		if (total_comp != 0.) {
			float	total_mandate = 0.;

			if (editable == EDIT_MANDATEMENTS) {
				for (int n = 0; n < _mandatement.length (); ++n) {
					Mandatement* m = (Mandatement*) _mandatement [n];

					total_mandate += MAND_EQTD (m);
				}
				sprintf (tc, "complmentaires (dont %g mandates)", total_mandate);
				ret_val = (total_comp > total_mandate); 
			} else
				sprintf (tc, "complmentaires");
			_outp_ += sprintf (_outp_, fmt_total_enseignement_2, total_comp, tc);
		}
		if (total_comp_ext + total_stat_ext + total_grat != 0.)
			_outp_ += sprintf (_outp_, fmt_total_enseignement_2,
							total_comp_ext + total_stat_ext + total_grat,
							"extrieures ou bnvoles");
		if (total_charge != 0.)
			_outp_ += sprintf (_outp_, fmt_total_ch, total_charge);
	}
	_outp_ += sprintf (_outp_, "</table>\n");
	if (avec_second_choix)
		_outp_ += sprintf (_outp_, "%s", fmt_total_enseignement_3);

	if (avec_priorite)
		_outp_ += sprintf (_outp_, "%s", fmt_total_enseignement_4);

	return ret_val;
}

/*
 *	Affiche le service d'un enseignant au format dtail.
 */

void Enseignant::affiche_detail (void)
{
	_outp_ += sprintf (_outp_, "<p><hr><font size=+1><b>%s</b></font><p>\n", _nom);
	_outp_ += sprintf (_outp_, "<table cellspacing=0 cellpadding=2 border=0>\n");
	// afficher le service.
	(void) affiche_service (NON_EDITABLE, false);	// a priori on ignre le fait d'etre mandator...
}

/*
 *	Affiche un enseignant au format liste.
 */

void Enseignant::affiche_liste (int largeur, int* sstat, int* sfait, bool clair, int* spolytech, int* scfa)
{
	float		total = 0.;
	float		total_stat = 0.;
	float		total_polytech = 0.;
	float		total_polytech_stat = 0.;
	float		total_cfa = 0.;
	char		orig [128];

	// Format lambda: sans Polytech ni CFA.
	static char	fmt_liste_lambda [] =
		"<tr>\n"
		"	<td width=%d>\n"
		"		<table bgcolor=\"%s\" width=%d cellspacing=0 cellpadding=0 border=0>\n"
		"		<tr><td valign=top align=left>%s<a href=\"service_%s.cmd\">%s %s</a></td></tr>\n"
		"		</table>\n"
		"	</td>\n"
		"	<td bgcolor=\"%s\" align=center><font color=#000000>%s</td>\n"
		"	<td width=25%% bgcolor=\"%s\" align=left><font color=#000000>&nbsp;%s <font size=-3>%s</td>\n"
		"	<td bgcolor=\"%s\" align=right><font color=#000000>%d&nbsp;</td>\n"
		"	<td bgcolor=\"%s\" align=right><font color=#000000>%d&nbsp;</td>\n"
		"	<td bgcolor=\"%s\" align=right><font color=#000000>%d&nbsp;</td>\n"
		"	<td width=12></td>\n"
		"</tr>\n";

	// Format avec colonnes supplmentaires (Polytech et CFA).

	static char	fmt_liste [] =
		"<tr>\n"
		"	<td width=%d>\n"
		"		<table bgcolor=\"%s\" width=%d cellspacing=0 cellpadding=0 border=0>\n"
		"		<tr><td valign=top align=left>%s<a href=\"service_%s.cmd\">%s %s</a></td></tr>\n"
		"		</table>\n"
		"	</td>\n"
		"	<td bgcolor=\"%s\" align=center><font color=#000000>%s</td>\n"
		"	<td width=25%% bgcolor=\"%s\" align=left><font color=#000000>&nbsp;%s <font size=-3>%s</td>\n"
		"	<td bgcolor=\"%s\" align=right><font color=#000000>%d&nbsp;</td>\n"
		"	<td bgcolor=\"%s\" align=right><font color=#000000>%d&nbsp;</td>\n"
		"	<td bgcolor=\"%s\" align=right><font color=#000000>%d&nbsp;</td>\n"
		"	<td bgcolor=\"%s\" align=right><font color=#000000>%d&nbsp;</td>\n"
		"	<td bgcolor=\"%s\" align=right><font color=#000000>%d&nbsp;</td>\n"
		"	<td width=12></td>\n"
		"</tr>\n";

	// l'origine de l'enseignant.
	switch (_employeur -> _type) {
		case UNIVERSITE:
			sprintf (orig, "%s", _dept -> _nom);
			break;
		case FONCTIONNAIRE:
			sprintf (orig, "%s", _employeur -> _nom);
			break;
		case PRIVE:
			sprintf (orig, "%s", _employeur -> _nom);
			break;
		case BOURSIER:
			sprintf (orig, "Boursier %s",
				_employeur -> _nom [0] == '_'
				 ? _employeur -> _nom + 1		// ne pas afficher le soulign.
				 : _employeur -> _nom );
			break;
	}

	for (int i = 0; i < _service.length (); ++i) {
		Service*	s = (Service*) _service [i];
		bool	moniteur_polytech = false;
		char	nom_doctorant [128];
		// est-ce un tutorat de doctorant Polytech?
		if (0 == strncmp (s -> _ens -> _groupe -> _identification, "Tutorat", 7) &&
			s -> _ens -> _intitule != 0 &&
			0 == strncmp (s -> _ens -> _intitule, "Tutorat de ", 11))
		{
			strcpy (nom_doctorant, s -> _ens -> _intitule+11);
			Enseignant*	moniteur = name2prof (nom_doctorant);

			moniteur_polytech = (moniteur != 0) && moniteur -> _dept -> search ("Polytech");
		}
		// accumuler les heures faites  Polytech.
		if (0 == strncmp ("POLYTECH", s -> _ens -> _groupe -> _filiere -> _identification, 8)
		||	0 == strncmp ("CH5", s -> _ens -> _groupe -> _identification, 3)
		||	moniteur_polytech)
		{
			total_polytech_stat += s -> eq_td (HEURE_STAT_A) + s -> eq_td (HEURE_STAT_B)
								 + s -> eq_td (HEURE_STAT_EXT);
			total_polytech += s -> eq_td (HEURE_STAT_A) + s -> eq_td (HEURE_STAT_B)
							+ s -> eq_td (HEURE_COMP_A) + s -> eq_td (HEURE_COMP_B)
							+ s -> eq_td (HEURE_COMP_EXT) + s -> eq_td (HEURE_STAT_EXT)
							+ s -> eq_td (HEURE_GRAT) + s -> eq_td (HEURE_GRAT_B);
		}
		// accumuler les heures faites en CFA.
		if (0 == strncmp ("_CFA",
				s -> _ens -> _groupe -> _filiere -> _identification - 4 +
				strlen (s -> _ens -> _groupe -> _filiere -> _identification), 4)
		||	0 == strncmp ("CHARGES-CFA-MIAGE", s -> _ens -> _groupe -> _identification, 17))
		{
			total_cfa += s -> eq_td (HEURE_STAT_A) + s -> eq_td (HEURE_STAT_B)
					   + s -> eq_td (HEURE_COMP_A) + s -> eq_td (HEURE_COMP_B)
					   + s -> eq_td (HEURE_COMP_EXT) + s -> eq_td (HEURE_STAT_EXT)
					   + s -> eq_td (HEURE_GRAT) + s -> eq_td (HEURE_GRAT_B);
		}
		// accumuler les heures statutaires effectue.
		total_stat += s -> eq_td (HEURE_STAT_A) + s -> eq_td (HEURE_STAT_B)
			   		+ s -> eq_td (HEURE_STAT_EXT);
		// et le total gnral.
		total += s -> eq_td (HEURE_STAT_A) + s -> eq_td (HEURE_STAT_B)
			   + s -> eq_td (HEURE_COMP_A) + s -> eq_td (HEURE_COMP_B)
			   + s -> eq_td (HEURE_COMP_EXT) + s -> eq_td (HEURE_STAT_EXT)
			   + s -> eq_td (HEURE_GRAT) + s -> eq_td (HEURE_GRAT_B);
	}
	if (_tous || (_employeur -> _type == UNIVERSITE
					&& (_dept -> _a_nous || _dept -> search ("Polytech")))
			  || total > 0.)
	{
		char	tc [256];
		// Extraire le prnom de l'enseignant.
		char	prenom [128];
		char	nom [128];

		nom_prenom (nom, prenom);
		// mettre le nom en capitales.
		for (unsigned char* cp = (unsigned char*) nom; *cp != 0; ++cp)
			if (('a' <= *cp && *cp <= 'z') || 0340 <= *cp)
				*cp -= 'a' - 'A';

		if (_email [0] != 0)
			sprintf (tc, "<a href=\"mailto:%s\"><img src=email-trans.gif align=top border=0></a> &nbsp;", _email);
		else
			sprintf (tc, "<img src=trans.gif align=top border=0> &nbsp;");
		// alterner les couleurs de fond: clair/sombre.
		colors	fond_titre = clair ? TitreNormalClair : TitreNormal;
		colors	fond_liste = /* clair ? ListeClair : */ Liste;
		// plus du tiers du service statutaire d'un titulaire du dept. Info. fait  Polytech?
		if (_employeur -> _type == UNIVERSITE && _dept -> _a_nous &&
			(spolytech != 0 || scfa != 0) &&
			_serv_annuel > 0 && total_polytech_stat > (_serv_annuel / 3.))
		{
			fond_titre = ListeClair;	// oui, le mettre en exergue sur fond violet...
		}
		_outp_ += sprintf (_outp_,
						(spolytech == 0 || scfa == 0) ? fmt_liste_lambda : fmt_liste,
						largeur+50, page_colors [fond_liste],
						largeur+36,
						tc,
						nospaces (),
#ifdef PRENOM_NOM
						prenom, nom,
#else
						nom, prenom,
#endif
						page_colors [fond_titre], orig,
			 			page_colors [fond_titre], cat2str (), disponibilite [(int)_disponibilite]._intitule,
						page_colors [fond_titre], _serv_annuel,
						page_colors [fond_titre], (int) (total+0.5),
						page_colors [fond_titre], ((int) (total+0.5) - _serv_annuel),
						page_colors [fond_titre], (int) (total_polytech+0.5),
						page_colors [fond_titre], (int) (total_cfa+0.5));
		if (sstat != 0) *sstat = _serv_annuel;
		if (sfait != 0) *sfait = (int) (total+0.5);
		if (spolytech != 0) *spolytech = (int) (total_polytech+0.5);
		if (scfa != 0) *scfa = (int) (total_cfa+0.5);
	}
	// ne pas considrer le service si on ne l'affiche pas (vite les erreurs dans les totaux).
	else {
		if (sstat != 0) *sstat = 0;
		if (sfait != 0) *sfait = 0;
		if (spolytech != 0) *spolytech = 0;
		if (scfa != 0) *scfa = 0;
	}
}

/*
 *	Affiche le service d'un enseignant au format page (avec saisie du service).
 */

static char fmt_link [] = "service_%s.cmd";
static char fmt_ens [] =
		"<td align=center height=40 bgcolor=\"%s\">"
		"<a href=\"mailto:%s\"><font size=4 color=#000000 face=Helvetica>"
		"%s</a> (%s)";

static char	fmt_avec_etat_stat [] =
				"</td><td align=center height=40 bgcolor=\"%s\">"

				"<font size=2 color=#FFFF00> Stat-Comp. <a href=\"etat_stat_%s.pcf\"><font color=#FFFF00 size=1>PDF</font></a> / "
				"<a href=\"etat_stat_%s.cp?\"><font color=#FFFF00 size=1>PS</font></a>"
				"&nbsp;&nbsp;"
				"<font size=2 color=#FFFF00> Comp-Ext. <a href=\"etat_stat_ext_%s.pcf\"><font color=#FFFF00 size=1>PDF</font></a> / "
				"<a href=\"etat_stat_ext_%s.cp?\"><font color=#FFFF00 size=1>PS</font></a>"
				"&nbsp;&nbsp;"
				"Prv. <a href=\"etat_prev_%s.pcf\"><font color=#FFFF00 size=1>PDF</font></a> / "
				"<a href=\"etat_prev_%s.cp?\"><font color=#FFFF00 size=1>PS</font></a>"
				"&nbsp;&nbsp;"
				"Hybr. <a href=\"etat_hybride_%s.pcf\"><font color=#FFFF00 size=1>PDF</font></a> / "
				"<a href=\"etat_hybride_%s.cp?\"><font color=#FFFF00 size=1>PS</font></a>"
				"&nbsp;&nbsp;"
				"Faits <a href=\"att_service_%s.pcf\"><font color=#FFFF00 size=1>PDF</font></a> / "
				"<a href=\"att_service_%s.cp?\"><font color=#FFFF00 size=1>PS</font></a>&nbsp;";

void Enseignant::affiche_page (Editable editable)
{
#if WITH_CHG_PASSWD
	static char fmt_chg_passwd [] =
		"<p><a href=\"passwd_%s.cmd\">Changer de mot de passe</a><p>\n";
#endif
	static char fmt_page_post_1 [] =
		"<form method=post action=\"add_request\">\n"
		"<input type=hidden name=N value=\"%s\">"
		"La premire fois que vous voudrez modifier votre service une identification vous sera "
		"demande. <br><b><font color=red>Le champ 'User ID' est  remplir par votre prnom suivi d'un "
		"espace suivi de votre nom</font> (avec ou sans accents, en majuscules ou minuscules)</b>."
		" Le mot de passe vous a t transmis avec le mail gnral indiquant l'ouverture du serveur.<p>"
		"<b><input type=button onClick='submit()' value=\"Ajouter\"></b> un enseignement  votre service."
		"</form>\n"
		"<p>\n"
		"<form method=post action=\"remove_request\">\n"
		"<input type=hidden name=P value=\"%s\">\n";

	static char fmt_page_post_2 [] =
			"<p><input type=button onClick='submit()' value=\"Supprimer\">"
			" les enseignements slectionns.</form>\n";

	char	link_prev [128];
	char	link_next [128];
	char	tc [4096], *ptc;
	Enseignant*	prec = precedent ();
	Enseignant*	suiv = suivant ();

	sprintf (link_prev, fmt_link, prec -> nospaces ());
	sprintf (link_next, fmt_link, suiv -> nospaces ());
	ptc = tc + sprintf (tc, fmt_ens, page_colors [TitreNormal], _email, _nom, cat2str ());
	if ((_employeur -> _type == UNIVERSITE && (_dept -> _a_nous || get_type () == POLYTECH))
		|| _categorie == MON)
	{
		ptc += sprintf (ptc, fmt_avec_etat_stat, page_colors [TitreEmphase],
						nospaces (), nospaces (),
						nospaces (), nospaces (),
						nospaces (), nospaces (),
						nospaces (), nospaces (),
						nospaces (), nospaces ());
	}
	ptc += sprintf (ptc, "</td>\n");
	TNG_arrow_title (link_prev, link_next, (char*)"", tc);
	_outp_ += sprintf (_outp_, "<p>\n");

#if WITH_CHG_PASSWD
	_outp_ += sprintf (_outp_, fmt_chg_passwd, nospaces ());
#endif

	// le formulaire en mode dition ou si on est superuser.
	if (editable == EDIT_SERVICE)
		_outp_ += sprintf (_outp_, fmt_page_post_1, _nom, _nom);
	_outp_ += sprintf (_outp_, "<table cellspacing=0 cellpadding=2 border=0>\n");
	// afficher le service.
	(void) affiche_service (editable, false);	// a priori on ignre le fait d'etre mandator.
	// mettre le bouton 'supprimer' si le serveur est en mode dition ou si on est superuser.
	if (editable == EDIT_SERVICE)
		_outp_ += sprintf (_outp_, "%s", fmt_page_post_2);
}

/*
 *	Affiche la page d'un enseignant pour diter des mandatements.
 */

void Enseignant::affiche_page_mandatements (void)
{
	static char fmt_page_post_1 [] =
		"<form method=post action=select_mandatement>\n"
		"<input type=hidden name=P value=\"%s\"><p>\n";

	static char fmt_page_post_2 [] =
			"<b><input type=button onClick='submit()' value=\"Mandater\">"
			"</b> les enseignements slectionns.";

	char	link_prev [256];
	char	link_next [256];
	char	tc [4096], *ptc;

	// chercher d'abord s'il y a des heures complmentaires pour pouvoir les mandater.
	for (int i = 0; i < _service.length (); ++i)
		if (((Service*) _service [i]) -> eq_td (HEURE_COMP_A) > 0.)
			goto mandater;

	// s'il n'y en a pas c'est comme une page normale.
	affiche_page (NON_EDITABLE);
	return;

mandater:
	Enseignant*	prec = precedent ();
	Enseignant*	suiv = suivant ();

	sprintf (link_prev, fmt_link, prec -> nospaces ());
	sprintf (link_next, fmt_link, suiv -> nospaces ());
	ptc = tc + sprintf (tc, fmt_ens, page_colors [TitreNormal], _email, _nom, cat2str ());
	// avec possibilit d'tat d'heures statutaires pour les permanents.
	if ((_employeur -> _type == UNIVERSITE && (_dept -> _a_nous || get_type () == POLYTECH))
		|| _categorie == MON)
	{
		ptc += sprintf (ptc, fmt_avec_etat_stat, page_colors [TitreEmphase],
						nospaces (), nospaces (),
						nospaces (), nospaces (),
						nospaces (), nospaces (),
						nospaces (), nospaces (),
						nospaces (), nospaces ());
	}
	ptc += sprintf (ptc, "</td>\n");
	TNG_arrow_title (link_prev, link_next, (char*)"", tc);
	// le formulaire en mode dition
	_outp_ += sprintf (_outp_, fmt_page_post_1, _nom);
	_outp_ += sprintf (_outp_, "<table cellspacing=0 cellpadding=2 border=0>\n");
	// afficher le service.
	if (affiche_service (EDIT_MANDATEMENTS, true))	// a priori on est mandator!
		_outp_ += sprintf (_outp_, "%s", fmt_page_post_2);
	_outp_ += sprintf (_outp_, "</form>\n");
	// affiche les mandatements.
	affiche_mandatements ();
}

/*
 *	Affiche/saisit les mandatements.
 */

void Enseignant::affiche_mandatements ()
{
	static char	fmt_head_mandatement [] =
		"<form method=post action=\"mandatement_request\">\n"
		"<input type=hidden name=P value=\"%s\">\n"
		"Pour modifier un mandatement ou imprimer la fiche d'engagement de paiement "
		"d'heures complmentaires cliquer sur le bouton correspondant.<p>\n"
		"<table cellpadding=2 cellspacing=0 border=2 width=\"100%%\">\n"
		"<tr bgcolor=#404060>"
			"<td align=center><b>Date</b></td>"
			"<td align=center><b>Type</b></td>"
			"<td align=center><b>Division</b></td>"
			"<td align=center><b>Nb. Heures</b></td>"
			"<td align=center><b>Taux horaire</b></td>"
			"<td align=center><b>Total brut</b></td>"
		"</tr>\n";
	static char	fmt_tail_mandatement [] =
		"<tr bgcolor=#004070>"
			"<td>&nbsp;&nbsp;</td>"
			"<td>&nbsp;&nbsp;</td>"
			"<td align=center><b>Total</b></td>"
			"<td align=right><font face=verdana><b>%g&nbsp;</b></font></td>"
			"<td>&nbsp;</td>"
			"<td align=right><font face=verdana><b>%.2f&nbsp;</b></font></td>"
		"</tr></table></form>\n";

	_outp_ += sprintf (_outp_, fmt_head_mandatement, _nom);
	float	tot_hc = 0.;	// total des heures complmentaires.
	float	tot_mand = 0.;	// total des mandatements.
	for (int i = 0; i < _mandatement.length (); ++i) {
		Mandatement*	m = (Mandatement*) _mandatement [i];

		m -> affiche_liste (i);
		tot_hc += MAND_EQTD (m);
		tot_mand += MAND_EQTD (m) * m -> _taux;
	}
	_outp_ += sprintf (_outp_, fmt_tail_mandatement, tot_hc, tot_mand);
}

/*
 *	Affiche le formulaire html pour creer/modifier un mandatement.
 */

void Enseignant::form_mandatement (int n, float* volume, Service* s)
{
	static char	fmt_suppr [] =
		"<form method=post action=\"remove_mandatement_return\">\n"
		"<b><input type=button onClick='submit()' value=\"%s\"></b>"
		"<input type=hidden name=P value=\"%s\">\n"
		"<input type=hidden name=N value=\"%d\">\n"
		"</form>\n";

	static char	fmt_retour [] =
		"<form method=post action=\"mandatement_retour\">\n"
		"<b><input type=button onClick='submit()' value=\"Retour\"></b>"
		"<input type=hidden name=P value=\"%s\">\n"
		"</form>\n";

	static char	fmt_valid [] =
		"<b><input type=button onClick='submit()' value=Valider></b>"
		"</form>\n";

	static char	fmt_saisie [] =
		"<form method=post action=\"mandatement_return\">\n"
		"<input type=hidden name=P value=\"%s\">\n"
		"<input type=hidden name=N value=\"%d\">\n"
		"<center><table cellspacing=0 cellpadding=2 border=2 " BGCOLOR2 ">\n"
		"<tr bgcolor=#404060>"
			"<td colspan=12 align=center>%s [%s]</td>"
		"</tr>\n"
		"<tr bgcolor=#004070>"
			"<td align=center>Date</td>"
			"<td align=center>Formation</td>"
			"<td align=center>Division</td>"
			"<td align=center>Cours</td>"
			"<td align=center>TD</td>"
			"<td align=center>TP</td>"
			"<td align=center>MP</td>"
			"<td align=center>Demi-TP</td>"
			"<td align=center>Soutien-TD</td>"
			"<td align=center>Soutien Session2</td>"
			"<td align=center>Dbut priode</td>"
			"<td align=center>Fin priode</td>"
		"</tr>\n"
		"<tr>\n"
			// date de mandatement.
			"<td><input name=D type=text value=\"%s\" size=11></td>\n"
			// type de formation.
			"<td align=center>"
				"<select name=F>"
					"<option %s value=I>FI</option>"
					"<option %s value=C>FC</option>"
				"</select></td>\n"
			// la division qui paie.
			"<td align=center><select name=v>"
					"<option %s value=0>%s</option>"
					"<option %s value=1>%s</option>"
					"<option %s value=2>%s</option>"
					"<option %s value=3>%s</option>"
					"<option %s value=4>%s</option>"
				"</select></td>\n"
			// volumes horaires.
			"<td align=center><input name=V0 type=text value=\"%g\" size=3></td>\n"
			"<td align=center><input name=V1 type=text value=\"%g\" size=3></td>\n"
			"<td align=center><input name=V2 type=text value=\"%g\" size=3></td>\n"
			"<td align=center><input name=V3 type=text value=\"%g\" size=3></td>\n"
			"<td align=center><input name=V4 type=text value=\"%g\" size=3></td>\n"
			"<td align=center><input name=V5 type=text value=\"%g\" size=3></td>\n"
			"<td align=center><input name=V6 type=text value=\"%g\" size=3></td>\n"
			// dbut priode.
			"<td align=center><input name=d type=text value=\"%s\" size=11></td>\n"
			// fin priode.
			"<td align=center><input name=f type=text value=\"%s\" size=11></td>\n"
		"</tr></table></center><p>\n";

	static char fmt_ens [] =
		"<td align=center height=40 bgcolor=\"%s\">"
		"&nbsp;<a href=\"mailto:%s\"><font size=4 color=#000000 face=Helvetica>"
		"%s</a></td>\n";

#if !FICHE_LATEX
	static char avec_fiche [] =
			"<td align=right height=40 bgcolor=\"%s\">\n"
			"	<font color=#000000>Fiche d'engagement HC </font>\n"
			"	<a href=\"engagement_%d_%s.pcf\"><font color=#000000 size=1>PDF</a>\n"
			" / <a href=\"engagement_%d_%s.cp?\"><font color=#000000 size=1>PS</a>&nbsp;\n"
			"</td>\n";
	static char avec_fiche_sans_date [] =
			"<td align=right height=40 bgcolor=\"%s\">\n"
			"	<font color=#000000>Fiche d'engagement HC (sans dates) </font>\n"
			"	<a href=\"engagement_V%d_%s.pcf\"><font color=#000000 size=1>PDF</a>\n"
			" / <a href=\"engagement_V%d_%s.cp?\"><font color=#000000 size=1>PS</a>&nbsp;\n"
			"</td>\n";
#endif
	static char	selected [] = "selected";
	static char	empty [] = "";
	Mandatement*	m;
	bool			modif = (volume == 0);
	char			mdate [32];
	char			debp [32];
	char			finp [32];

	// afficher l'entte.
	TNG_round_title_begin ();
	_outp_ += sprintf (_outp_, fmt_ens, page_colors [TitreNormal], _email, _nom);
	// modification d'un mandatement (on peut donc imprimer la fiche d'engagement).
	if (modif) {
		m = (Mandatement*) _mandatement [n];
#if !FICHE_LATEX
		_outp_ += sprintf (_outp_, avec_fiche, page_colors [TitreNormal],
								   n, nospaces (), n, nospaces ());
#endif
	} else {
		// nouvau mandatement : n'afficher que ce qui reste  mandater
		// regarder si un mandatement existe pour ce service.
		float	vol_mandate [N_ENS_TYPES];

		for (int i = COURS; i < N_ENS_TYPES; ++i)
			vol_mandate [i] = 0.;

		for (int i = 0; i < _mandatement.length (); ++i) {
			m = (Mandatement*) _mandatement [i];
			if (m -> _s == s) {
				for (int j = COURS; j < N_ENS_TYPES; ++j)
					vol_mandate [j] += m -> _vol [j];
			}
		}
		n = _mandatement.length ();
		m = new Mandatement ();
		_mandatement [n] = m;
		m -> _date = time (0);				// mandatement d'aujourd'hui.
		m -> _fi = true;					// formation initiale par dfaut.
		m -> _division = FORMATION;			// la division de la formation paie par dfaut.
		for (int i = COURS; i < N_ENS_TYPES; ++i) {
			m -> _vol [i] = volume [i] - vol_mandate [i];
			// ne pas ngativer les volumes dj mandats.
			if (m -> _vol [i] < 0.)
				m -> _vol [i] = 0.;
		}
		Per2Time (s -> _ens -> _periode, m -> _deb_periode, m -> _fin_periode);
		m -> _taux = Taux::get (m -> _deb_periode);
		m -> _s = s;							// service rfrenc.
#if !FICHE_LATEX
		_outp_ += sprintf (_outp_, avec_fiche_sans_date, page_colors [TitreNormal],
								   n, nospaces (), n, nospaces ());
#endif
	}
	// afficher l'entte.
	TNG_round_title_end ();
	_outp_ += sprintf (_outp_, "<p>");
	// afficher le formulaire.
	strcpy (mdate, AscTime (&m -> _date));
	strcpy (debp, AscTime (&m -> _deb_periode));
	strcpy (finp, AscTime (&m -> _fin_periode));
	_outp_ += sprintf (_outp_, fmt_saisie,
						_nom, n,
						m -> _s -> _ens -> get_intitule (),
						m -> _s -> _ens -> _groupe -> _filiere -> _nom,
						mdate,
						m -> _fi ? selected : empty,
						m -> _fi ? empty : selected,
						m -> _division == ORIENTATION ? selected : empty,
						Mandatement::_nom_div [ORIENTATION],
						m -> _division == FORMATION ? selected : empty,
						Mandatement::_nom_div [FORMATION],
						m -> _division == RECHERCHE ? selected : empty,
						Mandatement::_nom_div [RECHERCHE],
						m -> _division == STAPS ? selected : empty,
						Mandatement::_nom_div [STAPS],
						m -> _division == FP ? selected : empty,
						Mandatement::_nom_div [FP],
						m -> _vol [COURS],
						m -> _vol [TD],
						m -> _vol [TP],
						m -> _vol [MP],
						m -> _vol [DTD],
						m -> _vol [STD],
						m -> _vol [SSS],
						debp, finp);
	// afficher le bouton de validation.
	_outp_ += sprintf (_outp_, "%s", fmt_valid);
	// afficher le bouton de suppression.
	_outp_ += sprintf (_outp_, fmt_suppr,
						modif ? "Supprimer" : "Abandonner",
						_nom, n);
	// afficher le bouton de retour.
	_outp_ += sprintf (_outp_, fmt_retour, _nom);
}

/*
 *	Ajoute un service.
 */

void Enseignant::add_service (Service* s)
{
	_service [_service.length ()] = s;
}

/*
 *	Retire un service.
 */

void Enseignant::remove_service (Service* s)
{
	for (int i = 0; i < _service.length (); ++i)
		if (_service [i] == (void*) s) {
				_service.remove (i, 1);
			return;
		}
}

/*
 *	crit le service de l'enseignant sur le fichier 'fp'
 */

void Enseignant::save_service (FILE* fp)
{
	static const char	*fmt1 = "'%s' S %s %c%c%c%c%c%c%c %d ";
	static const char	*fmt2 = "'%s' M %s %d %s %g %g %g %g %g %g %g %g %d %d\n";
	// a peut varier d'une machine  l'autre.
	if (sizeof(time_t) == sizeof (long)) {
		fmt1 = "'%s' S %s %c%c%c%c%c%c%c %ld ";
		fmt2 = "'%s' M %s %ld %s %d %g %g %g %g %g %g %g %g %ld %ld\n";
	}
	// sauver les services.
	for (int n = 0; n < _service.length (); ++n) {
		Service*	s = (Service*) _service [n];

#define prio(i) s -> _prioritaire [i] ? '*' : '-'
		fprintf (fp, fmt1, _nom, s -> _ens -> _identification,
						   prio (COURS),
						   prio (TD),
						   prio (TP),
						   prio (MP),
						   prio (DTD),
						   prio (STD),
						   prio (SSS),
						   s -> _date);
#undef prio
		for (int th = 0; th < N_HRS_TYPES; ++th)
			for (int i = 0; i < N_ENS_TYPES; ++i)
				fprintf (fp, "%g ", s -> _heures [th][i]);
		fprintf (fp, "\n");
	}
	// puis les mandatements.
	for (int n = 0; n < _mandatement.length (); ++n) {
		Mandatement*	m = (Mandatement*) _mandatement [n];

		fprintf (fp, fmt2, _nom,
					m -> _s -> _ens -> _identification,
					m -> _date,
					m -> _fi ? "FI" : "FC",
					m -> _division,
					m -> _vol [COURS],
					m -> _vol [TD],
					m -> _vol [TP],
					m -> _vol [MP],
					m -> _vol [DTD],
					m -> _vol [STD],
					m -> _vol [SSS],
					m -> _taux,
					m -> _deb_periode,
					m -> _fin_periode);
	}
}

/*
 *	Remplace le service de l'enseignant par ce qui est indiqu dans la chane 'line'
 */

#if ANCIEN_FORMAT

// fonction pour l'ancien format obsolte.

bool Enseignant::load_service (const char* line)
{
	static const char	*fmt1 = "%s %d %g %g %g %g %g %g %g %g %g %g %g %g %g %g %g %g %g %g %g %g";
	static const char	*fmt2 = "%s %d %s %g %g %g %g %g %d %d";
	// a peut varier d'une machine  l'autre.
	if (sizeof(time_t) == sizeof (long)) {
		fmt1 = "%s %ld %g %g %g %g %g %g %g %g %g %g %g %g %g %g %g %g %g %g %g %g";
		fmt2 = "%s %ld %s %d %g %g %g %g %g %ld %ld";
	}

	// le service doit tre vide initialement
	Service*	s = new Service ((Enseignement*) 0);
	char		cours [128];	// identification de l'enseignement.

	line += 2;		// sauter le type et l'espace suivant.
	if (line [-2] == 'S') {
		// ligne de service.
		if (22 != sscanf (line, fmt1,
			cours,
			& s -> _date,

			& s -> _heures [HEURE_STAT_A] [COURS],
			& s -> _heures [HEURE_STAT_A] [TD],
			& s -> _heures [HEURE_STAT_A] [TP],
			& s -> _heures [HEURE_STAT_A] [MP],

			& s -> _heures [HEURE_COMP_A] [COURS],
			& s -> _heures [HEURE_COMP_A] [TD],
			& s -> _heures [HEURE_COMP_A] [TP],
			& s -> _heures [HEURE_COMP_A] [MP],

			& s -> _heures [HEURE_STAT_B] [COURS],
			& s -> _heures [HEURE_STAT_B] [TD],
			& s -> _heures [HEURE_STAT_B] [TP],
			& s -> _heures [HEURE_STAT_B] [MP],

			& s -> _heures [HEURE_COMP_B] [COURS],
			& s -> _heures [HEURE_COMP_B] [TD],
			& s -> _heures [HEURE_COMP_B] [TP],
			& s -> _heures [HEURE_COMP_B] [MP],

			& s -> _heures [HEURE_COMP_EXT] [COURS],
			& s -> _heures [HEURE_COMP_EXT] [TD],
			& s -> _heures [HEURE_COMP_EXT] [TP],
			& s -> _heures [HEURE_COMP_EXT] [MP]))
		{
			return false;
		}
		// le cours existe-t-il bien?
		s -> _ens = Filiere::name2ens (cours);
		if (s -> _ens == 0)
			return false;

		// mettre les priorits  faux.
		for (int i = 0; i < N_ENS_TYPES; ++i)
			s -> _prioritaire [i] = false;
		// retirer ces heures du manque de cet enseignement.
		for (int n = 0; n < N_ENS_TYPES; ++n)
			for (int th = 0; th < N_HRS_TYPES; ++th)
				s -> _ens -> _manque [n] -= s -> _heures [th][n];
		// on l'ajoute au bout pour conserver l'ordre.
		add_service (s);
		return true;
	}
	// ligne de mandatement.
	Enseignement*	ens;
	Mandatement*	m = new Mandatement ();
	char			fifc [4];

	if (line [-2] != 'M' ||
		11 != sscanf (line, fmt2,
			cours,
			& m -> _date,
			fifc,
			& m -> _division,
			& m -> _vol [COURS],
			& m -> _vol [TD],
			& m -> _vol [TP],
			& m -> _vol [MP],
			& m -> _taux,
			& m -> _deb_periode,
			& m -> _fin_periode))
	{
		return false;	// mauvais format.
	}
	m -> _fi = (fifc [0] == 'F' && fifc [1] =='I');
	ens = Filiere::name2ens (cours);
	if (ens == 0)
		return false;	// mauvais format #2

	for (int i = 0; i < _service.length (); ++i) {
		s = (Service*) _service [i];
		// ok: on a trouv le service correspondant  ce mandatement.
		if (s -> _ens == ens) {
			m -> _s = s;
			_mandatement [_mandatement.length ()] = m;
			return true;
		}
	}
	return false;
}
#endif

/*
 *	Version pour le nouveau format.
 */

#if ANCIEN_FORMAT

const char* Enseignant::load_service_new (FILE* fp, int format)

#else

const char* Enseignant::load_service (FILE* fp, int format)
{
	// ordonnancement des types d'heures dans les diverses versions du fichier de sauvegarde.
	static const int	order [NB_VERSIONS][N_HRS_TYPES] = {
		// version 1.
		{
			HEURE_STAT_A, HEURE_COMP_A,
			HEURE_STAT_B, HEURE_COMP_B,
			HEURE_COMP_EXT, HEURE_STAT_EXT,
			HEURE_GRAT, -1, -1, -1
		},
		// version 2.
		{
			HEURE_STAT_A, HEURE_STAT_B,
			HEURE_COMP_A, HEURE_COMP_B,
			HEURE_COMP_EXT, HEURE_COMP_EXT_B,
			HEURE_STAT_EXT, HEURE_STAT_EXT_B,
			HEURE_GRAT, HEURE_GRAT_B
		},
		// version 3.
		{
			HEURE_STAT_A, HEURE_STAT_B,
			HEURE_COMP_A, HEURE_COMP_B,
			HEURE_COMP_EXT, HEURE_COMP_EXT_B,
			HEURE_STAT_EXT, HEURE_STAT_EXT_B,
			HEURE_GRAT, HEURE_GRAT_B
		}
	};
	static const int	n_hrs_types [NB_VERSIONS] = { 7, N_HRS_TYPES, N_HRS_TYPES };
	static const int	n_ens_types [NB_VERSIONS] = { 4, 4, N_ENS_TYPES };

	char		priorite [32];
	char		type [8];
	Service*	s = new Service ((Enseignement*) 0);	// le service doit tre vide initialement
	char		cours [128];	// identification de l'enseignement.

	if (2 != fscanf (fp, "%s %s", type, cours))
		return "bad format (type + cours)";

	// service.
	if (type [0] == 'S') {
		if (2 != fscanf (fp, "%s %d", priorite, (int*)&s -> _date)) {
zlog ("priorite: %s cours: %s\n", priorite, cours);
			return "bad format (prio + date)";
		}
		for (int th = 0; th < n_hrs_types [format]; ++th)
			for (int i = 0; i < n_ens_types [format]; ++i)
				if (1 != fscanf (fp, "%g ", &s -> _heures [order [format][th]][i])) {
					return "bad format...";
				}
		// le cours existe-t-il bien?
		s -> _ens = Filiere::name2ens (cours);
		if (s -> _ens == 0) {
zlog ("cours: %s\n", cours);
			return "Enseignement inconnu 1";
		}

		// mettre les priorits.
		for (int i = 0; i < n_ens_types [format]; ++i)
			s -> _prioritaire [i] = (priorite [i] == '*');
		// retirer ces heures du manque de cet enseignement.
		for (int n = 0; n < n_ens_types [format]; ++n)
			for (int th = 0; th < n_hrs_types [format]; ++th)
				s -> _ens -> _manque [n] -= s -> _heures [th][n];
		// reprer les demandes de prime.
		float	fl = 0;
#ifdef CHARGES_EN_HC
		for (int th = HEURE_STAT_A; th <= HEURE_GRAT; th += 2) // sauter les choix B.
			fl += s -> _heures [th][TD];
#else
		fl = s -> _heures [HEURE_STAT_A][TD];
#endif
		s -> _prime = s -> _ens -> _charge_np && fl == 0;
		// et les dfalquer du manque de cette charge.
		if (s -> _prime)
			s -> _ens -> _manque [TD] -= s -> _ens -> _volume [TD];
		// on l'ajoute au bout pour conserver l'ordre.
		add_service (s);
		return 0;
	}
	// ligne de mandatement.
	Enseignement*	ens;
	Mandatement*	m = new Mandatement ();
	char			fifc [4];

	if (type [0] == 'M') {
		bool	scan_ok;

		if (format >= 2) 
			scan_ok = (13 == fscanf (fp, "%d %s %d %g %g %g %g %g %g %g %g %d %d\n",
								(int*)&m -> _date, fifc, (int*) &m -> _division,
								&m -> _vol [COURS], &m -> _vol [TD], &m -> _vol [TP],
								&m -> _vol [MP],
								&m -> _vol [DTD], &m -> _vol [STD], &m -> _vol [SSS],
								&m -> _taux,
								(int*)&m -> _deb_periode, (int*)&m -> _fin_periode));
		else
			scan_ok = (10 == fscanf (fp, "%d %s %d %g %g %g %g %g %d %d\n",
								(int*)&m -> _date, fifc, (int*) &m -> _division,
								&m -> _vol [COURS], &m -> _vol [TD], &m -> _vol [TP],
								&m -> _vol [MP],
								&m -> _taux,
								(int*)&m -> _deb_periode, (int*)&m -> _fin_periode));
		if (scan_ok) {
			m -> _fi = (fifc [0] == 'F' && fifc [1] =='I');
			ens = Filiere::name2ens (cours);
			if (ens == 0)
				return "Enseignement inconnu 2";

			for (int i = 0; i < _service.length (); ++i) {
				s = (Service*) _service [i];
				// ok: on a trouv le service correspondant  ce mandatement.
				if (s -> _ens == ens) {
					m -> _s = s;
					_mandatement [_mandatement.length ()] = m;
					return 0;
				}
			}
			return "Mandatement inconnu";
		}
	}
zlog ("type = %s\n", type);
	return "bad format (S or M)";
}
#endif

/*
 *	Retourne le service que fait l'enseignant dans cet enseignement.
 */

Service* Enseignant::get_service (Enseignement* ens)
{
	for (int i = 0; i < _service.length (); ++i) {
		Service* s = (Service*) _service [i];
		if (s -> _ens == ens)
			return s;
	}
	return 0;	// pas de service dans cet enseignement.
}
