/*****************************************************************

  Copyright (c) 1999 - 2003 by Patrick Amar - pa@lri.fr

    http://www.lri.fr/~pa

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

#ifndef ENSEIGNANT_H
#define ENSEIGNANT_H

#define BGCOLOR1 "bgcolor=#b67650"
#define BGCOLOR2 "bgcolor=#b67660"

#include "Vector.h"
#include "Service.h"
#include "Employeur.h"

void zlog (const char* fmt, ...);	// pour la journalisation.

class Enseignement;	// on a juste besoin d'un pointeur.

// Structure dfinissant un enseignant.

#define	DEUG	1
#define	NFI		2
#define IUP		4

#define NELEM(t) ((sizeof (t))/(sizeof(t[0])))

// nombre max d'enseignants.
#define MAX_PROF	1024

enum Genre {
	INCONNU, TITULAIRE, MONITEUR, ATERS, IUT, POLYTECH,
	AUTRE_UPS, CH_CNRS, CH_INRIA, CONTR_PAST, VAC_PRIV, VAC_PUB
};

enum Categorie {
	PR, MC,				// pour les titulaires IUT, UPS.
	MON, AMN, ATER, DEMI_ATER,
	PRAG, PAST, CONTR, PRCE,
	CR, DR, IR, IE, TECH,	// les CNRS et INRIA.
	VAC,				// pour les autres (vacataires).
	ATE,				// Attach temporaire d'enseignement.
	PCA					// Pas-Cette Anne.
};

struct Dispo {
	char*	_motclef;
	char*	_intitule;
	char*	_intitule_ps;
};

class Enseignant {
public:
	static int			hash (char* name, int size);
	static Enseignant*	president_dept (void);	// retourne le prsident du dpartement d'enseignement.
	static Enseignant*	name2prof (char* name, Enseignant* prof =0);
	static Enseignant*	_head;		// tte de liste des enseignants.
	static Enseignant*	_enseignants [MAX_PROF];	// tables de hashcode.
	static unsigned char	iso2ascii [];
	static Dispo		disponibilite [128];
	static const char*		genre2str [];
	static const char*		genre2orig [];
	static bool		_edit_mode;		// mode dition / consultation.
	static bool		_tous;			// tous les enseignants / ceux  service non vide.

	Vector			_service;
	Enseignant*		_suivant;		// pour naviguer ...
	Enseignant*		_precedent;		// sans sombrer !
	Perso*			_perso;			// informations personnelles.
	Categorie		_categorie;		// type de poste.
	Employeur*		_employeur;		// son employeur.
	Departement*	_dept;			// le dpartement dont dpend l'enseignant.
	Vector			_mandatement;	// les mandatements.
	char*			_nom;			// identit.
	char*			_email;			// adresse lectronique.
	int				_serv_annuel;	// nombre d'heures a faire dans l'anne.
	int				_disponibilite;	// en dtachement ?
	int				_affectation;	// pour les moniteurs.
	int				_annee;			// pour les moniteurs.
	char*			_poste;			// pour les vacataires extrieurs.
	int				_cnu;			// section CNU.
#if 66
	char			_passwd [128];
#endif

private:
	bool affiche_service (Editable, bool);
	Enseignant*	precedent (void);
	Enseignant*	suivant (void);

public:
	Enseignant (Employeur*, Departement*, Categorie,
				char* nom, char* email, int serv,
				int disponibilite, int affectation,
				int annee, char* poste, int cnu);

	bool compare (char* s);
	bool pw_compare (char* s);
static char* nospaces (char*);	// remplace les espaces par des _ pour faire une url.
	char* nospaces (void);		// la mme avec le nom implicite.
	char* cat2str (bool =false);
	void affiche_liste (int largeur =160, int* sstat=0, int* sfait=0, bool clair =false, int* spoly =0, int* scfa =0);
	void affiche_page (Editable);
	void affiche_detail (void);
	float service_filiere (Filiere* f, int th, bool tous, Periode periode);
static void attestations_service (int);
	void attestations_service (int, int, bool);
	void attestations_hybride (int, int);
	void etat_statutaire (void);
	const char*	find_comp (void);
	void etat_statutaire_complementaire (bool =false);
	void etat_statutaire_complementaire (int, int, bool =false);
#if FICHE_LATEX
	void fiche_engagement (int index_service);
#else
	void fiche_engagement (int, bool);
#endif
	void save_service (FILE*);
	void affiche_page_mandatements (void);	// affiche la page de l'enseignant avec les mandatements.
	void affiche_mandatements ();
	void form_mandatement (int n, float* vol =0, Service* =0);	// affiche le formulaire pour modifier un mandatement.
	const char* load_service (FILE*, int);
	void add_service (Service*);
	void remove_service (Service*);
	Service* get_service (Enseignement* ens);
	Genre	get_type (void);	// retourne le genre d'enseignant.
	void nom_prenom (char* nom, char* prenom);	// extrait le prnom et le nom.
	bool get_email (char* real_mail);

#if HEURES_FLOAT
	void potentiel (
		float& sstat_th, float& dech_st, char*& floatit_dech, float& serv_du,
		float& hpres_ufr, float& hpres_aut, const char*& aut_comp,
		float& hpedag, float& hadm, float& sfait, float& hcc_ufr, char* orig);
#else
	void potentiel (
		int& sstat_th, int& dech_st, char*& intit_dech, int& serv_du,
		int& hpres_ufr, int& hpres_aut, const char*& aut_comp,
		int& hpedag, int& hadm, int& sfait, int& hcc_ufr, char* orig);
#endif
};

#endif
