/*****************************************************************

  Copyright (c) 1999 - 2012 by Patrick Amar - pa@lri.fr

    http://www.lri.fr/~pa

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

#include <stdio.h>
#include "display.h"
#include "Service.h"	// Enseignement.h est dj inclus par Service.h
#include "Enseignant.h"

extern char* _outp_;

Enseignement*	Enseignement::_tous_les_enseignements [10240];
int				Enseignement::_nb_enseignements = 0;

/*
 *	Retourne l'enseignement prcdent
 */

Enseignement* Enseignement::precedent ()
{
	int			i;
	Groupe*		g;
	Filiere*	f;

	for (i = 0; (Enseignement*) (_groupe -> _groupes [i]) != this; ++i)
		;
	// si c'est pas le premier, c'est facile!
	if (i > 0)
		return (Enseignement*) (_groupe -> _groupes [i-1]);
	// remonter au module
	f = _groupe -> _filiere;
	for (i = 0; (Groupe*) (f -> _groupes [i]) != _groupe; ++i)
		;
	if (i > 0) {
		g = (Groupe*) (f -> _groupes [i-1]);
		return (Enseignement*) g -> _groupes [g -> _groupes.length () - 1];
	}
	// remonter  la filire.
	for (i = 0;	f != Filiere::filieres [i]; ++i)
		;
	if (i == 0)
		// c'est le tout premier retourner le tout dernier!
		while (Filiere::filieres [i] != 0)
			++i;
	f = Filiere::filieres [i-1];
	g = (Groupe*) (f -> _groupes [f -> _groupes.length () - 1]);
	return (Enseignement*) g -> _groupes [g -> _groupes.length () - 1];
}

/*
 *	Retourne l'enseignement suivant.
 */

Enseignement* Enseignement::suivant ()
{
	int			i;
	Groupe*		g;
	Filiere*	f;

	for (i = 0; (Enseignement*) (_groupe -> _groupes [i]) != this; ++i)
		;
	// si c'est pas le dernier, c'est facile!
	if (i < _groupe -> _groupes.length () - 1)
		return (Enseignement*) (_groupe -> _groupes [i+1]);
	// remonter au module
	f = _groupe -> _filiere;
	for (i = 0; (Groupe*) (f -> _groupes [i]) != _groupe; ++i)
		;
	if (i < f -> _groupes.length () - 1) {
		g = (Groupe*) (f -> _groupes [i+1]);
		return (Enseignement*) g -> _groupes [0];
	}
	// remonter  la filire.
	for (i = 0;	f != Filiere::filieres [i]; ++i)
		;
	if (Filiere::filieres [i+1] == 0)
		i = -1;	// c'est le tout dernier, retourner le tout premier!
	f = Filiere::filieres [i+1];
	g = (Groupe*) (f -> _groupes [0]);
	return (Enseignement*) g -> _groupes [0];
}

Enseignement::Enseignement (char* identification, char* apogee, char* intitule, char* periode,
							int effectifs, float volume [N_ENS_TYPES], int ngrp [N_ENS_TYPES],
							char* act_pedago, char* verb, char* wr, char* parcours,
							bool cours_td, bool charge, bool form_init, bool su_only)
	: Verbatim (verb, wr)
{
	_su_only = su_only;		// accessible qu'au superuser.
	_activite_pedag = act_pedago;
	_parcours = parcours;
	_groupe = 0;
	_effectifs = effectifs;
	_identification = identification;
	_apogee_id = apogee;
	_intitule = intitule;
	_periode = periode;
	for (int i = 0; i < N_ENS_TYPES; ++i) {
		_volume [i] = volume [i];
		_ngrp [i] = ngrp [i];
		_manque [i] = volume [i] * ngrp [i];	// il manque tout au dpart.
	}
	_cours_td = cours_td;
	_charge = charge;
	_charge_np = false;		// A PARSER ET A TRANSMETTRE...
	_form_initiale = form_init;
	_total_grp = ngrp [TD] + ngrp [TP];	// nombre de groupes de TD+TP
	_base = 0;		// ce cours n'en rfrence pas un autre.
	_next = 0;		// ce cours n'est pas rfrenc par un autre.
	_tous_les_enseignements [_nb_enseignements++] = this;
}

/*
 *	Fait le formulaire html pour entrer un service pour cet enseignement.
 */

void Enseignement::formulaire (Enseignant* prof) {
	static char	fmt_entete [] =
		"<hr><b>Pour saisir un voeu de <font size =+2>%s</font></b>\n"
		"<form target=\"_top\" method=post action=\"add_return\">\n"

		"<table cellpadding=2 cellspacing=0><tr>"
		"<td>Remplissez le formulaire ci-dessous puis</td>"
		"<td><b><input type=button onClick='submit()' value=\"cliquez ici\"></b></td>"
		"<td>pour valider vos choix.</td>"
		"</tr></table>"

		"<br>Les heures <b>extrieures</b> sont imputes  un autre dpartement "
		"d'enseignement (IUT, Physique, etc.). Cette information est utile pour les responsables "
		"de filires, mais le dpartement d'informatique ignorera ces heures dans ses calculs.<p>"
		"&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Bonne chance %s !<p>"
		"<input type=hidden name=E value=\"%s\">\n"
		"<input type=hidden name=U value=\"%s\">";

	static char	fmt_saisie [] =
		"<tr><td align=right><font color=#ffff00><b>%s </b></font></td>\n"
		"<td><font color=#ffff00><b><input name=\"%cV\" type=text value=0 size=3>\n"
		"<select name=\"%cU\">"
		"<option value=g>groupes</option>"
		"<option value=h>heures</option></select></b></font></td>\n"
		"<td><font color=#ffff00><b><input name=\"%cC\" type=checkbox> second choix </b></font></td>\n"
		"<td><font color=#ffff00><b><select name=\"%cH\">\n";
	static char	fmt_stat [] = "<option value=\"s\">H. statutaires</option>\n";
	static char	fmt_comp [] = "<option value=\"c\">H. complmentaires</option>\n";
	static char	fmt_comp_ext [] = "<option value=\"e\">H. complmentaires extrieures</option>\n";
	static char	fmt_stat_ext [] = "<option value=\"x\">H. statutaires extrieures</option>\n";
	static char	fmt_grat [] = "<option value=\"g\">H. bnvoles</option>\n";
	static char	fmt_end_select [] = "</select></b></font></td>\n";
	static char	fmt_prio [] =
		"<td><font color=#ffff00><b><input name=\"%cP\" type=checkbox %s> prioritaire </b></font></td></tr>\n";
	bool		avec_prio = false;
	char*		nom_ens = prof -> _nom;
	char		prenom [64];
	int			n;
	Service*	s;

	/*
	 *	Formulaire spcifique aux charges (un seul enseignant peut la
	 *	choisir, et elle ne comporte que du TD).
	 */
	if (_charge_np) {
		// afficher l'intitul de la charge.
		affiche (1, nom_ens, 0);
		// le formulaire.
		_outp_ += sprintf (_outp_,
				"<form target=_top method=post action=add_return>\n"
				"<p><b>"
				"<input type=hidden name=E value=\"%s\">\n"
				"<input type=hidden name=U value=\"%s\">\n"
#ifdef AVEC_PRIME
				"<input type=radio name=R value=\"1\"> demander une prime correspondant  cette charge<br>\n"
				"<input type=radio name=R value=\"0\"> ajouter les heures  votre service statutaire<p>\n"
#else
				"<input type=hidden name=R value=0>\n"
#endif
#ifdef CHARGES_EN_HC
				"<select name=1H>\n"
					"<option selected value=s>Heures Statutaires</option>\n"
					"<option value=x>Heures Statutaires Extrieures</option>\n"
					"<option value=c>Heures Complmentaires</option>\n"
					"<option value=e>Heures Complmentaires Extrieures</option>\n"
					"<option value=g>Heures Bnvoles</option>\n"
				"</select><p>\n"
#endif
				"<input type=button onClick='submit()' value=Slectionner> "
				"cette charge.<p>\n"
				"<input type=button onClick=\"top.location='service_%s.cmd'\" value=Retourner> "
				" votre page de service.</b>\n",
				_identification, nom_ens, prof -> nospaces ());
		// Indiquer que le service actuel pour cette charge est vide.
		for (int th = 0; th < N_HRS_TYPES; ++th)
			for (int i = 0; i < N_ENS_TYPES; ++i)
				_outp_ += sprintf (_outp_,
							"<input type=hidden name=%d%d value=0>\n", i, th);
		// affecter la charge si on soumet le formulaire.
		_outp_ += sprintf (_outp_,
				"<input type=hidden name=1V value=1>\n"
				"<input type=hidden name=1U value=g>\n"
#ifndef CHARGES_EN_HC
				"<input type=hidden name=1H value=s>\n"
#endif
				"</form>\n");
		return;
	}
	/*
	 *	Enseignement standard.
	 */

	// afficher l'intitul du cours.
	affiche (1, nom_ens, 0);
	s = prof -> get_service (this);	// dj un service dans cet enseignement?
	// Extraire le prnom de l'enseignant.
	for (n = 0; (prenom [n] = nom_ens [n]) != ' '; ++n)
		;
	prenom [n] = 0;
	// Stocker le service actuel dans cet enseignement dans des variables caches.
	_outp_ += sprintf (_outp_, fmt_entete, nom_ens, prenom, _identification, nom_ens);
	for (int th = 0; th < N_HRS_TYPES; ++th)
		for (int i = 0; i < N_ENS_TYPES; ++i)
			_outp_ += sprintf (_outp_,
						"<input type=hidden name=%d%d value=\"%g\">\n", i, th,
						s != 0	? s -> _heures [th][i]
								: 0.0);
	_outp_ += sprintf (_outp_, "<center><table cellspacing=0 cellpadding=2 border=2 " BGCOLOR2 ">\n");
	// afficher le beau formulaire!
	for (int i = 0; i < N_ENS_TYPES; ++i) {
		int	ctdtp = (i==COURS && _cours_td) ? COURS_TD : i;

		if (_volume [i] * _ngrp [i] > 0) {
			_outp_ += sprintf (_outp_, fmt_saisie, _genre [ctdtp],
								i+'0', i+'0',
								i+'0', i+'0');
			// n'afficher que les choix plausibles.
			// ex: un ATER ne peut faire que des heures statutaires,
			//	 un extrieur ne peut pas faire d'heures statutaires.
			switch (prof -> get_type ()) {
				case TITULAIRE:
				case POLYTECH:
				case CONTR_PAST:
					avec_prio = true;
					_outp_ += sprintf (_outp_, "%s", fmt_stat);
				case AUTRE_UPS:
					if (prof -> _categorie == MON || prof -> _categorie == AMN ||
						prof -> _categorie == ATER)
					{
						_outp_ += sprintf (_outp_, "%s", fmt_stat_ext);
						_outp_ += sprintf (_outp_, "%s", fmt_grat);
						break;
					}
					// fall thru.
				default:
					_outp_ += sprintf (_outp_, "%s", fmt_comp);
					_outp_ += sprintf (_outp_, "%s", fmt_comp_ext);
					_outp_ += sprintf (_outp_, "%s", fmt_stat_ext);
					_outp_ += sprintf (_outp_, "%s", fmt_grat);
					break;
				case MONITEUR:
					avec_prio = true;
				case ATERS:
					_outp_ += sprintf (_outp_, "%s", fmt_stat);
					_outp_ += sprintf (_outp_, "%s", fmt_grat);
					break;
			}
			_outp_ += sprintf (_outp_, "%s", fmt_end_select);
			if (avec_prio) {
				const char*	checked = (s == 0) ? ""
									 : s -> _prioritaire [i] ? "checked" : "";
				_outp_ += sprintf (_outp_, fmt_prio, '0'+i, checked);
			} else
				_outp_ += sprintf (_outp_, "</tr>\n");
		}
	}
	_outp_ += sprintf (_outp_, "</table></center></form>\n");
}

/*
 *	Affiche la liste de tous les profs qui enseignent ce cours.
 */

void Enseignement::affiche_profs (int n)
{
	for (Enseignant* prof = Enseignant::_head;;) {
		// s'il existe un service ici pour le prof : afficher son nom.
		Service*	s = prof -> get_service (this);
		if (s != 0)
			s -> affiche_liste (prof -> _nom, n);

		prof = prof -> _suivant;
		if (prof == Enseignant::_head)
			break;
	}
	_outp_ += sprintf (_outp_, "</td>\n");
}

/*
 *	Retourne l'intitul d'un cours (traite les cours communs)
 */

char* Enseignement::get_intitule (bool anchor)
{
	return _intitule == 0 ? _base -> _intitule : _intitule;
}

/*
 *	Affiche le dtail dtaill de l'enseignement.
 */

void Enseignement::affiche_detail (char* nom_ens)
{
	static char	fmt_td [] =
		"<tr><td valign=top>%d * %g h</td><td valign=top><b>%s</b></td><td valign=top>";

	// affiche la description.
	if (_verbatim)
		_outp_ += sprintf (_outp_, "<tr><td colspan=3>%s</td></tr>\n", _verbatim);

	// affiche le nom du groupe.
	_outp_ += sprintf (_outp_,
			"<tr><td><b>Module:</b></td><td colspan=2><a href=\"groupe_%s%s%s.cmd\">%s</a></td></tr>\n"
			"<tr><td>&nbsp;</td></tr>\n",
						_groupe -> _identification,
						nom_ens[0] ? Service::_sep : "", nom_ens,
						_groupe -> _nom);
	// et le dtail de l'enseignement.
	float	hr_devant_et = 0.;	// total des heures prsentielles.

	for (int n = 0; n < N_ENS_TYPES; ++n) {
		if (_volume [n] * _ngrp [n] > 0.) {
			_outp_ += sprintf (_outp_, fmt_td, _ngrp [n], _volume [n],
							_genre [n==COURS && _cours_td ? COURS_TD : n]);
			affiche_profs (n);
			_outp_ += sprintf (_outp_, "</tr>\n");
			// ne pas compter les soutiens.
			if (n != STD && n != SSS)
				hr_devant_et += _volume [n];
		}
	}
	// ajouter les heures de cours en cas de cours commun.
	if (_base != 0)
		hr_devant_et += _base -> _volume [COURS];
	// afficher le total si non nul.
	if (hr_devant_et > 0.)
		_outp_ += sprintf (_outp_,
				"<tr><td colspan=3><hr>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;"
				"<font color=pink>%g h prsentielles (hors soutien)</font></td></tr>\n",
				hr_devant_et);
	// un peu de place.
	_outp_ += sprintf (_outp_, "<tr><td>&nbsp;</td></tr>\n");
}

/*
 *	Affichent le dtail de l'enseignement.
 */

void Enseignement::affiche (int format, char* nom_ens, int depth) {
	static char	fmt_entete [] = "<p><a href=\"cours_%s%s%s.cmd\">%s - %s %s (%d tudiants)</a>%s</p>\n";
	static char fmt_label [] =
			"<a href=\"mail_ens_UE_%s.cmd\"><img src=email-trans.gif align=top border=0></a> "
			"&nbsp; &nbsp;%s - %s &nbsp;&nbsp;&nbsp;&nbsp;%s";

	static char fmt_label_affecter [] =
			"<a href=\"mail_ens_UE_%s.cmd\"><img src=email-trans.gif align=top border=0></a> "
			"&nbsp; &nbsp;%s - %s &nbsp;&nbsp;&nbsp;&nbsp;%s"
			" &nbsp;&nbsp;&nbsp;&nbsp;<button onclick=\"window.location.href='liste_enseignants_cours%s%s.cmd'\">Affecter</button>";

	static char fmt_label_url [] =
			"<a href=\"mail_ens_UE_%s.cmd\"><img src=email-trans.gif align=top border=0></a> "
			"&nbsp; &nbsp;<a href=\"cours_%s.cmd\"><font color=black>%s - %s</font></a> &nbsp;&nbsp;&nbsp;&nbsp;%s"
			" &nbsp;&nbsp;&nbsp;&nbsp;<button onclick=\"window.location.href='liste_enseignants_cours%s%s.cmd'\">Affecter</button>";

	static char fmt_link [] = "cours_%s%s%s.cmd";
	static char fmt_table [] = "<table cellspacing=0 cellpadding=0 border=0>\n";

	// si c'est une redirection, rediriger!
	if (_lien && _base != 0)
		return _base -> affiche (format, nom_ens, depth);

	char*	image_commun = (_base != 0 || _total_grp > _ngrp [TD] + _ngrp [TP])
					? (char*) "<img src=\"icone_commun.png\" align=bottom border=0>"
					: (char*) "";
	Enseignement*	prec;
	Enseignement*	suiv;
	bool			pas_encore = true;	// a priori il n'y a pas de trou/bosse.
	char			tc [1280],
					tc2 [1280],
					tc3 [1280],
					tc4 [1280];
	extern bool	is_superuser;	// sommes nous superuser? (affect dans Commands.cc)

	nom_ens = Enseignant::nospaces (nom_ens);
	switch (format) {
		// format liste.
		case 0:
			_outp_ += sprintf (_outp_, fmt_entete, _identification,
								nom_ens[0] ? Service::_sep : "", nom_ens,
								_groupe -> _identification, get_intitule (false), _periode,
								_effectifs, image_commun);
			break;

		// format saisie.
		case 1:
			depth = 1;	// pour la saisie, on veut en voir le maximum.
			prec = precedent ();
			suiv = suivant ();
			sprintf (tc, fmt_link, prec -> _identification, nom_ens[0] ? Service::_sep : "", nom_ens);
			sprintf (tc2, fmt_link, suiv -> _identification, nom_ens[0] ? Service::_sep : "", nom_ens);
			if (is_superuser) {
				if (nom_ens [0] != 0) {
					sprintf (tc3, fmt_label_url,
						_identification, _identification,
						_groupe -> _identification, get_intitule (true), _periode,
						Service::_sep, _identification);
				} else {
					sprintf (tc3, fmt_label_affecter,
						_identification,
						_groupe -> _identification, get_intitule (true), _periode,
						Service::_sep, _identification);
				}
			} else {
					sprintf (tc3, fmt_label,
						_identification,
						_groupe -> _identification, get_intitule (true), _periode);
			}
			if (_apogee_id != 0) {
				strcpy (tc4, tc3);
				sprintf (tc3, "(%s) %s", _apogee_id, tc4);
			}
			TNG_arrow_title (tc, tc2, tc3);
			break;

		// affiche les manques / bosses.
		case 2:
		case 3:
			for (int n = 0; n < N_ENS_TYPES; ++n) {
				int		ng = 0;
				float	reste = 0.;

				if (n == MP)
					continue;	// on ne compte pas les Mises en Place de TP dans les trous!

				if (format == 2 && _manque [n] > 0) {
					ng = (int) (_manque [n] / _volume [n]);
					reste = _manque [n] - (ng * _volume [n]);
				} else if (format == 3 && _manque [n] < 0) {
					ng = (int) (-_manque [n] / _volume [n]);
					reste =  -_manque [n] - (ng * _volume [n]);
				}
				if (ng == 0 && reste == 0.)
					continue;

				// il est peut tre temps d'afficher la bannire.
				if (pas_encore) {
					pas_encore = false;

					_outp_ += sprintf (_outp_, "%s", fmt_table);
					TNG_mid_table (92, 24);
					_outp_ += sprintf (_outp_,
						"<td><font color=cyan><b><a href=\"cours_%s.cmd\">%s - %s</a> %s</b></font></td></tr></table>\n",
						_identification, _groupe -> _identification, get_intitule (true), _periode);
				}

				_outp_ += sprintf (_outp_, "%s", fmt_table);
				// le nombre de groupes.
				if (ng > 0) {
					TNG_mid_table (92, 24);
					_outp_ += sprintf (_outp_,
						"	<td>%d * %g h</td>\n"
						"	<td>&nbsp;%s</td>\n"
						"</tr>\n",
						ng, _volume [n],
						_genre [n==COURS && _cours_td ? COURS_TD : n]);
				}
				// et/ou le nombre d'heures.
				if (reste > 0.) {
					TNG_mid_table (92, 24);
					_outp_ += sprintf (_outp_,
						"	<td>%g h</td>\n"
						"	<td>&nbsp;%s</td>\n"
						"</tr>\n",
						reste,
						_genre [n==COURS && _cours_td ? COURS_TD : n]);
				}
			}
			// terminer la table puisqu'il y en a une!
			if (!pas_encore)
				_outp_ += sprintf (_outp_, "</table>\n");
			return;
	}
	/*
	 *	N'afficher le dtail que si la profondeur est > 0.
	 */
	if (depth > 0) {
		Verbatim::affiche ();
		_outp_ += sprintf (_outp_, "<table>\n");
		suiv = _base;
		if (suiv == 0)
			suiv = this;

		do {
			suiv -> affiche_detail (nom_ens);
			suiv = suiv -> _next;
			if (suiv != 0)
				_outp_ += sprintf (_outp_, "<tr><td colspan=3><hr></td></tr>\n");
		} while (suiv != 0);
		_outp_ += sprintf (_outp_, "</table>\n");
	}
}

void Verbatim::affiche (int, char*, int) {
	if (_parcours)
		_outp_ += sprintf (_outp_, "<p><b>Parcours:</b> %s\n", _parcours);
	if (_web_ref)
		_outp_ += sprintf (_outp_,
		"<p><b>Plus d'informations:</b> <a href=\"%s\">%s</a>\n", _web_ref, _web_ref);
}

/*
 *	Calcule le nombre d'heures de chaque type effectivement enseignes.
 */

float Enseignement::volume_enseigne (int th, int type)
{
	Service*	s;
	float		volume = 0.;

	// parcourir les services de tous les enseignants.
	for (Enseignant* prof = Enseignant::_head;;) {
		for (int i = 0; i < prof -> _service.length (); ++i) {
			s = (Service*) prof -> _service [i];
			// est-ce dans le bon enseignement?
			if (s -> _ens != this)
				continue;
			volume += s -> _heures [th][type];
		}

		prof = prof -> _suivant;
		if (prof == Enseignant::_head)
			break;
	}
	return volume;
}

/*
 *	Calcule s'il y a un trou ou une bosse (tb == 3: bosse, tb == 2: trou)
 */

bool Enseignement::troubosse (int tb)
{
	for (int n = 0; n < N_ENS_TYPES; ++n) {
		if (n == MP)
			continue;	// on ne compte pas les Mises en Place de TP dans les trous ou les bosses!

		if (_manque [n] < 0 && tb == 3)
			return true;	// on cherche les bosses et il y en a une.

		if (_manque [n] > 0 && tb == 2)
			return true;	// on cherche les trou et il y en a un.
	}
	return false;
}

/*
 *	Calcule le total des heures consommes dans chaque filire.
 */

void Filiere::total (float& stat, float& comp, float& ext, bool commun)
{
	Service*	s;

#ifndef PRORATA_COURS_COMMUNS
	// fini les quiproqos avec les cours communs.
	commun = false;
#endif
	// parcourir les services de tous les enseignants.
	for (Enseignant* prof = Enseignant::_head;;) {
		for (int i = 0; i < prof -> _service.length (); ++i) {
			s = (Service*) prof -> _service [i];
			// est-ce dans la bonne filire?
			if (s -> _ens -> _groupe -> _filiere != this)
				continue;
			// accumuler les diffrents types d'heures.
			stat += s -> eq_td (HEURE_STAT_A, commun) + s -> eq_td (HEURE_STAT_B, commun);
			comp += s -> eq_td (HEURE_COMP_A, commun) + s -> eq_td (HEURE_COMP_B, commun);
			ext += s -> eq_td (HEURE_COMP_EXT, commun);
		}
		prof = prof -> _suivant;
		if (prof == Enseignant::_head)
			break;
	}
}

void Filiere::total (float heures [6])
{
	// parcourir les services de tous les enseignants.
	for (Enseignant* prof = Enseignant::_head;;) {
		// les enseignements non effectus cette anne.
		if (prof -> _categorie == PCA)
			for (int i = 0; i < prof -> _service.length (); ++i) {
				Service* s = (Service*) prof -> _service [i];
				// est-ce dans la bonne filire?
				if (s -> _ens -> _groupe -> _filiere != this)
					continue;
				// accumuler les diffrents types d'heures (thoriquement ce ne devrait etre que des heures gratos).
				heures [5] += s -> eq_td (HEURE_STAT_A) + s -> eq_td (HEURE_STAT_B)
							+ s -> eq_td (HEURE_COMP_A) + s -> eq_td (HEURE_COMP_B)
							+ s -> eq_td (HEURE_STAT_EXT)
							+ s -> eq_td (HEURE_COMP_EXT)
							+ s -> eq_td (HEURE_GRAT);
			}
		else
		// les "vrais".
			for (int i = 0; i < prof -> _service.length (); ++i) {
				Service* s = (Service*) prof -> _service [i];
				// est-ce dans la bonne filire?
				if (s -> _ens -> _groupe -> _filiere != this)
					continue;
				// accumuler les diffrents types d'heures.
				heures [0] += s -> eq_td (HEURE_STAT_A) + s -> eq_td (HEURE_STAT_B);
				heures [1] += s -> eq_td (HEURE_COMP_A) + s -> eq_td (HEURE_COMP_B);
				heures [2] += s -> eq_td (HEURE_STAT_EXT);
				heures [3] += s -> eq_td (HEURE_COMP_EXT);
				heures [4] += s -> eq_td (HEURE_GRAT);
			}
		prof = prof -> _suivant;
		if (prof == Enseignant::_head)
			break;
	}
}

/*
 *	Calcule le total des heures consommes dans chaque filire avec dtail titulaires/ATER-Moniteurs.
 */

void Filiere::total (float& stat, float& comp, float& ext,
					 float& tit_stat, float& tit_hcc, float& ater_mon,
					 bool commun)
{
	Service*	s;

#ifndef PRORATA_COURS_COMMUNS
	// fini les quiproqos avec les cours communs.
	commun = false;
#endif
	// parcourir les services de tous les enseignants.
	for (Enseignant* prof = Enseignant::_head;;) {
		for (int i = 0; i < prof -> _service.length (); ++i) {
			s = (Service*) prof -> _service [i];
			// est-ce dans la bonne filire?
			if (s -> _ens -> _groupe -> _filiere != this)
				continue;
			// accumuler les diffrents types d'heures.
			switch (prof -> get_type ()) {
				case TITULAIRE:
				case AUTRE_UPS:	// au cas o ...
					tit_stat += s -> eq_td (HEURE_STAT_A, commun) + s -> eq_td (HEURE_STAT_B, commun);
					tit_hcc += s -> eq_td (HEURE_COMP_A, commun) + s -> eq_td (HEURE_COMP_B, commun);
					break;

				case MONITEUR:
				case ATERS:
					ater_mon += s -> eq_td (HEURE_STAT_A, commun) + s -> eq_td (HEURE_STAT_B, commun);
				default:
					break;
			}
			stat += s -> eq_td (HEURE_STAT_A, commun) + s -> eq_td (HEURE_STAT_B, commun);
			comp += s -> eq_td (HEURE_COMP_A, commun) + s -> eq_td (HEURE_COMP_B, commun);
			ext += s -> eq_td (HEURE_COMP_EXT, commun);
		}
		prof = prof -> _suivant;
		if (prof == Enseignant::_head)
			break;
	}
}

/*
 *	Affiche les enseignements de la filire.
 */

void Filiere::affiche (int format, char* nom_ens, int depth) {
	static char	fmt_filiere_hdr [] =
		"<p><table width=100%% cellspacing=2 cellpadding=0 border=0>\n"
		"<tr>\n"
		"	<td width=30><img src=left-round-30x60.png></td>\n";

	static char fmt_filiere_detail [] =
		"	<td>\n"
		"<table height=60 width=100%% cellspacing=0 cellpadding=0 border=0>\n"
		"	<tr><td align=center bgcolor=\"%s\">&nbsp;<a href=\"enseignants_%s.cmd\"><font color=\"%s\">Enseignants</a>&nbsp;</td></tr>\n"
		"	<tr><td align=center bgcolor=\"%s\"><a href=\"filiere_%s.cmd\"><font color=\"%s\">Dtail</a></td></tr>\n"
		"</table>\n"
		"</td>\n"
		"<td align=center width=80%% height=60 bgcolor=\"%s\">\n"
		"<font size=5 color=#000000 face=Helvetica>%s</font>\n"
		"</td>\n"
		"<td>\n"
		"<table height=60 width=100%% cellspacing=0 cellpadding=0 border=0>\n"
		"	<tr><td align=center bgcolor=\"%s\"><a href=\"trous_%s.cmd\"><font color=\"%s\">Trous</a></td></tr>\n"
		"	<tr><td align=center bgcolor=\"%s\">&nbsp;<a href=\"bosses_%s.cmd\"><font color=\"%s\">Bosses</a>&nbsp;</td></tr>\n"
		"</table>\n";

	static char fmt_filiere_edit [] =
		"<td align=center width=100%% height=60 bgcolor=\"%s\">\n"
		"<font size=5 color=#000000 face=Helvetica>%s%s</font>\n";

	static char fmt_filiere_foot [] =
		"	</td>\n"
		"	<td width=30><img src=right-round-30x60.png></td>\n"
		"</tr>\n"
		"</table><p>\n";

	static char	fmt_recap [] = 
		"<center><b>Total des heures enseignes dans la filire</b><p>\n";

	static char	fmt_total [] =
		"<table cellpadding=2 border=2>\n"
			"<tr bgcolor=#ffccff><td><b>Heures statutaires</b></td><td align=right><b>%g</b></td></tr>\n"
			"<tr bgcolor=#ffccff><td><b>Heures complmentaires</b></td><td align=right><b>%g</b></td></tr>\n"
			"<tr bgcolor=#ffccff><td><b>Heures extrieures</b></td><td align=right><b>%g</b></td></tr>\n"
		"</table></center><br>\n";

	nom_ens = Enseignant::nospaces (nom_ens);
	char*	prev_outp = _outp_;	// pour ne rien afficher dans trous/bosses quand il n'y en a pas!
	char*	sv_outp = 0;
	// afficher l'entte.
	_outp_ += sprintf (_outp_, "%s", fmt_filiere_hdr);
	// si c'est pour choisir un cours  faire, ou si on l'a dj, ne pas afficher le dtail de la filire.
	if (nom_ens [0] != 0 || depth > 0) {
		if (format == 2) {
			_outp_ += sprintf (_outp_, fmt_filiere_edit, page_colors [TitreNormal], "<font color=red>Manques en :</font> ", _nom);
		} else if (format == 3) {
			_outp_ += sprintf (_outp_, fmt_filiere_edit, page_colors [TitreNormal], "<font color=red>Surcharge en :</font> ", _nom);
		} else {
			_outp_ += sprintf (_outp_, fmt_filiere_detail,
						page_colors [TitreEmphase], _identification, page_colors [Detail],	// dtails.
						page_colors [TitreEmphase], _identification, page_colors [Detail],	// trous.
						page_colors [TitreNormal],
						_nom,
						page_colors [TitreEmphase], _identification, page_colors [Detail],	// bosses.
						page_colors [TitreEmphase], _identification, page_colors [Detail]);	// enseignants.
		}
	} else {
		_outp_ += sprintf (_outp_, fmt_filiere_detail,
					page_colors [TitreEmphase], _identification, page_colors [Detail],	// dtails.
					page_colors [TitreEmphase], _identification, page_colors [Detail],	// trous.
					page_colors [TitreNormal],
					_nom,
					page_colors [TitreEmphase], _identification, page_colors [Detail],	// bosses.
					page_colors [TitreEmphase], _identification, page_colors [Detail]);	// enseignants.
	}
	// afficher la fin.
	_outp_ += sprintf (_outp_, "%s", fmt_filiere_foot);
	if (depth <= 0) {
		// sans les dtails.
		_outp_ += sprintf (_outp_,
			"<table width=100%% cellspacing=2 cellpadding=0 border=0>\n");

		for (int n = 0; n < _groupes.length (); ++n) {
			_outp_ += sprintf (_outp_, "<tr><td width=120>&nbsp;</td><td>\n");
			((Verbatim*) _groupes [n]) -> affiche (format, nom_ens, depth-1);
			_outp_ += sprintf (_outp_, "</td></tr>\n");
		}
		_outp_ += sprintf (_outp_, "</table>\n");
		return;
	}
	// le total des heures (sauf pour les trous et bosses)
	if (format < 2) {
		float	stat = 0.;
		float	comp = 0.;
		float	ext = 0.;

		// le verbiage.
		Verbatim::affiche ();
		total (stat, comp, ext);
		_outp_ += sprintf (_outp_, "%s", fmt_recap);
		_outp_ += sprintf (_outp_, fmt_total, stat, comp, ext);
	}
	// avec le dtail.
	sv_outp = _outp_;
	for (int n = 0; n < _groupes.length (); ++n)
		((Groupe*) _groupes [n]) -> affiche (format, nom_ens, depth-1);
	// dans le cas des bosses/trous, si c'est vide, virer le titre.
	if (format >= 2 && _outp_ == sv_outp) {
		_outp_ = prev_outp;
		*_outp_ = 0;
	}
}


/*
 *	Retourne la liste de tous les profs qui enseignent dans ce groupe.
 */

int Groupe::liste_enseignants (Enseignant** liste_ens, int size)
{
	int	n_ens = 0;

	// parcourir la liste des enseignements du groupe.
	for (int n = 0; n < _groupes.length (); ++n) {
		Enseignement* ens = (Enseignement*) _groupes [n];
		// ne compter que le groupe original.
		if (ens -> _groupe != this && ens -> _next == 0)
			continue;

		// parcourir la liste des enseignants.
		for (Enseignant* prof = Enseignant::_head;;) {
			Service*	s = prof -> get_service (ens);

			if (s != 0 && prof -> _email != 0) {
				// ne le mettre qu'une fois!
				for (int i = 0; i < n_ens; ++i)
					if (liste_ens [i] == prof)
						goto suivant;

				liste_ens [n_ens++] = prof;
				// peu probable, mais ne pas se planter btement.
				if (n_ens > size)
					return n_ens;
			}
		suivant:
			prof = prof -> _suivant;
			if (prof == Enseignant::_head)
				break;
		}
	}
	return n_ens;
}

void Groupe::affiche (int format, char* nom_ens, int depth) {
	static char	aide [] =
		"<hr><b>&nbsp;&nbsp;Slectionnez l'enseignement qui vous intresse "
		"pour avoir le formulaire de saisie.</b><br><hr><br>";

	static char	fmt_group [] =
		"<font size=+1><a href=\"groupe_%s%s%s.cmd\">%s</a></font><br>\n";

	nom_ens = Enseignant::nospaces (nom_ens);
	// affichage des trous ou les bosses.
	if (format >= 2) {
		bool	non_empty = false;

		for (int n = 0; n < _groupes.length (); ++n) {
			if (non_empty)
				break;

			Enseignement* ens = (Enseignement*) _groupes [n];
			// ne le compter que dans le groupe original.
			if (ens -> _groupe == this || ens -> _next != 0)
				non_empty = ens -> troubosse (format);
		}
		if (!non_empty)
			return;

		_outp_ += sprintf (_outp_, "<blockquote>\n");
		TNG_top_table (92, _nom);
		for (int n = 0; n < _groupes.length (); ++n) {
			Enseignement* ens = (Enseignement*) _groupes [n];
			// ne le compter que dans le groupe original.
			if (ens -> _groupe == this || ens -> _next != 0)
				ens -> affiche (format, nom_ens, depth-1);
		}
		TNG_bot_list (92);
		_outp_ += sprintf (_outp_, "</blockquote>\n");
		return;
	}
	if (depth > 0) {
		// afficher un peu d'aide pour la saisie.
		if (nom_ens [0] != 0)
			_outp_ += sprintf (_outp_, "%s", aide);
		// puis le titre.
		char	tc [1024];

		sprintf (tc, "<a href=\"mail_ens_GR_%s.cmd\"><img src=email-trans.gif align=top border=0></a> &nbsp; &nbsp;%s",
					 _identification, _nom);
		TNG_title (tc);
		// la description du module.
		Verbatim::affiche ();
		// et enfin la liste des cours.
		_outp_ += sprintf (_outp_, "<blockquote>\n");
		TNG_top_list (30, _nom);
		for (int n = 0; n < _groupes.length (); ++n) {
			Enseignement* ens = (Enseignement*) _groupes [n];
			// ne le compter que dans le groupe original.
			if (format == 0 || ens -> _groupe == this || ens -> _next != 0) {
				TNG_mid1_list (30, format >= 2 ? 0 : 30);
				_outp_ += sprintf (_outp_, "<td align=left>\n");
				ens -> affiche (format, nom_ens, depth-1);
				TNG_mid2_list ();
			}
		}
		TNG_bot_list ();
		_outp_ += sprintf (_outp_, "</blockquote>\n");
	} else {
		_outp_ += sprintf (_outp_, fmt_group,
							_identification, nom_ens[0] ? Service::_sep : "", nom_ens,
							_nom);
	}
}

/*
 *	Affiche le dtail de l'enseignement (format table)
 */

#define g_print(f) { \
	if (f == 0) \
		_outp_ += sprintf (_outp_, fmt_table_s, "&nbsp;"); \
	else \
		_outp_ += sprintf (_outp_, fmt_table_g, f); \
}

#define c_print(a,b,f) { \
	if (f == 0) \
		_outp_ += sprintf (_outp_, fmt_table_s, "&nbsp;"); \
	else { \
		if (a == b) \
			_outp_ += sprintf (_outp_, fmt_table_s, "1"); \
		else \
			_outp_ += sprintf (_outp_, fmt_table_coeff, a, b); \
	} \
}

void Filiere::table ()
{
	static char	fmt_total [] =
		"<center><table bgcolor=#000000 cellpadding=2 border=2>\n"
			"<tr bgcolor=#004070><td><b>Total de la filire</b></td><td align=right><font face=verdana><b>&nbsp;%d&nbsp;</b></font></td></tr>\n"
			"<tr bgcolor=#a07020><td><b>Manque</b></td><td align=right><font face=verdana><b>&nbsp;%d&nbsp;</b></font></td></tr>\n"
			"<tr bgcolor=#700040><td><b>Pas enseignes cette anne</b></td><td align=right><font face=verdana><b>&nbsp;%d&nbsp;</b></font></td></tr>\n"
			"<tr bgcolor=#400070><td><b>Heures statutaires</b></td><td align=right><font face=verdana><b>&nbsp;%d&nbsp;</b></font></td></tr>\n"
			"<tr bgcolor=#400070><td><b>Heures complmentaires</b></td><td align=right><font face=verdana><b>&nbsp;%d&nbsp;</b></font></td></tr>\n"
			"<tr bgcolor=#400070><td><b>Heures statutaires ext.</b></td><td align=right><font face=verdana><b>&nbsp;%d&nbsp;</b></font></td></tr>\n"
			"<tr bgcolor=#400070><td><b>Heures complmentaires ext.</b></td><td align=right><font face=verdana><b>&nbsp;%d&nbsp;</b></font></td></tr>\n"
			"<tr bgcolor=#400070><td><b>Heures bnvoles</b></td><td align=right><font face=verdana><b>&nbsp;%d&nbsp;</b></font></td></tr>\n"
		"</table></center><br>\n";
	float	heures [6];
	float	total_filiere = 0.;

	// la table rcapitulative.
	TNG_title ((char*)"Rcapitulatif des enseignements");
	_outp_ += sprintf (_outp_, "<table border=2 cellpadding=2 cellspacing=1>\n");
	for (int n = 0; n < _groupes.length (); ++n)
		total_filiere += ((Groupe*)_groupes [n]) -> table ();

	_outp_ += sprintf (_outp_, "</table><p>\n");
	// le total des heures (sauf pour les trous)
	for (int i = 0; i < 6; ++i) heures [i] = 0.;
	total (heures);
	char	tc [128];
	sprintf (tc, "Total %s", _nom);
	TNG_title (tc);
	int	manque = (int) (total_filiere+0.5);

	for (int i = 0; i < 6; ++i)
		manque -= (int) (heures[i]+0.5);

	_outp_ += sprintf (_outp_, fmt_total,
				(int) (total_filiere+0.5),
				manque,
				(int) (heures[5]+0.5),
				(int) (heures[0]+0.5),
				(int) (heures[1]+0.5),
				(int) (heures[2]+0.5),
				(int) (heures[3]+0.5),
				(int) (heures[4]+0.5));
}

float Groupe::table ()
{
	float	total_groupe = 0.;
	static char	fmt_table [] =
		"<tr><td colspan=15 bgcolor=#404060 align=center><b>%s</b></td></tr>"
		"<tr>"
		"<td bgcolor=#004070 align=center><b>Intitul</b></td>"
		"<td bgcolor=#004070 align=center><b>Cours</b></td>"
		"<td bgcolor=#004070 align=center><b>Coeff</b></td>"
		"<td bgcolor=#004070 align=center><b>C/TD</b></td>"
		"<td bgcolor=#004070 align=center><b>#&nbsp;grp</b></td>"
		"<td bgcolor=#004070 align=center><b>TD</b></td>"
		"<td bgcolor=#004070 align=center><b>#&nbsp;grp</b></td>"
		"<td bgcolor=#004070 align=center><b>TP</b></td>"
		"<td bgcolor=#004070 align=center><b>#&nbsp;grp</b></td>"
		"<td bgcolor=#004070 align=center><b>Total</b></td>"
		"<td bgcolor=#004070 align=center><b>Manque</b></td>"
		"<td bgcolor=#004070 align=center><b>Cours</b></td>"
		"<td bgcolor=#004070 align=center><b>Cours/TD</b></td>"
		"<td bgcolor=#004070 align=center><b>TD</b></td>"
		"<td bgcolor=#004070 align=center><b>TP</b></td>"
		"</tr>\n";

	_outp_ += sprintf (_outp_, fmt_table, _nom);
	for (int n = 0; n < _groupes.length (); ++n) {
		Enseignement* ens = (Enseignement*) _groupes [n];
		// ne le compter que dans le groupe original.
		if (ens -> _groupe == this || ens -> _next != 0)
			total_groupe += ens -> table ();
	}
	return total_groupe;
}

float Enseignement::table (void)
{
	float			total = 0.;
	float			coeff;
	Enseignement*	ref = _base;
	static char		fmt_table_s [] = "<td align=center><b>%s</b></td>\n";
	static char		fmt_table_g [] = "<td align=center><b>%g</b></td>\n";
	static char		fmt_table_coeff [] = "<td align=center><b>%d/%d</b></td>\n";

	_outp_ += sprintf (_outp_, "<tr>");
	// le nom de l'enseignement.
	_outp_ += sprintf (_outp_, "<td align=center><b><a href=\"cours_%s.cmd\">%s - %s</a></b></td>",
								_identification,
								_groupe -> _identification,
								ref ? ref -> _intitule : _intitule);
	// le cours
	if (! _cours_td) {
		if (ref != 0) {
#ifdef PRORATA_COURS_COMMUNS
			// il s'agit de TD et/ou TP d'un enseignement commun.
			// calculer le coefficient d'imputation  la filire.
			coeff = ((float) _total_grp) / ((float) ref -> _total_grp);
			// retrouver le nombre d'heures effectues en cours dans le cours de base.
			// et lui affecter le taux et le coefficient.
			//
			// (on suppose que pour les cours communs, il n'y a qu'un seul groupe de cours)
			total += ref -> _volume [COURS] * Service::taux [COURS] * coeff;
			g_print (ref -> _volume [COURS]);
			c_print (_total_grp, ref -> _total_grp, ref -> _volume [COURS]);
#else
			g_print (0);
			c_print (_total_grp, ref -> _total_grp, 0);
#endif
		} else {
			// il s'agit du cours d'un enseignement commun de base.
			g_print (_volume [COURS]);
#ifdef PRORATA_COURS_COMMUNS
			if (_total_grp == 0)
#else
			if (true)
#endif
			{
				coeff = 1.;
				c_print (0, 0, _volume [COURS]);
			} else {
				int	num = _ngrp [TD] + _ngrp [TP];
				coeff = ((float) num) / ((float) _total_grp);
				c_print (num, _total_grp, _volume [COURS]);
			}
			total += _ngrp [COURS] * _volume [COURS] * Service::taux [COURS] * coeff;
		}
		_outp_ += sprintf (_outp_, fmt_table_s, "&nbsp;");	// pas de cours/td.
		if (_ngrp [COURS] <= 1)
			// pas de groupes cours/td.
			_outp_ += sprintf (_outp_, fmt_table_s, "&nbsp;");
		else
			// nb de groupes de cours.
			_outp_ += sprintf (_outp_, fmt_table_g, (float) _ngrp [COURS]);
	} else {
	// les cours/td
		_outp_ += sprintf (_outp_, fmt_table_s, "&nbsp;");	// pas de cours
		_outp_ += sprintf (_outp_, fmt_table_s, "&nbsp;");	// pas de coeff
		g_print (_volume [COURS]);
		g_print ((float) _ngrp [COURS]);
		total += _ngrp [COURS] * _volume [COURS] * Service::taux [COURS_TD];
	}
	// les td, tp, (sans les mp).
	for (int i = TD; i < N_ENS_TYPES-1; ++i) {
		g_print (_volume [i]);
		g_print ((float) _ngrp [i]);
		total += _volume [i] * _ngrp [i] * Service::taux [i];
	}
	// le total  enseigner.
	g_print (total);
	// ce qui manque (les cours communs ne sont compts QUE dans l'enseignement de base).
	float	manque = 0.;
	for (int i = 0; i < N_ENS_TYPES; ++i) {
		// on ne compte pas les Mises en Place de TP dans les trous!
		if (i == MP || _manque [i] <= 0)
			continue;

		manque += _manque [i] * Service::taux [i==COURS && _cours_td ? COURS_TD : i];
	}
	g_print (manque);
	// la liste des enseignants.
	int	i;
	if (_cours_td) {
		_outp_ += sprintf (_outp_, "<td>&nbsp;</td>");	// pas de cours.
		i = COURS;
	} else {
		_outp_ += sprintf (_outp_, "<td>&nbsp;");
		affiche_profs (COURS);
		_outp_ += sprintf (_outp_, "<td>&nbsp;</td>");	// pas de cours/td.
		i = TD;
	}
	// ne pas afficher les mp.
	for (; i < N_ENS_TYPES-1; ++i) {
		_outp_ += sprintf (_outp_, "<td>&nbsp;");
		affiche_profs (i);
	}
	_outp_ += sprintf (_outp_, "</tr>\n");
	return total;
}
