/*****************************************************************

  Copyright (c) 1999 - 2003 by Patrick Amar - pa@lri.fr

    http://www.lri.fr/~pa

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

#ifndef DEFENS_H
#define DEFENS_H

#ifndef bcopy
#include <string.h>
#define index(s,c) strchr((s),(c))
#define rindex(s,c) strrchr((s),(c))
#define bcopy(s,d,n) memcpy((d),(s),(n))
#define bcmp(a,b,n) memcmp((a),(b),(n))
#define bzero(a,n) memset((a),0,(n))
#endif

#include "Vector.h"
#include <sys/types.h>	// pour avoir time_t.

class Enseignant;	// juste besoin d'un pointeur.
class Filiere;		// juste besoin d'un pointeur.

/*
 *		    Verbatim
 *			   |
 *		   +-----+-------+
 *		    |		     |
 *	  Enseignement   FBase
 *					 |
 *				    +--+---+
 *				    |		|
 *			      Groupe   Filiere
 */

class Verbatim {
public:
	char*	_verbatim;			// commentaire.
	char*	_web_ref;			// page web de rfrence.
	char*	_parcours;			// commune  divers parcours, facultative, etc...
	bool	_su_only;			// accessible qu'au superuser.

	Verbatim (char* verb, char* wr =0) :
		_verbatim (verb), _web_ref (wr), _parcours (0), _su_only(false)
	{}
	virtual ~Verbatim () {}
	virtual void affiche (int =0, char* =(char*)"", int depth =0);
};

class FBase : public Verbatim {
public:
	Vector	_groupes;
	Enseignant*	_responsable;	// le responsable de la filire.
	char*	_identification;	// identification de la filire
	char*	_nom;				// son nom

	FBase (char* id, char* nom, char* verb, char* wr =0) :
		Verbatim (verb, wr), _groupes (0)
	{
		_identification = id;
		_nom = nom;
	}
	virtual ~FBase () {}
};

class Groupe : public FBase {
public:
	Filiere*	_filiere;			// lien arrire vers la filire.
	int			_effectifs;			// nombre d'tudiants dans le groupe.

	Groupe (char* id, char* nom, int effectifs =0, char* verb =0, char* wr =0)
		: FBase (id, nom, verb, wr)
	{
		_responsable = 0;
		_filiere = 0;
		_effectifs = effectifs;
	}
	virtual ~Groupe () {}
	void affiche (int =0, char* =(char*)"", int depth =0);
	float table (void);
	int liste_enseignants (Enseignant** liste_ens, int size);
};

enum Periode { ANNEE, SEMESTRE_UN, SEMESTRE_DEUX };

class Enseignement : public Verbatim {
public:
	static Enseignement*	_tous_les_enseignements [10240];
	static int				_nb_enseignements;
	Groupe*	_groupe;				// lien arrire vers le groupe.
	char*	_identification;		// identification du cours.
	char*	_apogee_id;				// identification Apoge.
	char*	_intitule;				// titre.
	char*	_periode;				// priode dans l'anne.
	float	_manque [N_ENS_TYPES];	// volumes horaire manquant.
	float	_volume [N_ENS_TYPES];	// volumes horaires des cours, td, tp, mp.
	int		_ngrp [N_ENS_TYPES];	// nombre de groupes de chaque.
	bool	_cours_td;				// Cours/TD ou cours seulement?
	bool	_charge_np;				// Charge non partageable.
	bool	_charge;				// Charge partageable.
	bool	_lien;					// Lien vers un autre cours.
	bool	_form_initiale;			// formation initiale / continue.
	int		_total_grp;				// nombre total de groupes (pour la facturation!)
	int		_effectifs;				// nombre d'tudiants qui suivent cet enseignement.
	char*	_activite_pedag;		// activit pdagogique.
	Enseignement*	_next;			// Cours commun suivant (liste circulaire).
									// zro si le cours n'est commun  rien.
	Enseignement*	_base;			// Enseignement de base pour un cours commun.
									// zro pour un cours de base!

public:
	static Enseignement* name2ens (char* ident);
	Enseignement (char* ident, char* apogee) : Verbatim (0), _identification (ident), _apogee_id (apogee), _lien(false) {}
	Enseignement (char* identification, char* apogee, char* intitule, char* periode,
				  int effectifs,
				  float volume [N_ENS_TYPES], int ngrp [N_ENS_TYPES],
				  char* act_pedago, char* verb =0, char* wr =0, char* parcours =0,
				  bool cours_td =false, bool charge =false,
				  bool form_init =true, bool su_only =false);

	virtual ~Enseignement () {}
	char*			get_intitule (bool anchor);	// traite les cours commun.
	char*			get_intitule (void) {	// ne traite pas les cours communs.
						return _base ? _base -> _intitule : _intitule;
	}
	float			volume_enseigne (int, int);
	void			formulaire (Enseignant*);
	Enseignement*	suivant ();
	Enseignement*	precedent ();
	void			affiche (int =0, char* =(char*)"", int depth =0); // depth == 0 => le plus dtaill.
	float			table (void);
	bool			troubosse (int);	// indique s'il y a un trou ou une bosse.
	void			affiche_profs (int);
	void			affiche_detail (char*);
};

class Filiere : public FBase {
friend class Enseignant;
public:
	static void init (const char* name);
	static Filiere*	filieres [128];
	static Filiere*	name2filiere (char* ident);
	static Groupe*	name2group (char* ident);
	static Enseignement* name2ens (char* ident);

	Filiere (char* id, char* nom, char* responsable, char* verb =0, char* wr =0);
	void affiche (int =0, char* =(char*)"", int depth =0);
	void table ();
	void total (float heures [6]);
	void total (float& stat, float& comp, float& ext, bool commun=true);
	void total (float& stat, float& comp, float& ext, float& tit_stat,
		 		float& tit_hcc, float& ater_mon, bool commun = false);
};

#endif
