/*****************************************************************

  Copyright (c) 1999 - 2003 by Patrick Amar - pa@lri.fr

    http://www.lri.fr/~pa

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

/*
 *	Analyse du fichier de description des modules d'enseignement.
 */

/*
	Grammaire du langage de description d'un module:

description	: 'Description' chaine | VIDE
responsable	: 'Responsabl '=' chaine | VIDE
effectifs		: 'Effectifs' '=' nombre | VIDE
voeux		: filiere | filiere voeux
filiere		: 'Filiere' nom intitule responsable description '{' s_groupe '}'
s_groupe		: groupe | groupe s_groupe
groupe		: 'Groupe' ident intitule effectifs [SuperU] description '{' s_cours '}'
s_cours		: cours | cours s_cours
cours		: 'Cours' ident intitule periode description description
courscom	: 'CoursCommun' ident ident_ref periode description description
lien			: 'Lien' ident_ref
charge		: 'Charge' ident intitule periode description description

 */

#include <stdio.h>
#include <string.h>
#include "Lexer.h"
#include "Enseignant.h"	// Enseignement.h est dj inclus par Service.h

Filiere*	Filiere::filieres [128];

#define T_FILIERE		T_VOID+1
#define T_DESCRIPTION	T_FILIERE+1
#define T_WEBREF		T_DESCRIPTION+1
#define T_PARCOURS		T_WEBREF+1
#define T_GROUPE		T_PARCOURS+1
#define T_COURS			T_GROUPE+1
#define T_COURSCOMMUN	T_COURS+1
#define T_LIEN			T_COURSCOMMUN+1
#define T_CHARGE		T_LIEN+1
#define T_RESPONSABLE	T_CHARGE+1
#define T_EFFECTIFS		T_RESPONSABLE+1
#define T_ACT_PEDAGO	T_EFFECTIFS+1
#define T_SU_ONLY		T_ACT_PEDAGO+1
#define T_APOGEE		T_SU_ONLY+1
#define T_FC			T_APOGEE+1
#define T_FI			T_FC+1

#define T_LBRA			T_FI+1
#define T_RBRA			T_LBRA+1
#define T_EQ			T_RBRA+1

#define _KEYWORDS_ \
	"Filiere", "Description", "Webref", "Parcours", "Groupe", "Cours", "CoursCommun", "Lien", \
	"Charge", "Responsable", "Effectifs", "CP", "SuperU", "AP", "FC", "FI"


static const char*	keywords [] = {
	_KEYWORDS_,
	0
};

static const char*	err [] = {
	"a character string", "the end of file", "a number", "????",
	_KEYWORDS_,
	"left brace", "right brace", "equal sign",
	0
};

static Special	ctype [] = {
	{ '{',	T_LBRA },
	{ '}',	T_RBRA },
	{ '=',	T_EQ },
	{ 0, 0 }
};

/*
 *	Analyseur syntaxique LL(1) rcursif descendant.
 */

/*
 *	responsable	: Responsable = chaine | VIDE
 */

static char* p_responsable (Lexer& lx)
{
	Lexeme	l = lx.lex ();

	if (l._type == T_RESPONSABLE) {
		(void) lx.expect (T_EQ);
		l = lx.expect (T_STRING);
		return l._string;
	}
	lx.unlex (l);
	return 0;
}

/*
 *	effectifs	: Effectifs = nombre | VIDE
 */

static int p_effectifs (Lexer& lx)
{
	Lexeme	l = lx.lex ();

	if (l._type == T_EFFECTIFS) {
		(void) lx.expect (T_EQ);
		l = lx.expect (T_NUMBER);
		return (int) (long) l._string;
	}
	lx.unlex (l);
	return 0;
}

/*
 *	description	: 'Description' chaine | 'Webref' chaine | 'Parcours' chaine | VIDE
 */

static void p_description (Lexer& lx, char*& descr, char*& wr, char*&pc)
{
	for (;;) {
		Lexeme	l = lx.lex ();

		switch (l._type) {
			case T_DESCRIPTION:
				l = lx.expect (T_STRING);
				descr = l._string;
				continue;

			case T_WEBREF:
				l = lx.expect (T_STRING);
				wr = l._string;
				continue;

			case T_PARCOURS:
				l = lx.expect (T_STRING);
				pc = l._string;
				continue;

			default:
				break;
		}
		lx.unlex (l);
		return;
	}
}

/*
 *	Traiter les rfrences en avant pour les cours communs.
 */

static void forward_references (int forward_refs)
{
	while (forward_refs > 0) {
		for (Filiere** f = Filiere::filieres; *f != 0; ++f) {
			for (int n = 0; n < (*f) -> _groupes.length (); ++n) {
				Groupe*	g = (Groupe*) (*f) -> _groupes [n];
				for (int i = 0; i < g -> _groupes.length (); ++i) {
					Enseignement* ens = (Enseignement*) g -> _groupes [i];

					if (ens -> _base == (Enseignement*) 2) {
						Enseignement*	ref = Enseignement::name2ens (ens -> _identification);	// le cours rfrenc.
						if (ref == 0) {
							zlog ("Le cours commun %s 'existe pas!\n",
											 ens -> _identification);
							exit (1);
							return;
						}
						// encore une autre rfrence avant.
						if (ref -> _base == (Enseignement*) 2)
							continue;
						// c'est le bon.
						g -> _groupes [i] = ref;
						delete ens;
						// peut-etre est-ce termin?
						if (--forward_refs == 0)
							return;
					}
				}
			}
		}
	}
}

/*
 *	cours :		'Cours' ident intitule periode code_act_pedago ['Effectifs' '=' <nbr>] [SuperU] [Description] volumes
 *	charge :		'Charge' ident intitule periode code_act_pedago [SuperU] [Description] volumes
 *	courscom :	'CoursCommun' ident ident_ref periode [SuperU] [Description] volumes
 *	lien :			'Lien' ident ident_ref
 */

static int	forward_refs = 0;	// Nombre de rfrences  un cours commun avant sa dfinition.
static int	forward_refs_2 = 0;	// Nombre de rfrences  un cours commun avant sa dfinition.

static Enseignement* p_cours (Lexer& lx)
{
	Enseignement*	ens;
	Lexeme			l;
	char*			ident;
	char*			apogee;
	char*			intitule;
	char*			periode;
	char*			description = 0;
	char*			horaires;
	char*			d;
	bool			cours_td = false;
	bool			charge = false;
	bool			commun = false;
	bool			lien = false;
	bool			form_initiale = true;
	int				ngrp [N_ENS_TYPES];
	float			volume [N_ENS_TYPES];
	float			vol;
	char			genre [128];
	char*			act_pedago = (char*)"AP0";
	bool			su_only = false;
	char*			parcours = 0;
	char*			web_ref = 0;
	int				effectifs = 0;

	// mettre les volumes  zro.
	for (int i = 0; i < N_ENS_TYPES; ++i) {
		ngrp [i] = 0;
		volume [i] = 0.0;
	}

	l = lx.lex ();
	switch (l._type) {
		case T_CHARGE:		charge = true; break;
		case T_COURSCOMMUN:	commun = true; break;
		case T_LIEN:		lien = true; break;
		case T_COURS:		break;
		default:
			lx.unlex (l);
			(void) lx.expect (0, "'Cours', 'CoursCommun', 'Lien' or 'Charge'");
			return 0;
	}
	l = lx.expect (T_STRING);
	ident = l._string;
	// identification APOGEE facultative pour l'instant.
	apogee = 0;
	l = lx.lex ();
	if (l._type == T_APOGEE) {
		(void) lx.expect (T_EQ);
		apogee = lx.expect (T_STRING)._string;
	} else
		lx.unlex (l);
	// rfrence arrire  un cours commun?
	Enseignement* ref = Enseignement::name2ens (ident);
	if (ref != 0)
		return ref;

	l = lx.lex ();
	switch (l._type) {
		case T_FC:
			form_initiale = false;
			// fall thru.
		case T_FI:
			l = lx.expect (T_STRING);
			// fall thru.

		case T_STRING:
			intitule = l._string;
			break;

		case T_COURS:
		case T_RBRA:
			lx.unlex (l);
			// rfrence avant  un cours commun?
			ens = new Enseignement (ident, apogee);
			ens -> _base = (Enseignement*) 2;	//  voir plus tard.
			++forward_refs_2;
			return ens;

		default:
			lx.unlex (l);
			(void) lx.expect (0, "'FC', 'Cours', '}' or course name");
			return 0;
	}
	if (lien) {
		ens = new Enseignement (ident, apogee);
		ens -> _lien = true;
		ens -> _intitule = intitule;
	} else {
		l = lx.expect (T_STRING); periode = l._string;
		if ((l = lx.lex (), l._type) == T_ACT_PEDAGO) {
			(void) lx.expect (T_EQ);
			act_pedago = lx.expect (T_STRING)._string;
		} else
			lx.unlex (l);

		if (! charge)
			effectifs = p_effectifs (lx);

		if ((l = lx.lex (), l._type) == T_SU_ONLY) {
			su_only = true;
		} else
			lx.unlex (l);

		p_description (lx, description, web_ref, parcours);
		l = lx.expect (T_STRING); horaires = l._string;

		// Tout mettre en minuscules.
		for (char* p = horaires; *p; ++p)
			if ('A' <= *p && *p <= 'Z')
				*p += 'a' -'A';
		// Est-ce une charge ?
		if (charge) {
			if (2 != sscanf (horaires, "%f h %s", &vol, genre) ||
				0 != strcmp ("td", genre))
			{
				// rien compris !
				zlog ("Format error: %s\n", horaires);
				return 0;
			}
			volume [TD] = vol;
			ngrp [TD] = 1;
			return new Enseignement (ident, apogee, intitule, periode, effectifs, volume,
									 ngrp, act_pedago, description, web_ref, parcours,
									 false, true, form_initiale, su_only);
		}
		// Non, c'est un cours.
		for (char* p = d = horaires;; ++p)
			if (*p == ',' || *p == 0) {
				char	sv = *p;
				int		num;

				*p = 0;
				num = 1;
				if (2 != sscanf (d, "%f h %s", &vol, genre) &&
					3 != sscanf (d, "%d * %f h %s", &num, &vol, genre))
				{
					// rien compris !
					*p = sv;
					lx.warning ((char*)"Format error");
					exit (1);
					return 0;
				}
				// affecter les donnes aux bons items.
				if (!strcmp ("cours", genre)) {
					ngrp [COURS] = num;
					volume [COURS] = vol;
					if (commun)
						zlog ("Bizarre de d'avoir du cours dans un cours commun!\n");
				}
				else if (!strcmp ("cours/td", genre)) {
					ngrp [COURS] = num;
					volume [COURS] = vol;
					cours_td = true;
					if (commun)
						zlog ("Bizarre de d'avoir du cours/TD dans un cours commun!\n");
				}
				else if (!strcmp ("td", genre)) {
					ngrp [TD] = num;
					volume [TD] = vol;
				}
				else if (!strcmp ("tp", genre)) {
					ngrp [TP] = num;
					volume [TP] = vol;
				}
				else if (!strcmp ("mp", genre)) {
					ngrp [MP] = num;
					volume [MP] = vol;
				}
				else if (!strcmp ("dtd", genre)) {
					ngrp [DTD] = num;
					volume [DTD] = vol;
				}
				else if (!strcmp ("std", genre)) {
					ngrp [STD] = num;
					volume [STD] = vol;
				}
				else if (!strcmp ("sss", genre)) {
					ngrp [SSS] = num;
					volume [SSS] = vol;
				}
				else {
					*p = sv;
					zlog ("Genre ?? %s\n", genre);
					return 0;
				}
				d = p+1;	// au suivant.
				if ((*p = sv) == 0)
					break;
			}
		int ngroups = 0;
		for (int i = COURS; i <= TP; ++i)
			if (ngroups < ngrp [i])
				ngroups = ngrp [i];
		effectifs *= ngroups;
		ens = new Enseignement (ident, apogee, intitule, periode, effectifs, volume,
								ngrp, act_pedago, description, web_ref, parcours,
								cours_td, false, form_initiale, su_only);
	}
	// Traitement spcifique des cours communs.
	if (commun || lien) {
		Enseignement*	ref = Filiere::name2ens (intitule);	// le cours rfrenc.
		if (ref == 0) {
			++forward_refs;
			ens -> _base = (Enseignement*) 1;	//  voir plus tard.
		} else {
			ens -> _base = ref;
			ens -> _intitule = 0;
			if (lien)
				ens -> _periode = ref -> _periode;
			else {
				ref -> _effectifs += effectifs;
				ref -> _total_grp += ngrp [TD] + ngrp [TP];	// augmenter le nombre total de groupes.
				if (ref -> _next == 0)
					ref -> _next = ens;
				else {
					Enseignement*	p;
					for (p = ref -> _next; p -> _next != 0; p = p -> _next)
						;
					p -> _next = ens;
				}
			}
		}
	}
	return ens;
}

/*
 * s_cours : cours | cours s_cours
 */

static void p_s_cours (Lexer& lx, Groupe* g)
{
	Lexeme			l;
	Enseignement*	ens;

	do {
		ens = p_cours (lx);
		if (ens -> _groupe == 0) {
			ens -> _groupe = g;		// le lien arrire (que pour le cours de base).
			// si le groupe n'est acccessible qu'au surutilisateur, ses enseignements aussi.
			if (g -> _su_only)
				ens -> _su_only = true;
		}
		g -> _groupes.add (ens);	// ajouter le nouveau cours
		l = lx.lex ();				// au groupe 'g' .
		lx.unlex (l);
		} while (l._type == T_COURS || l._type == T_COURSCOMMUN ||
				 l._type == T_CHARGE || l._type == T_LIEN);
}

/*
 *	groupe : 'Groupe' ident intitule effectifs [SuperU] description '{' s_cours '}'
*/

static Groupe* p_groupe (Lexer& lx)
{
	Lexeme	l;
	Groupe*	g;
	char*	ident;
	char*	intitule;
	char*	description = 0;
	char*	web_ref = 0;
	char*	parcours = 0;
	int		effectifs;
	char*	responsable = 0;
	bool	su_only = false;

	(void) lx.expect (T_GROUPE);
	l = lx.expect (T_STRING); ident = l._string;
	l = lx.expect (T_STRING); intitule = l._string;
	effectifs = p_effectifs (lx);
	if ((l = lx.lex (), l._type) == T_SU_ONLY) {
		su_only = true;
	} else
		lx.unlex (l);

	responsable = p_responsable (lx);
	p_description (lx, description, web_ref, parcours);
	g = new Groupe (ident, intitule, effectifs, description, web_ref);
	g -> _su_only = su_only;
	if (responsable != 0)
		g -> _responsable = Enseignant::name2prof (responsable);
	(void) lx.expect (T_LBRA);
	p_s_cours (lx, g);
	(void) lx.expect (T_RBRA);
	return g;
}

/*
 * s_groupe : groupe | groupe s_groupe
 */

static void p_s_groupe (Lexer& lx, Filiere* f)
{
	Lexeme	l;
	Groupe*	g;

	do {
		g = p_groupe (lx);		// le nouveau groupe
		g -> _filiere = f;		// dans sa filire.
		// hriter du responsable de filire s'il n'est pas mentionn.
		if (g -> _responsable == 0)
			g -> _responsable = f -> _responsable;
		f -> _groupes.add (g);	// l'ajouter.
		l = lx.lex ();
		lx.unlex (l);
		} while (l._type == T_GROUPE);
}

/*
 *	filiere : 'Filiere' nom intitule responsable description '{' s_groupe '}'
 */

static Filiere* p_filiere (Lexer& lx)
{
	Lexeme		l;
	Filiere*	f;
	char*		ident;
	char*		intitule;
	char*		description = 0;
	char*		web_ref = 0;
	char*		parcours = 0;
	char*		responsable;

	(void) lx.expect (T_FILIERE);
	l = lx.expect (T_STRING); ident = l._string;
	l = lx.expect (T_STRING); intitule = l._string;
	responsable = p_responsable (lx);
	p_description (lx, description, web_ref, parcours);

	f = new Filiere (ident, intitule, responsable, description, web_ref);
	(void) lx.expect (T_LBRA);
	p_s_groupe (lx, f);
	(void) lx.expect (T_RBRA);
	return f;
}

/*
 * voeux : filiere |filiere voeux
 */

void Filiere::init (const char* name)
{
	Filiere**	p = filieres;
	Lexeme		l;
	Lexer		lx (name, keywords, err, ctype);

	do {
		*p++ = p_filiere (lx);
		*p = 0;		// Pour que la liste soit termine lors des consultations en cours de
					// route pour les cours communs.
		l = lx.lex ();
		lx.unlex (l);
		} while (l._type == T_FILIERE);

	(void) lx.expect (T_EOF);
	if (forward_refs_2 > 0)
		forward_references (forward_refs_2);
	/*
	 *	Traiter les rfrences en avant pour les cours communs.
	 */
	if (forward_refs > 0) {
		for (Filiere** f = filieres; *f != 0; ++f) {
			for (int n = 0; n < (*f) -> _groupes.length (); ++n) {
				Groupe*	g = (Groupe*) (*f) -> _groupes [n];
				for (int i = 0; i < g -> _groupes.length (); ++i) {
					Enseignement* ens = (Enseignement*) g -> _groupes [i];
					if (ens -> _base == (Enseignement*) 1) {
						Enseignement*	ref = name2ens (ens -> _intitule);	// le cours rfrenc.
						if (ref == 0) {
							zlog ("Le cours commun %s rfrenc par %s n'existe pas!\n",
											 ens -> _intitule, ens -> _identification);
							exit (1);
						} else {
							ens -> _base = ref;
							ens -> _intitule = 0;
							if (ens -> _lien)
								ens -> _periode = ref -> _periode;
							else {
								// augmenter le nombre total de groupes.
								ref -> _total_grp += ens -> _ngrp [1] + ens -> _ngrp [2];
								if (ref -> _next == 0)
									ref -> _next = ens;
								else {
									Enseignement*	p;
									for (p = ref -> _next; p -> _next != 0; p = p -> _next)
										;
									p -> _next = ens;
								}
							}
						}
						if (--forward_refs == 0)
							goto fini;	// il n'y a pas de superbreak!
					}
				}
			}
		}
	}
fini:
	zlog ("%d filires\n", (int) (p-filieres));
	zlog ("%d enseignements\n", Enseignement::_nb_enseignements);
	/*
	 *	Initialiser les charges de tutorat.
	 */
	Enseignant*		moniteur;
	Enseignement*	ens;
	Enseignement*	modele;

	for (moniteur = Enseignant::_head;
		 moniteur -> _categorie != MON && moniteur -> _categorie != AMN;
		 moniteur = moniteur -> _suivant)
	{
		if (moniteur == Enseignant::_head -> _precedent)
			return;		// Il n'y a pas de moniteur!
	}

	Groupe*	tut = name2group ((char*)"Tutorat");
	if (tut == 0 || tut -> _groupes.length () == 0)
		return;		// il n'y a pas de charge pour le tutorat des moniteurs.

	modele = (Enseignement*) tut -> _groupes [0];	// le modle de tutorat.
	tut -> _groupes.clean ();						// vider le groupe des tutorats.
	// parcourir tous les enseignants car les moniteurs ne sont pas forcment contigus.
	for (;
		 moniteur != Enseignant::_head;	// on suppose que le premier n'est pas moniteur!
		 moniteur = moniteur -> _suivant)
	{
		char	ident [16];
		char	intitule [64];
		char	prenom [128];
		char	nom [128];

		if (moniteur -> _categorie != MON && moniteur -> _categorie != AMN)
			continue;	// sauter les non-moniteurs.

		if (!moniteur -> _dept -> _a_nous &&
			!moniteur -> _dept -> search ("Polytech") &&
			!moniteur -> _dept -> search ("IFIPS"))
			continue;	// sauter les moniteurs des autres dpartements.

		moniteur -> nom_prenom (nom, prenom);
		(void) sprintf (ident, "Tut-%2.2s_%3.3s", prenom, nom);
		for (unsigned char* p = (unsigned char*)ident+4; *p != 0; ++p)
			*p = Enseignant::iso2ascii [(int)*p];

		(void) sprintf (intitule, "Tutorat de %s", moniteur -> _nom);
		ens = new Enseignement (strdup (ident), 0, strdup (intitule),
								(char*)"[A-E]", 1, modele -> _volume, modele -> _ngrp,
								(char*)"AP3", (char*)"", 0, 0, false, false);	// a n'est plus une chage.
		ens -> _groupe = tut;
		tut -> _groupes.add (ens);
	}
}

/*
 *	Constructeur de la classe.
 */

Filiere::Filiere (char* id, char* nom, char* responsable, char* verb, char* wr)
				: FBase (id, nom, verb, wr)
{
	_responsable = (responsable != 0)
					? Enseignant::name2prof (responsable)
					: 0;
}

/*
 *	Retrouve une filire  partir de son identification.
 */

Filiere* Filiere::name2filiere (char* ident)
{
	for (Filiere**f = filieres; *f != 0; ++f) {
		if (!strcmp (ident, (*f) -> _identification))
			return *f;
	}
	return 0;	// pas trouv.
}

/*
 *	Retrouve un groupe  partir de son identification.
 */

Groupe* Filiere::name2group (char* ident)
{
	for (Filiere** f = filieres; *f != 0; ++f) {
		for (int n = 0; n < (*f) -> _groupes.length (); ++n) {
			Groupe*	g = (Groupe*) (*f) -> _groupes [n];
			if (!strcmp (ident, g -> _identification))
				return g;
		}
	}
	return 0;	// pas trouv.
}

/*
 *	Retrouve un enseignement  partir de son identification.
 */

Enseignement* Filiere::name2ens (char* ident)
{
	for (Filiere** f = filieres; *f != 0; ++f) {
		for (int n = 0; n < (*f) -> _groupes.length (); ++n) {
			Groupe*	g = (Groupe*) (*f) -> _groupes [n];
			for (int i = 0; i < g -> _groupes.length (); ++i) {
				Enseignement* e = (Enseignement*) g -> _groupes [i];
				if (!strcmp (ident, e -> _identification))
					return e;
			}
		}
	}
	return 0;	// pas trouv.
}

/*
 *	Retrouve un enseignement  partir de son identification.
 */

Enseignement* Enseignement::name2ens (char* ident)
{
	for (int n = 0; n < Enseignement::_nb_enseignements; ++n) {
		Enseignement* e = Enseignement::_tous_les_enseignements [n];
		if (!strcmp (ident, e -> _identification))
			return e;
	}
	return 0;	// pas trouv.
}
