/*****************************************************************

  Copyright (c) 1999 - 2003 by Patrick Amar - pa@lri.fr

    http://www.lri.fr/~pa

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

#include <string.h>
#include <stdlib.h>
#include "Lexer.h"

/*
 *	Analyseur lexical gnrique.
 */

Lexer::Lexer (const char* filename, const char** keywords, const char** err, Special* spec)
: _ringur (T_VOID, (char*)"")
{
	_fp = fopen (filename, "r");
	if (_fp == 0) {
		zlog ("Fatal error: cannot open '%s' giving up.\n", filename);
		exit (1);
	}
	_splicing = spec;
	_filename = filename;
	_line_number = 1;
	_keywords = keywords;
	_err = err;
}

/*
 *	est-ce que 'c' est un sparateur?
 */

int Lexer::is_splicing (int c)
{
	for (Special* p = _splicing-1; (++p) -> character != 0;)
		if (c == p -> character)
			return (int) p -> type;
	return 0;
}

/*
 *	L'analyseur par lui mme.
 */

Lexeme Lexer::lex (void)
{
	static char	tc [4];
	int			type;
	int			c;
	int			n;
	char		buffer [10240];
	char*		p;
	char*		q;

	// en avait-on lu un de trop la dernire fois ?
	if (_ringur._type != T_VOID) {
		Lexeme result (_ringur._type, _ringur._string);

		_ringur._type = T_VOID;
		_ringur._string = (char*)"";
		return result;
	}
	for (;;) switch (c = getc (_fp)) {
		case EOF:
			return Lexeme (T_EOF, (char*)"EOF");

		case '\"':
			for (p = buffer; (c = getc (_fp)) != '\"'; *p++ = c)
				switch (c) {
					case '\n':
						++ _line_number;
						c = ' ';	// transformer les fin de lignes en espaces.
						continue;

					case '\r':		// sauter le \r (MSDOS)
					case '\\':		// caractre spcial.
						c = getc (_fp);
						continue;
				}
	
			*p++ = 0;
			q = (char*) malloc (p-buffer+1);
			strcpy (q, buffer);
			return Lexeme (T_STRING, q);

		case '#':
			while ((c = getc (_fp)) != '\n')
				;
			++ _line_number;
			continue;	// commentaire # blabla \n

		case '\n':	// ignorer la fin de ligne, mais les compter!
			++ _line_number;
			continue;

		case '0': case '1': case '2': case '3': case '4':
		case '5': case '6': case '7': case '8': case '9':
			n = c - '0';
			while ((c = getc (_fp)) >= '0' && c <= '9')
				n = n*10 + (c-'0');
			ungetc (c, _fp);
			return Lexeme (T_NUMBER, (char*) (long) n);

		default:
			if (0 <= c && c <= ' ')
				continue;	// sauter les espaces et les caractre non imprimables

			// caractre sparateur ?
			if (0 != (type = is_splicing (c))) {
				tc [0] = c;
				return Lexeme (type, tc);
			}
			// non donc c'est un identificateur.
			for (p = buffer;
					c != ' ' &&
					c != '\t' &&
					c != '\"' &&
					c != '\n' &&
					! is_splicing (c); c = getc (_fp))
				*p++ = c;
			*p = 0;
			ungetc (c, _fp);	// ringurgiter le sparateur.
			for (int n = 0; _keywords [n] != 0; ++n)
				if (!strcmp (buffer, _keywords [n]))
					return Lexeme (n + T_VOID+1, (char*)_keywords [n]);
			q = (char*) malloc (p-buffer+1);
			strcpy (q, buffer);
			return Lexeme (T_STRING, q);
	}
}

/*
 *	Erreur de syntaxe.
 */

Lexeme Lexer::expect (int t, const char* msg)
{
	Lexeme	l = lex ();

	if (msg != 0 || l._type != t) {
		zlog ("%s:%d Expected %s, got %s: ", _filename, _line_number, msg ? msg :_err [t], _err [l._type]);
		if (l._type == T_NUMBER)
			zlog ("'%d'\n", (int) (long) l._string);
		else
			zlog ("'%s'\n", l._string);
		exit (1);
	}
	return l;
}
