/*****************************************************************

  Copyright (c) 1999 - 2003 by Patrick Amar - pa@lri.fr

    http://www.lri.fr/~pa

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

#include "Lexer.h"
#include "Security.h"

Security*	Security::_access_list [MAX_IP];
int			Security::_host_number = 0;

/*
 *	Nouvelle description d'hte.
 */

Security::Security (bool banned, int ip [NN])
{
	_banned = banned;
	for (int i = 0; i < NN; ++i) {
		_ip_number [i] = ip [i];
		_star [i] = ip [i] < 0;
	}
	_access_list [_host_number++] = this;	// le mettre dans la liste.
}

/*
 *	Indique si un hte est indsirable.
 */

bool Security::banned (unsigned char* ip)
{
	bool	match;
	int		i;
	int		n;

	for (i = 0; i < _host_number; ++i) {
		for (n = 0; n < NN; ++n) {
			match = _access_list [i] -> _star [n] ||
					_access_list [i] -> _ip_number [n] == ip [n];
			if (!match)
				break;
		}
		if (match)
			return _access_list [i] -> _banned;
	}
	return false;	// autoris par dfaut.
}

/*
 *	Initialise la liste des htes.
 */

#define T_ACCEPT		T_VOID+1
#define T_REJECT		T_VOID+2
#define T_STAR			T_VOID+3
#define T_DOT			T_VOID+4

void Security::init (const char* filename)
{
	static Special	ctype [] = {
		{ '*',	T_STAR },
		{ '.',	T_DOT },
		{ 0, 0 }
	};
	static const char*	keywords [] = { "Accept", "Reject", 0 };
	static const char*	err [] = {	"a character string", "the end of file", "a number", "????",
								"Accept", "Reject",
							 };
	Lexer	lx (filename, keywords, err, ctype);
	Lexeme	l;
	int		ip [NN];
	bool	banned;
/*
 *	Format:
 *			Accept 192.*.*.*
 *			Accept 129.175.*.*
 *			Reject 203.*.*.*
 */
	for (;;) {
		l = lx.lex ();
		switch (l._type) {
			case T_EOF:
				return;
			case T_ACCEPT:
				banned = false;
				break;
			case T_REJECT:
				banned = true;
				break;
			default:
				lx.unlex (l);
				(void) lx.expect (T_VOID, "'Accept' or 'Reject'");
				return;		// not reached.
		}
		for (int i = 0;;) {
			l = lx.lex ();
			switch (l._type) {
				case T_NUMBER:
					ip [i] = (int) (long) l._string;
					break;
				case T_STAR:
					ip [i] = -1;
					break;
				default:
					lx.unlex (l);
					(void) lx.expect (T_VOID, "'*' or a number");
					return;	// not reached.
			}
			if (++i == NN)
				break;
			(void) lx.expect (T_DOT);
		}
		(void) new Security (banned, ip);
	}
}
