/*****************************************************************

  Copyright (c) 1999 - 2016 by Patrick Amar - pa@lri.fr

    http://www.lri.fr/~pa

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "Enseignant.h"

extern char*	_outp_;

const char* _genre [] = { "Cours", "TD", "TP", "MP", "Demi-TP", "Soutien-TD", "Soutien Session-2", "Cours/TD" };
extern char*	_outp_;

static const char*	fmt_ab_1 [] = { "", "<i>", "", "<i>", "", "<i>", "", "<i>", "", "<i>" };
static const char*	fmt_ab_2 [] = { "", "</i>", "", "</i>", "", "</i>", "", "</i>", "", "</i>" };
static const char*	fmt_hc [] = { "", "", "[HC]", "[HC]", "[HC-EXT]", "[HC-EXT]", "[ST-EXT]", "[ST-EXT]", "[GRAT]", "[GRAT]" };

#ifdef AVANT_2010
float	Service::taux [N_ENS_TYPES+1] = { 1.5, 1., 3./4., 1., 1., 1., 1., 1.25 };
#else
// nouveaux taux pour 2010:  1h TP = 1h TD.
float	Service::taux [N_ENS_TYPES+1] = { 1.5, 1., 1., 1., 1., 1., 1., 1.25 };
#endif
char	Service::_sep [4] = { '.', 0, 0, 0 };

/*
 *	Convertit une priode en date (nouveau modle).
 */

int Per2Time (char* periode, time_t& debut, time_t& fin)
{
#define	JANVIER		1
#define FEVRIER		2
#define	DECEMBRE	12
#define	SEPTEMBRE	9
#define	JUILLET		7
	static struct {
		time_t		mois;
		const char*	nom;
	} per [DECEMBRE] = {
		{ 0,	"jan" },
		{ 0,	"fev" },
		{ 0,	"mar" },
		{ 0,	"avr" },
		{ 0,	"mai" },
		{ 0,	"jui" },
		{ 0,	"jul" },
		{ 0,	"aou" },
		{ 0,	"sep" },
		{ 0,	"oct" },
		{ 0,	"nov" },
		{ 0,	"dec" },
	};

	// Initialiser les priodes avec l'anne universitaire (mentionne dans  profs.def .
	if (per [0].mois == 0) {
		for (int i = JANVIER; i < SEPTEMBRE; ++i)
			per [i-1].mois = MakeTime (1, i, Taux::_year2);
		for (int i = SEPTEMBRE; i <= DECEMBRE; ++i)
			per [i-1].mois = MakeTime (1, i, Taux::_year1);
	}
	// chercher la priode considre.
	char	md [32];
	char	mf [32];
	int		j, k;
	
	debut = per [SEPTEMBRE-1].mois;
	fin = per [JUILLET-1].mois;
	// syntaxe: [ mois_deb - mois_fin ]
	if (periode == 0 || periode [0] != '[' || periode [1] == ']')
		return 0;
	// le mois de dbut.
	for (j = 0, k = 1; (md [j] = periode [k]) != '-'; ++j, ++k)
		md [j] |= 040;	// mettre en minuscule.
	md [j] = 0;
	// le mois de fin.
	for (j = 0, ++k; (mf [j] = periode [k]) != ']'; ++j, ++k)
		mf [j] |= 040;	// mettre en minuscule.
	mf [j] = 0;

	// chercher dans la table.
	debut = 0;
	fin = 0;
	for (unsigned int i = 0; i < DECEMBRE; ++i) {
		if (debut == 0 && 0 == strncmp (per [i].nom, md, 3)) {
			debut = per [i].mois;
			if (fin != 0)
				break;
		}
		if (fin == 0 && 0 == strncmp (per [i].nom, mf, 3)) {
			fin = per [i].mois;
			if (debut != 0)
				break;
		}
	}
	if (debut <= per [FEVRIER-1].mois && fin <= per [FEVRIER-1].mois)
		return 0;	// premier semestre.
	if (debut >= per [JANVIER-1].mois && fin <= per [JUILLET-1].mois)
		return 1;	// deuxime semestre.
	// sinon.
	return 0;
}

int Per2Semestre (char* periode) {
	time_t	debut;
	time_t	fin;

	return Per2Time (periode, debut, fin);
}

/*
 *	Convertit une date UNIX en date.
 */

char* AscTime (time_t* d) {
	struct tm*	t;
	static char	date [32];

	t = gmtime (d);
	sprintf (date, "%02d/%02d/%d", t -> tm_mday, t -> tm_mon+1, t -> tm_year+1900);
	return date;
}

/*
 *	Convertit une date en date UNIX.
 */

time_t MakeTime (int j, int m, int a)
{
	struct tm	lt;

	lt.tm_sec = 0;
	lt.tm_min = 0;
	lt.tm_hour = 10;
	lt.tm_mday = j;
	lt.tm_mon = m-1;
	lt.tm_year = a-1900;
	lt.tm_isdst = -1;
	return mktime (&lt);
}

/*
 *	Convertit une date chane en date UNIX.
 */

time_t String2Time (char* date)
{
	char*	p;
	int		n = 0;
	int		t [4];

	// couper la date en morceaux.
	p = date;
	for (int i = 0; date [i] != 0; ++i)
		if (date [i] == '/') {
			date [i] = 0;
			t [n++] = atoi (p);
			p = date+i+1;
		}
	t [n] = atoi (p);	// ne pas oublier le dernier.
	// vrifier un peu.
	if (1 <= t[0] && t[0] <= 31 &&
		1 <= t[1] && t[1] <= 12 &&
		1999 <= t[2] && t[2] <= 2037)
	{
		return MakeTime (t[0], t[1], t[2]);
	}
	return -1;	// date bizarre!
}

/*
 *	La classe Mandatement.
 */

float	Mandatement::taux [MAX_MAND] = { 244.18, 246.13, 0. };
const char*	Mandatement::_nom_div [((int)FP)+1] = {
	"Orientation", "Formation", "Recherche", "STAPS", "FP"
};

float Mandatement::affiche (Enseignant* prof)
{
	static char fmt_recap [] =
		"<tr bgcolor=#004070>"
			"<td><a href=\"service_%s.cmd\">%s</a></td>"	// nom.
			"<td align=center>%s</td>"			// Statut.
			"<td align=right><font face=verdana>%s&nbsp;</font></td>"	// Service statutaire pendant la priode.
			"<td align=center>du %s au %s</td>"	// Priode.
			"<td align=right><font face=verdana>%g&nbsp;</font></td>"	// Nombre d'heures complmentaires.
			"<td align=right><font face=verdana>%g&nbsp;</font></td>"	// Taux.
			"<td align=right><font face=verdana>%.2f&nbsp;</font></td>"	// Montant.
		"</tr>\n";

	char	deb [64];
	char	fin [64];
	char*	serv_stat;
	float	volume = MAND_EQTD (this);

	strcpy (deb, AscTime (&_deb_periode));
	strcpy (fin, AscTime (&_fin_periode));
	serv_stat = (prof -> _employeur -> _type == UNIVERSITE && prof -> _dept -> _a_nous)
				? (char*) "64"	//  prciser ventuellement.
				: (char*) "";
	_outp_ += sprintf (_outp_, fmt_recap,
						Enseignant::nospaces (prof -> _nom), prof -> _nom,
						prof -> cat2str (false),
						serv_stat,
						deb, fin,
						volume,
						_taux,
						_taux * volume);
	return _taux * volume;
}

float Mandatement::imprime (Enseignant* prof)
{
	char	deb [64];
	char	fin [64];
	int		serv_stat;
	float	volume = MAND_EQTD (this);

	strcpy (deb, AscTime (&_deb_periode));
	strcpy (fin, AscTime (&_fin_periode));
	serv_stat = (prof -> _employeur -> _type == UNIVERSITE && prof -> _dept -> _a_nous)
				? 64	//  prciser ventuellement.
				: 0;
	_outp_ += sprintf (_outp_, "(%s) (%s) (%d) (TD) (%s - %s) (%g) (%.2f) (%.2f) ligne\n",
						prof -> _nom,
						prof -> cat2str (false),
						serv_stat,
						deb, fin,
						volume,
						_taux,
						_taux * volume);
	return _taux * volume;
}

void Mandatement::affiche_liste (int n)
{
	static char	fmt_mand_list [] =
	"<tr bgcolor=#400070>"
		"<td><font face=verdana><input name=\"M%d\" type=radio onClick='submit()'>"
			"&nbsp;&nbsp;%s</font></td>"	// date.
		"<td align=center><font face=verdana>%s</font></td>"	// type de formation.
		"<td align=center><font face=verdana>%s</font></td>"	// qui paie.
		"<td align=right><font face=verdana>%g&nbsp;</font></td>"	// nb heures.
		"<td align=right><font face=verdana>%.2f&nbsp;</font></td>"	// taux
		"<td align=right><font face=verdana>%.2f&nbsp;</font></td>"	// total
	"</tr>\n";
	float	volume = MAND_EQTD (this);

	_outp_ += sprintf (_outp_, fmt_mand_list,
						n, AscTime (&_date), _fi ? "FI" : "FC",
						Mandatement::_nom_div [_division],
						volume, _taux, volume * _taux);
}

/*
 *	Initialise un service vide.
 */

Service::Service (Enseignement* ens)
{
	_ens = ens;
	_prime = false;	// les charges sont "payes" en heures de cours pas dfaut.
	_date = 0;	// 1/1/70, the epoch!
	for (int i = 0; i < N_ENS_TYPES; ++i) {
		_prioritaire [i] = false;
		for (int th = 0; th < N_HRS_TYPES; ++th)
			_heures [th][i] = 0.0;
	}
}

/*
 *	Indique si un service est vide.
 */

bool Service::vide ()
{
	for (int i = 0; i < N_ENS_TYPES; ++i) {
		for (int th = 0; th < N_HRS_TYPES; ++th)
			if (_heures [th][i] != 0.0)
				return false;
	}
	return true;
}


/*
 *	Retourne le volume d'un service en heures q TD.
 *	Corrig au besoin pour les cours communs.
 */

float Service::eq_td (int th, bool commun) {
	float	vol = 0;
	float	coeff;
	int		t;

	for (int i = 0; i < N_ENS_TYPES; ++i) {
		// Cours/td -> 5 alors que le reste est direct.
		t = (i==COURS && _ens -> _cours_td) ? COURS_TD : i;
		// Ajuster les cours communs.
		if (commun && t == COURS) {
			Enseignement*	ref = _ens -> _base;
			if (ref != 0) {
				// il s'agit de TD et/ou TP d'un enseignement commun.
				// calculer le coefficient d'imputation  la filire.
				coeff = ((float) _ens -> _total_grp) / ((float) ref -> _total_grp);
				// retrouver le nombre d'heures effectues en cours dans le cours de base.
				// et lui affecter le taux et le coefficient.
				vol += ref -> volume_enseigne (th, COURS) * taux [COURS] * coeff;
			} else {
				// il s'agit du cours d'un enseignement commun de base.
				if (_ens -> _total_grp == 0)
					coeff = 1.;
				else
					coeff = ((float) _ens -> _ngrp [TD] + _ens -> _ngrp [TP])
						  / ((float) _ens -> _total_grp);
				vol += _heures [th][COURS] * taux [COURS] * coeff;
			}
		} else
			vol += _heures [th][i] * taux [t];
	}
	return vol;
}

/*
 *	Affiche par type d'enseignement au format liste
 */

void Service::affiche_list_aux (char* nom_ens, int th, int i)
{
	static char	fmt_prof [] = "<a href=\"service_%s.cmd\">%s%s%s%s</a>(%g%s)%s &nbsp;";
	float	evol = _ens -> _volume [i];
	float	vol = _heures [th][i];
	float	reste;
	int		ng;

	// faire apparatre les demandeurs de primes pour les charges.
	if (_ens -> _charge_np && th == HEURE_STAT_A && i == TD && vol == 0) {
		// vrifier que c'est une prime et pas une charge en HC ou autre...
		for (ng = HEURE_STAT_A; ng <= HEURE_GRAT; ng += 2)
			if (_heures [ng][i] > 0.)
				return;

		_outp_ += sprintf (_outp_, fmt_prof, Enseignant::nospaces (nom_ens),
							fmt_ab_1 [th], nom_ens,
							_prioritaire [i] ? "*" : "",
							fmt_ab_2 [th], 1., "", fmt_hc [th]);
		return;
	}
	if (vol == 0) {
		return;		// rien  afficher.
	}

	// si un groupe reprsente 1h de td on affiche en heures plutot qu'en groupes.
	ng = (i == TD && evol == 1.)
		? 0
		: (int) (vol / evol);	// nombre de groupes complets.
	reste = vol - (ng * evol);	// reliquat d'heures.

	if (ng != 0.)	// FIXED?? > 0.
		_outp_ += sprintf (_outp_, fmt_prof, Enseignant::nospaces (nom_ens),
							fmt_ab_1 [th], nom_ens,
							_prioritaire [i] ? "*" : "",
							fmt_ab_2 [th], (float) ng, "", fmt_hc [th]);
	if (reste != 0.)	// il peut y avoir des services ngatifs
		_outp_ += sprintf (_outp_, fmt_prof, Enseignant::nospaces (nom_ens),
							fmt_ab_1 [th], nom_ens,
							_prioritaire [i] ? "*" : "",
							fmt_ab_2 [th], reste, "h", fmt_hc [th]);
}

/*
 *	Affiche un service de type 'i' au format: Nom (n g)
 */

void Service::affiche_liste (char* nom_ens, int i)
{
	for (int th = 0; th < N_HRS_TYPES; ++th)
		affiche_list_aux (nom_ens, th, i);
}

/*
 *	Retourne le contenu d'un service pour l'tat statutaire
 */

float Service::affiche_stat (int th, char** filiere, char** intitule, float volume [])
{
	// l'intitul de l'enseignement.
	*intitule  = (_ens -> _base == 0)
					? _ens -> _intitule
					: _ens -> _base -> _intitule;
	// le nom de la filire.
	if (_ens -> _charge_np || _ens -> _charge)
		*filiere = (char*)"Coordination pdagogique";
	else
		*filiere = _ens -> _groupe -> _filiere -> _nom;
	// initialiser les volumes.
	for (int i = 0; i < N_ENS_TYPES; ++i)
		volume [i] = 0.;
	// hardi petit!
	for (;;) {
		for (int i = 0; i < N_ENS_TYPES; ++i) {
			if (_heures [th][i] == 0.)
				continue;			// rien l dedans!

			switch (i) {
				// hack local: ajouter les MP aux TP (pour les faire disparatre).
				case MP:
					volume [TD] += _heures [th][MP];
					continue;
				case COURS:
					if (_ens -> _cours_td) {
						volume [TD] += _heures [th][COURS] / 2.;
						volume [COURS] += _heures [th][COURS] / 2.;
						continue;
					}
					// fall thru.
				case DTD:
				case STD:
				case SSS:
				case TD:
				case TP:
					volume [i] += _heures [th][i];
					continue;
			}
		}
		// sordide hack pour ajouter les heures extrieures statutaires aux heures statutaires.
		if (th != HEURE_STAT_A)
			break;
		th = HEURE_STAT_EXT;
	}
	return	volume [COURS] * taux [COURS] +
			volume [TD] * taux [TD] +
			volume [TP] * taux [TP] +
			volume [DTD] * taux [DTD] +
			volume [STD] * taux [STD] +
			volume [SSS] * taux [SSS];
}

/*
 *	Retourne le contenu d'un service pour le fichier 'qui_fait_quoi'
 */

float Service::affiche_service (int th, char** filiere, char** intitule, float volume [])
{
	// l'intitul de l'enseignement.
	*intitule  = (_ens -> _base == 0)
					? _ens -> _intitule
					: _ens -> _base -> _intitule;
	// le nom de la filire.
	if (_ens -> _charge_np || _ens -> _charge)
		*filiere = (char*)"Coordination pdagogique";
	else
		*filiere = _ens -> _groupe -> _filiere -> _nom;
	// initialiser les volumes.
	for (int i = 0; i < N_ENS_TYPES; ++i)
		volume [i] = 0.;
	// hardi petit!
	for (int i = 0; i < N_ENS_TYPES; ++i) {
		if (_heures [th][i] == 0.)
			continue;			// rien l dedans!

		switch (i) {
			// hack local: ajouter les MP aux TD (pour les faire disparatre).
			case MP:
				volume [TD] += _heures [th][MP];
				continue;
			case COURS:
				if (_ens -> _cours_td) {
					volume [TD] += _heures [th][COURS] / 2.;
					volume [COURS] += _heures [th][COURS] / 2.;
					continue;
				}
				// fall thru.
			case DTD:
			case STD:
			case SSS:
			case TD:
			case TP:
				volume [i] += _heures [th][i];
				continue;
		}
	}
	return	volume [COURS] * taux [COURS] +
			volume [TD] * taux [TD] +
			volume [TP] * taux [TP] +
			volume [DTD] * taux [DTD] +
			volume [STD] * taux [STD] +
			volume [SSS] * taux [SSS];
}

/*
 *	Affiche par type d'enseignement.
 */

static char fmt_checkbox_charge [] =
		"<td><input type=hidden value=\"%s\" name=E>"
		"<input type=checkbox value=\"%d\" name=T>";

static char fmt_checkbox [] =
		"<td><input type=checkbox value=\"%d\" name=T>";

static char fmt_page_enseignement [] =
		"<input type=hidden value=\"%g\" name=V%c></td>"
		"<td align=right>%.1f</td>"
		"<td width=20></td><td>%s%g %s de %s%s %s%s %s</td>"
	"</tr>";

static char fmt_page_charge_prime [] =
			"<input type=hidden value=\"%g\" name=V0></td>"	// heures statutaires
		"<td align=right><b>Demande<br>de prime</b></td>"
		"<td></td><td><b><font color=#e0ffe0>%s</font>"
			"&nbsp;&nbsp;&nbsp;[%02d/%02d/%d %02d:%02d]"
		"</b></td>"
	"</tr>\n";

static char fmt_page_ch [] =
		"<input type=hidden value=\"%g\" name=V%c></td>"
		"<td align=right>%.1f</td>"
		"<td width=20></td><td>%s %s</td>"
	"</tr>";

static char fmt_page_charge_fin [] =
			"<input type=hidden value=\"%g\" name=V2></td>"	// heures statutaires
		"<td align=right>%.1f</td>"
		"<td></td><td><b><font color=#e0ffe0>%s"
#ifdef CHARGES_EN_HC
			" %s"
#endif
			"</font>&nbsp;&nbsp;[%02d/%02d/%d %02d:%02d]"
		"</b>%s</td>"
	"</tr>\n";

void Service::affiche_aux (Enseignant* prof, int th, int i, Editable edit,
							bool deja_mandate, bool is_mandator)
{
	float	reste;
	float	evol = _ens -> _volume [i];
	float	vol = _heures [th][i];
	int		ng;
	int		t;

	if (vol == 0)
		return;		// rien  afficher.

	// Cours/td -> N_ENS_TYPES-1, alors que le reste est direct.
	t = (i==COURS && _ens -> _cours_td) ? COURS_TD : i;
	// si un groupe reprsente 1h de td on affiche en heures plutot qu'en groupes.
	ng = (i == TD && evol == 1.)
		? 0
		: (int) (vol / evol);	// nombre de groupes complets.
	reste = vol - (ng * evol);	// reliquat d'heures.

#if FICHE_LATEX
	static char	fmt_fiche [] = "<a href=\"engagement_%d_%s.lxf\"><img src=euro-sign.png border=0><a>";

	// retrouver le numro du service.
	int		n = 0;
	char	tmp [256] = { 0 };

	for (;; ++n) {
		if (n == prof -> _service.length ()) {
			zlog ("Service inconnu pour '%s'...\n", prof -> _nom);
			return;
		}
		if (this == (Service*) prof -> _service [n])
			break;
	}
	// met-on la boule marron du mandatement ou le signe de l'euro de la demande de HCC?
	if (th == HEURE_COMP_A) {
		if (edit == EDIT_SERVICE && deja_mandate)
			sprintf (tmp, "<img src=b_mand.gif border=0 align=center>");
		else if (edit == NON_EDITABLE && !deja_mandate)
			sprintf (tmp, fmt_fiche, n, prof -> nospaces ());
	}
#endif
	if (ng != 0) {
		_outp_ += sprintf (_outp_, "<tr>\n");
		switch (edit) {
			case NON_EDITABLE:
				_outp_ += sprintf (_outp_, "<td>");
#if FICHE_LATEX
				if (th == HEURE_COMP_A) {
					if (deja_mandate || is_mandator) {
						_outp_ += sprintf (_outp_, "<img src=b_mand.gif border=0>");
					} else {
						_outp_ += sprintf (_outp_, "%s", tmp);
					}
				}
#else
				if (th == HEURE_COMP_A && deja_mandate)
					_outp_ += sprintf (_outp_, "<img src=b_mand.gif border=0>");
#endif
				break;
			case EDIT_MANDATEMENTS:
				if (th != HEURE_COMP_A) {
					_outp_ += sprintf (_outp_, "<td>");
					break;
				}
				// fall thru.
			case EDIT_SERVICE:
				if (_ens -> _charge_np && !_prime)
					// mettre le 'name=E' avant le 'name=T'...
					_outp_ += sprintf (_outp_, fmt_checkbox_charge,
												_ens -> _identification, i);
				else
					_outp_ += sprintf (_outp_, fmt_checkbox, i);
				break;
		}
		// ce service est une charge...
		if (_ens -> _charge_np) {
			struct tm*	lt = localtime (&_date);	// date de dernire modif de ce service.

			if (_prime)
				_outp_ += sprintf (_outp_, fmt_page_charge_prime,
							ng *evol,
							_ens -> _intitule,
							lt -> tm_mday, lt -> tm_mon+1,
							1900 + lt -> tm_year, lt -> tm_hour, lt -> tm_min);
			else {
#if FICHE_LATEX
				_outp_ += sprintf (_outp_, fmt_page_charge_fin,
							ng * evol,
							ng * evol,
							_ens -> _intitule,
							fmt_hc [th],
							lt -> tm_mday, lt -> tm_mon+1,
							1900 + lt -> tm_year, lt -> tm_hour, lt -> tm_min, tmp);
#else
				_outp_ += sprintf (_outp_, fmt_page_charge_fin,
							ng * evol,
							ng * evol,
							_ens -> _intitule,
							fmt_hc [th],
							lt -> tm_mday, lt -> tm_mon+1,
							1900 + lt -> tm_year, lt -> tm_hour, lt -> tm_min,
							(edit == EDIT_SERVICE && th == HEURE_COMP_A && deja_mandate)
								? "<img src=b_mand.gif border=0 align=center>"
								: "");
#endif
			}
			return;	// les charges sont compltes.
		}
		// ce service est un cours ou une charge partageable.
		if (_ens -> _charge) {
			_outp_ += sprintf (_outp_, fmt_page_ch, ng * evol, '0'+th,
							ng * evol * taux [t],
							fmt_hc [th][0] != 0 ? fmt_hc [th] : "[STAT]",
							(edit == EDIT_SERVICE && th == HEURE_COMP_A && deja_mandate)
								? "<img src=b_mand.gif border=0 align=center>"
								: "");
		}
		else
			_outp_ += sprintf (_outp_, fmt_page_enseignement, ng * evol, '0'+th,
							ng * evol * taux [t], fmt_ab_1 [th],
							(float) ng, "g", _genre [t],
							_prioritaire [i] ? "*" : "",
							fmt_hc [th], fmt_ab_2 [th],
							(edit == EDIT_SERVICE && th == HEURE_COMP_A && deja_mandate)
								? "<img src=b_mand.gif border=0 align=center>"
								: "");
	}
	// il peut y avoir des services ngatifs !
	if (reste != 0.) {
		_outp_ += sprintf (_outp_, "<tr>\n");
		switch (edit) {
			case NON_EDITABLE:
				_outp_ += sprintf (_outp_, "<td>");
#if FICHE_LATEX
				if (th == HEURE_COMP_A) {
					if (deja_mandate) {
						_outp_ += sprintf (_outp_, "<img src=b_mand.gif border=0>");
					} else {
						_outp_ += sprintf (_outp_, "%s", tmp);
					}
				}
#else
				if (th == HEURE_COMP_A && deja_mandate)
					_outp_ += sprintf (_outp_, "<img src=b_mand.gif border=0>");
#endif
				break;
			case EDIT_MANDATEMENTS:
				if (th != HEURE_COMP_A) {
					_outp_ += sprintf (_outp_, "<td>");
					break;
				}
				// fall thru.
			case EDIT_SERVICE:
				_outp_ += sprintf (_outp_, fmt_checkbox, i);
				break;
		}
		if (_ens -> _charge) {
			_outp_ += sprintf (_outp_, fmt_page_ch, reste, '0'+th,
							reste * taux [t],
							fmt_hc [th][0] != 0 ? fmt_hc [th] : "[STAT]",
							(edit == EDIT_SERVICE && th == HEURE_COMP_A && deja_mandate)
								? "<img src=b_mand.gif border=0 align=center>"
								: "");
		}
		else
			_outp_ += sprintf (_outp_, fmt_page_enseignement, reste, '0'+th,
							reste * taux [t], fmt_ab_1 [th], reste, "h",
							_genre [t],
							_prioritaire [i] ? "*" : "",
							fmt_hc [th], fmt_ab_2 [th],
							(edit == EDIT_SERVICE && th == HEURE_COMP_A && deja_mandate)
								? "<img src=b_mand.gif border=0 align=center>"
								: "");
	}
}

/*
 *	Affiche un service.
 */

static char fmt_page_charge_part [] =
	"<tr><td></td><td></td><td></td><td><b><font color=#e0ffe0>%s</font>"
	"&nbsp;&nbsp;&nbsp;<font size=-1 color=orange>(%02d/%02d/%d %02d:%02d)</font>"
	"</b></td><input type=hidden value=\"%s\" name=E></tr>\n";

static char fmt_page_entete [] =
	"<tr><td></td><td></td><td></td><td><b><a href=\"cours_%s%s%s.cmd\">%s - %s</a>"
	"&nbsp;&nbsp;%s"
	"&nbsp;&nbsp;&nbsp;<font size=-1 color=orange>(%02d/%02d/%d %02d:%02d)</font>"
	"</b></td><input type=hidden value=\"%s\" name=E></tr>\n";

static char fmt_page_entete_tut [] =
	"<tr><td></td><td></td><td></td><td><b><a href=\"cours_%s%s%s.cmd\">%s</a>"
	"&nbsp;&nbsp;&nbsp;<font size=-1 color=orange>(%02d/%02d/%d %02d:%02d)</font>"
	"</b></td><input type=hidden value=\"%s\" name=E></tr>\n";

void Service::affiche (Enseignant* prof, Editable edit, bool is_mandator)
{
	struct tm*	lt = localtime (&_date);
	char*		nom_ens;

	nom_ens = Enseignant::nospaces (prof -> _nom);	// remplacer les ' ' par des '_'
	if (! _ens -> _charge_np) {
		if (_ens -> _charge) {
			_outp_ += sprintf (_outp_, fmt_page_charge_part,
							_ens -> get_intitule (false),
							lt -> tm_mday, lt -> tm_mon+1,
							1900 + lt -> tm_year, lt -> tm_hour, lt -> tm_min,
							_ens -> _identification);
		} else

		// ne pas afficher l'identification pour les tutorats.
		if (0 == strncmp ("Tut-", _ens -> _identification, 4))
			_outp_ += sprintf (_outp_, fmt_page_entete_tut,
							_ens -> _identification,
							edit != NON_EDITABLE ? _sep : "",
							edit != NON_EDITABLE ? nom_ens : "",
							_ens -> get_intitule (false),
							lt -> tm_mday, lt -> tm_mon+1,
							1900 + lt -> tm_year, lt -> tm_hour, lt -> tm_min,
							_ens -> _identification);
		else
			_outp_ += sprintf (_outp_, fmt_page_entete,
							_ens -> _identification,
							edit != NON_EDITABLE ? _sep : "",
							edit != NON_EDITABLE ? nom_ens : "",
							_ens -> _groupe -> _identification,
							_ens -> get_intitule (false),
							_ens -> _periode,
							lt -> tm_mday, lt -> tm_mon+1,
							1900 + lt -> tm_year, lt -> tm_hour, lt -> tm_min,
							_ens -> _identification);
	}
	for (int th = 0; th < N_HRS_TYPES; ++th) {
		// calculer les HC dj mandates pour
		// ne mettre le bouton de mandatement que pour les HC qui restent  mandater.
		if (th == HEURE_COMP_A) {
			float	vol_mandate [N_ENS_TYPES];
			// initialiser des volumes vides.
			for (int i = COURS; i < N_ENS_TYPES; ++i)
				vol_mandate [i] = 0.;
			// regarder si un mandatement existe pour ce service.
			for (int n = 0; n < prof -> _mandatement.length (); ++n) {
				Mandatement*	m = (Mandatement*) prof -> _mandatement [n];

				if (m -> _s == this) {
					for (int i = COURS; i < N_ENS_TYPES; ++i)
						vol_mandate [i] += m -> _vol [i];
					// attention: cours/td <==> 1/2 cours + 1/2 td.
					if (m -> _s -> _ens -> _cours_td) {
						vol_mandate [TD] -= vol_mandate [COURS];
						vol_mandate [COURS] *= 2;
					}
				}
			}
			// les cours/td/tp.
			for (int i = COURS; i < N_ENS_TYPES; ++i) {
				if (vol_mandate [i] >= _heures [HEURE_COMP_A][i] && _heures [HEURE_COMP_A][i] > 0)
				{
					// faire afficher une boule bleue quand le cours est mandat.
					if (edit == EDIT_SERVICE)
						affiche_aux (prof, HEURE_COMP_A, i, EDIT_SERVICE, true, is_mandator);
					else
						affiche_aux (prof, HEURE_COMP_A, i, NON_EDITABLE,
									edit != EDIT_MANDATEMENTS, is_mandator);
				} else
					affiche_aux (prof, HEURE_COMP_A, i, edit, false, is_mandator);
			}
		} else
			for (int i = 0; i < N_ENS_TYPES; ++i)
				affiche_aux (prof, th, i, edit, false, is_mandator);
	}
}
